/**
	$Id: JobRunner.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent;

import java.util.Arrays;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemModifiedException;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.JobAgentSettings;
import net.sf.basedb.core.SessionControl;

/**
	A runnable class that makes it possible to execute each job in it's own thread.
	When the agent has found a job to execute it creates one instance of this class
	and creates a new thread for it to run in. This class checks that there is a free 
	slot available and then uses the configured {@link JobExecutor} for the actual
	execution of the job.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/
public class JobRunner
	implements Runnable
{

	/**
		Log job agent events.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.jobagent.JobRunner");

	private final Job job;
	private final JobAgentSettings settings;
	private final JobExecutor jobExecutor;
	private final Agent agent;
	
	/**
		Create a new job execution thread.
		@param agent The agent that created this object
		@param job The job to be executed
		@param settings The settigns for the plugin that is executing the job
		@param jobExecutor The job executor that should execute the job
	*/
	JobRunner(Agent agent, Job job, JobAgentSettings settings, JobExecutor jobExecutor)
	{
		this.agent = agent;
		this.job = job;
		this.settings = settings;
		this.jobExecutor = jobExecutor;
	}
	
	/*
		From the Runnable interface
		-------------------------------------------
	*/
	/**
		Find a free slot to use and then pass control to the {@link 
		JobExecutor#executeJob(SessionControl, Agent, Job, JobAgentSettings, Job.ExecutionTime)}
		method.
	*/
	public void run()
	{
		Job.ExecutionTime slotToUse = agent.getSlot(job);
		if (slotToUse == null)
		{
			log.info("Couldn't find a free slot for executing job: " + job);
			return;
		}
		DbControl dc = null;
		SessionControl sc = null;
		try
		{
			log.info("Executing job " + job + " using executor " + jobExecutor);
			
			// Log in as the owner of the job
			sc = agent.getImpersonatedSessionControl(job);
			dc = sc.newDbControl();
			
			// Load a fresh copy of the job
			Job j = Job.getById(dc, job.getId());
			
			// Status may have changed since the queue was checked
			if (j.getStatus() != Job.Status.WAITING) return;
			
			// Change the status to Job.Status.PREPARED
			j.setPrepared(agent.getServerName(), agent.getJobAgent(dc));
			try
			{
				dc.commit();
			}
			catch (ItemModifiedException ex)
			{
				// Another thread or agent has started the job already, return 
				// without starting the job
				return;
			}
			
			// To protect the session control from timeouts
			Application.Pinger pinger = Application.newPinger(sc);
			try
			{
				jobExecutor.executeJob(sc, agent, j, settings, slotToUse);
			}
			catch (Throwable t)
			{
				// Something went wrong, update status to ERROR
				dc = sc.newDbControl();
				j = Job.getById(dc, j.getId());
				j.doneError(t.getMessage(), Arrays.asList(t));
				dc.commit();
				throw t;
			}
			finally
			{
				if (pinger != null) pinger.stop();
			}
			
			// Check that that the status has changed from prepared
			dc = sc.newDbControl();
			j = Job.getById(dc, j.getId());
			if (j.getStatus() == Job.Status.PREPARED)
			{
				j.doneError("Unknown error, job was never executed.");
				log.error("Unknown error: Job " + job + " was never executed by executor " + jobExecutor);
			}
			else 
			{
				log.info("Done executing job: " + job + " with executor " + jobExecutor);
			}
			dc.commit();
		}
		catch (Throwable t)
		{
			log.error(t.getMessage(), t);
		}
		finally
		{
			agent.jobDone(job, slotToUse);
			if (dc != null) dc.close();
			if (sc != null && sc.isLoggedIn()) 
			{
				try
				{
					sc.logout();
				}
				catch (Throwable t)
				{
					log.error(t.getMessage(), t);
				}
			}
		}
	}
	// -------------------------------------------

}
