/*
	$Id: FileTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005, 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.SharedItem;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.User;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import java.io.FileInputStream;
import java.math.BigInteger;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.sql.ResultSet;
import java.util.regex.Pattern;

/**
This class provides methods to transfer base 1 items that correspond to the
File object in BASE 2.

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/

abstract class FileTransfer extends CommonItemTransfer
{
	/**
	Indicates wether MD5 should be calculated or not. This is always true.
	 */
	protected boolean VERIFY_MD5 = true;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.FileTransfer");

	/**
	Constructs a new FileTransfer.
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	 */
	FileTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
	}

	/**
	Physically transfers a file. Retries to upload the file three times or
	until the md5sum is correct.
	
	@param file <code>File</code> in BASE 2 used for upload
	@param theOldBase1File Full path to the base 1 file
	@param md5 Calculated md5sum of theOldBase1File
	@see #md5sum(String)
	 */
	protected void transferFile(File file, String theOldBase1File, String md5)
	{
		if (!QUICK_TRANSFER)
		{
			log.info("Quick transfer is NOT selected - transferring the physical file to BASE2");
			try
			{
				java.io.File theOldFile = new java.io.File(theOldBase1File);
				if (theOldFile.exists() && theOldFile.isFile())
				{
					int attempts = 0;
					log.debug("Uploading file...");
					while (!md5.equals(file.getMd5()) && attempts <= 3)
					{
						file.setMimeTypeAuto(null, null);
						file.upload(theOldFile.toURI().toURL().openStream(), false);
						attempts++;
					}
					if (attempts <= 4) 
					{
						if (log.isDebugEnabled())
						{
							log.debug(file.getName() + " uploaded after " + attempts + " attempts");
							log.debug("Uploading file...DONE");
						}
					}
					else
					{
						log.warn("Couldn't upload the file. Tried 3 times");
					}
				}
			}
			catch (Exception e)
			{
				log.error("Couldn't upload the file", e);
				stop();
			}
		}
		else
		{
			log.info("Quick transfer selected, the physical file will not be transfered.");
		}
	}

	/**
	Calculates MD5sum for a file
	
	@param file <code>java.io.File</code> to calculate the md5sum on
	@return md5sum or null if no file exists
	 */
	protected String md5sum(java.io.File file)
	{
		BigInteger md5 = null;
		try
		{
			if (file.exists() && file.isFile())
			{
				FileInputStream fis = new FileInputStream(file);
				DigestInputStream dis = new DigestInputStream(fis,
						MessageDigest.getInstance("MD5"));
				int bufferSize = 8 * 1024;
				byte[] buf = new byte[bufferSize];
				while (dis.read(buf, 0, bufferSize) != -1)
				{}
				MessageDigest md = dis.getMessageDigest();
				md5 = new BigInteger(1, md.digest());
				dis.close();
				fis.close();
				return md5.toString(16);
			}
			else 
			{
				log.warn("BASE 1 file doesn't exists: " + file.toString());
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't calculate the Md5sum for file", e);
			stop();
		}
		return null;
	}

	/**
	Returns the md5sum for a file specified by the fileLocation.
	
	@param fileLocation String e.g. "/tmp/file.tif"
	@return md5sum or null if no file exists
	 */
	protected String md5sum(String fileLocation)
	{
		return md5sum(new java.io.File(fileLocation));
	}

	private static Pattern forbiddenCharacters = Pattern.compile("[\\~\\/\\:\\;\\*\\?\\<\\>\\|\\\\]");
	/**
	Replaces forbidden characters for file or directory names.
	 */
	protected static String replaceForbiddenCharacters(String in, String replacement)
	{
		// All forbidden characters are replaced
		String out = forbiddenCharacters.matcher(in).replaceAll(replacement);
		return out;
	}

	/**
	Returns an existing file if the name and md5sum matches a file already in
	the database.
	
	@param owner <code>User</code> that might already have this file.
	@param fileName Name of the file including suffix. e.g. A123file.txt
	@param location A complete path to the base 1 file.
	@return File or null if a file could not be created. The returned file
	        must be reattached to a DbControl if it is to be updated.
	 */
	protected File getExistingFile(User owner, String fileName, String location)
	{
		File file = null;
		try
		{
			// We impersonate the user when listing existing files. Beacuse we
			// need to list shared files.
			java.io.File base1File = new java.io.File(location);
			if (base1File.exists() && base1File.isFile())
			{
				log.debug("Impersonating '" + owner.getName() + "'");
				SessionControl impersonated = sessionControl.impersonateLogin(
						owner.getId(), "Looking up existing files");
				DbControl dbc = impersonated.newDbControl();
				ItemQuery<File> fileQuery = File.getQuery();
				fileQuery.include(Include.MINE);
				fileQuery.include(Include.REMOVED);
				fileQuery.include(Include.SHARED);
				fileQuery.restrict(Restrictions.eq(
						Hql.property("name"),
						Expressions.parameter("_fileName_")));
				fileQuery.setParameter("_fileName_", fileName, Type.STRING);
				fileQuery.restrict(Restrictions.eq(
						Hql.property("md5"), Expressions.parameter("_md5_")));
				// Check if the MD5 sum is the same
				String oldMd5 = md5sum(base1File);
				fileQuery.setParameter("_md5_", oldMd5, Type.STRING);
				ItemResultList<File> files = fileQuery.list(dbc);
				if (files.getTotalCount() == 1)
				{
					// Only one file can exists having the same name and md5sum
					file = files.get(0);
				}
				dbc.close();
				log.debug("Logging out '" + owner.getName() + "'");
				impersonated.logout();
			}
		}
		catch (BaseException e)
		{
			log.warn("FAILED when getting the Existing file");
			if (log.isDebugEnabled())
			{
				log.debug("\t" + e);
			}
		}
		return file;
	}

	/**
	Sets common item data for File objects.
	
	@param dc
	@param file File for which to set the data
	@param rs ResultSet containing data for the current file
	 */
	protected void setCommonItemData(DbControl dc, File file, ResultSet rs)
	{
		setSharedItemData(dc, (SharedItem) file, rs);
		try
		{
			String name = rs.getString(NAME);
			if (name == null)
			{
				name = "unkown";
			}
			else
			{
				name = replaceForbiddenCharacters(name, "");
			}
			file.setName(name);
			String description = rs.getString(DESCRIPTION);
			file.setDescription(description);
			file.setRemoved(rs.getBoolean(REMOVED));
		}
		catch (Exception e)
		{
			log.error("Couldn't set common data for File item " + file.getName(), e);
			stop();
		}
	}
}
