/*
	$Id: PluginTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.plugin.Response;

import java.io.PrintWriter;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
	Transfers BASE 1 plugins. The plugins are added as configurations
	for the {@link net.sf.basedb.plugins.Base1PluginExecuter}.
	<dl>
	<dt><b>Base 1 tables</b></dt>
	<dd>Program, ProgramParameter</dd>
	</dl>

	@author Nicklas, Martin
	@version 2.0.2
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/
public class PluginTransfer 
	extends CommonItemTransfer
{
	
	private int directoryId;
	private String pluginDir = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.PluginTransfer");
	
	/**
		@param userT A finished {@link UserTransfer}
		@param groupT A finished {@link GroupTransfer}
	 */
	PluginTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
	}

	public void start()
	{
		log.info("Transferring BASE1 plugins to BASE2 as configurations for Base1PluginExecuter...");
		int count = getInt("countPlugins");
		initIdMap(count);
		startProgress(count, "Plugins");
		
		DbControl dc = newDbControl();
		try
		{
			Path path = new Path("/BASE 1 plugin configuration files", Path.Type.DIRECTORY);
			Directory dir = Directory.getNew(dc, path);
			if (!dir.isInDatabase()) dc.saveItem(dir);
			dir.setDescription("This directory contains plugin configuration files that have been migrated from BASE 1.2");
			dc.commit();
			log.debug("Created directory to save the plugin configuration files in.");
			
			directoryId = dir.getId();
			pluginDir = getProperty("pluginDir");
			if (pluginDir == null) pluginDir = "/usr/local/base/plugins";
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
		runUnBatched(prepareStatementFromFile("selectPlugins"), 0, getBatchSize());
		done();
		log.info("Transferring BASE1 plugins...DONE");
	}

	public void createItem(ResultSet rs)
	{
		String base1Properties = null;
		DbControl dc = newDbControl();
		try
		{	
			base1Properties = rs.getInt(ID) + ":" + rs.getString(NAME); 
			if(log.isDebugEnabled())
			{
				log.debug("Transferring BASE1 name=" +  rs.getString(NAME) + "; id="+rs.getInt(ID));
			}
			// Create new plugin configuration
			PluginDefinition pluginDefinition = getPluginDefinition(dc);
			PluginConfiguration pluginConfiguration = pluginDefinition.newPluginConfiguration();
			setCommonItemData(dc, pluginConfiguration, rs);
			dc.saveItem(pluginConfiguration);
			
			// Load parameters for the plugin
			PreparedStatement ps = prepareStatementFromFile("selectPluginParameters");
			ps.setInt(1, rs.getInt("id"));
			ResultSet parameters = ps .executeQuery();

			// Create BASE 1 plugin configuration file and save to directory
			Directory dir = Directory.getById(dc, directoryId);
			File f = File.getNew(dc, dir);
			f.setOwner(pluginConfiguration.getOwner());
			f.setItemKey(pluginConfiguration.getItemKey());
			f.setProjectKey(pluginConfiguration.getProjectKey());
			dc.saveItem(f);
			createFile(f, rs, parameters);
			parameters.close();
			if (log.isDebugEnabled())
			{
				log.debug("Created Base1PluginExecuter-configuration file for BASE 1 plugin" + rs.getString(NAME));
			}
			
			// Set file and path parameter
			PluginConfigurationRequest request = pluginConfiguration.configure();
			request.setParameterValue("file", f);
			request.setParameterValue("execPath", pluginDir);
			PluginResponse response = request.invoke();
			
			if (response.getStatus() == Response.Status.CONTINUE)
			{
				// Set parsed file parameters
				request = response.getNextRequest();
				RequestInformation ri = request.getRequestInformation();
				for (PluginParameter pp : ri.getParameters())
				{
					request.setParameterValue(pp.getName(), pp.getDefaultValue());
				}
				response = request.invoke();
			}
			
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				if (log.isDebugEnabled())
				{
					log.debug("BASE1 plugin " + rs.getString(NAME) + " successfully transfered to a configuration file for Base1PluginExecuter");
					log.debug("--------------------------------------------------------------------------");
				}
			}
			else
			{
				for (Throwable err : response.getErrorList())
				{
					log.debug(err);
				}
				throw new BaseException(response.getMessage());
			}
		}
		catch (BaseException e)
		{
			log.warn("Couldn't create configuration file[" + base1Properties + "] for BASE 1 plugin: " + e.getMessage());
			if (log.isDebugEnabled()) log.debug("", e);
		}
		catch (SQLException e)
		{
			log.error("SQL statement couldn't be executed", e);
			stop();
		}
		finally
		{
			commit(dc);
		}		
	}

	
	private void createFile(File f, ResultSet rs, ResultSet parameters)
		throws SQLException
	{
		String uniqueName = fixUniqueName(rs.getString("uniqueName")) + "." + rs.getInt("id");
		f.setName(uniqueName);
		f.setDescription("Configuration file for BASE 1 plugin: " + rs.getString("name"));
		
		PrintWriter out = new PrintWriter(f.getUploadStream(false));
		out.println("BASEfile");
		out.println("section\tplugin");
		out.println("uniqueName\t"+encode(rs.getString("uniqueName")));
		out.println("versionNumber\t"+encode(rs.getString("versionNumber")));
		out.println("name\t"+encode(rs.getString("name")));
		out.println("descr\t"+encode(rs.getString("descr")));
		out.println("execName\t"+encode(rs.getString("execName")));
		out.println("geneAverages\t"+encode(rs.getString("geneAverages")));
		out.println("serialFormat\t"+encode(rs.getString("serialFormat")));
		out.println("url\t"+encode(rs.getString("url")));
		out.println("minChannels\t"+encode(rs.getString("minChannels")));
		out.println("maxChannels\t"+encode(rs.getString("maxChannels")));
		out.println("leaveStdin\t"+encode(rs.getString("leaveStdin")));
		out.println("leaveStdout\t"+encode(rs.getString("leaveStdout")));
		out.println("estimatedTime\t"+encode(rs.getString("estimatedTime")));
		out.println("defaultMaxRam\t"+encode(rs.getString("defaultMaxRam")));
		out.println("usedColumns\t"+encode(rs.getString("usedColumns")));
		out.println("usedFields\t"+encode(rs.getString("usedFields")));
		out.println("columns\tposition\tvalueType\tname\tcommonName\toptions\tdefaultValue\tenumOptions\tremoved");
		out.println("%");
		
		while (parameters.next())
		{
			out.print(encode(parameters.getString("position"))+"\t");
			out.print(encode(parameters.getString("valueType"))+"\t");
			out.print(encode(parameters.getString("name"))+"\t");
			out.print(encode(parameters.getString("commonName"))+"\t");
			out.print(encode(parameters.getString("options"))+"\t");
			out.print(encode(parameters.getString("defaultValue"))+"\t");
			out.print(encode(parameters.getString("enumOptions"))+"\t");
			out.print(encode(parameters.getString("removed"))+"\n");
		}
		
		out.close();
		
	}

	private PluginDefinition getPluginDefinition(DbControl dc)
		throws BaseException
	{
		String className = "net.sf.basedb.plugins.Base1PluginExecuter";
		PluginDefinition pluginDefinition = null;
		try
		{
			pluginDefinition = PluginDefinition.getByClassName(dc, className);
		}
		catch (Exception e)
		{
			log.warn("Couldn't create plugin definition of Base1PluginExecuter", e);
			if (log.isDebugEnabled()) log.debug("\t" + e);
		}
		return pluginDefinition;
	}
	
	private String fixUniqueName(String base1UniqueName)
	{
		String fixedName = null;
		if (base1UniqueName != null)
		{
			fixedName = FileTransfer.replaceForbiddenCharacters(base1UniqueName, "_");
		}
		return fixedName;
	}
	
	/**
		Replace tabs and newlines with \n, \r and \t
	*/
	private String encode(String in)
	{
		String out = "";
		if (in != null)
		{
			out = in.replaceAll("\\t", "\\\\t").replaceAll("\\n", "\\\\n").replaceAll("\\r", "\\\\r");
		}
		return out;
	}
	
}
