/**
	$Id: ExperimentExplorer.java 4548 2008-09-26 12:29:03Z nicklas $

	Copyright (C) 2006 Johan Enell, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.PageContext;

import net.sf.basedb.util.Values;
import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.clients.web.taglib.table.TableColumn;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.DynamicResultIterator;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemContext;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.ItemContext.SortDirection;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Aggregations;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.util.BioAssaySetUtil;
import net.sf.basedb.util.formatter.Formatter;

/**
	A class for storing settings and doing work related to the experiment 
	explorer tool in the web interface. The JSP pages that uses this class
	are located in the <code>www/views/experiments/explorer</code> directory
	and it's subdirectory.
	<p>
	This class is used two things:
	<ul>
	<li>Storing the settings
	<li>Creating the queries that retrieves the data according to the settings
	</ul>
	
	<p>
	Most of the settings are stored in regular {@link ItemContext}: objects. This
	means that they will be saved to the database when the user logs out and
	loaded again the next time they are needed. Each experiment explorer uses
	two <code>ItemContext</code> objects. One for storing the settings for the
	reporter search page and one for the view page. The reporter search settings
	are stored in the context <code>SessionControl.getCurrentContext(Item.REPORTER,
	"explorer.id")</code> and the view page settings are stored in 
	<code>SessionControl.getCurrentContext(Item.SPOTDATA, "explorer.id")</code> where
	"id" is replaced by the bioassayset's id.


	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-26 14:29:03 +0200 (Fri, 26 Sep 2008) $
*/
public class ExperimentExplorer
{

	/**
		Special value to use in the {@link #setPositionIndex(int)} method 
		to show all positions.
	*/
	public static final int SPOT_ALL = -1;
	
	/**
		Special value to use in the {@link #setPositionIndex(int)} method 
		to calculate the average for each bioassay.
	*/
	public static final int SPOT_AVG = -2;
	
	/**
		The size of the cache that stores reporter id's matching the search.
		@see #getCacheSize()
	*/
	public static final int DEFAULT_CACHE_SIZE = 100;

	/**
	 	Predefined colors for annotation groups
	*/
	public static final String[] COLORS = 
		{"#ffcc00", "#00ffcc", "#00b3ff", "#ff00b3", "#00e090", "#ff0033", "#a8a8a8", "#33ff00"};
	
	/**
		The default context for the reporter search page.
	*/
	private static final ItemContext defaultReporterContext = 
		Base.createDefaultContext("@externalId", "name,externalId");
	
	/**
		Stores default context for the spot data list on the view page. 
		Different defaults for each raw data type since they can have different
		number of channels.
	*/
	private static final Map<RawDataType, ItemContext> defaultSpotContexts = 
		new HashMap<RawDataType, ItemContext>();

	static
	{
		for (RawDataType rdt : RawDataTypes.getRawDataTypes())
		{
			// Default visible columns are: position, ch1, ch2, ...
			StringBuilder columns = new StringBuilder("bioassay,position");
			for (int ch = 1; ch <= rdt.getChannels(); ++ch)
			{
				columns.append(",ch").append(ch);
			}
			defaultSpotContexts.put(rdt, 
				Base.createDefaultContext("POSITION", columns.toString()));
		}
	}
	
	/**
		Get the existing experiment explorer for a bioassayset or create a new
		one if none exist yet. The returned object is cached in the {@link
		SessionControl}:s session settings.
		@param bioAssaySet The bioassay set to get an experiment explorer for
		@return An <code>ExperimentExplorer</code> object
	*/
	public static ExperimentExplorer getExplorer(BioAssaySet bioAssaySet)
	{
		String explorerSetting = "explorer." + bioAssaySet.getId();
		SessionControl sc = bioAssaySet.getSessionControl();
		ExperimentExplorer explorer = (ExperimentExplorer)sc.getSessionSetting(explorerSetting);
		if (explorer == null)
		{
			explorer = new ExperimentExplorer(bioAssaySet);
			sc.setSessionSetting(explorerSetting, explorer);
		}
		return explorer;
	}
	
	/**
		The bioassay set we are exploring.
	*/
	private final BioAssaySet bioAssaySet;
	
	/**
		Maps a bioassays cube column number to it's id for fast
		loading by the {@link #getBioAssay(DbControl, short)} method.
	*/
	private final Map<Short, Integer> bioAssays;
	
	/**
		The cache of reporter ids.
		@see #getCacheSize()
	*/
	private int[] reporterIds;
	
	/**
		The current cache size.
	*/
	private int cacheSize;

	/**
		The index of the first cached reporter id.
		@see #getReporterIndex() 
	*/
	private int firstCachedIndex = 0;

	/**
		The index of the first cached reporter id.
		@see #getReporterIndex()
	*/
	private int lastCachedIndex = -1;
	
	/**
		The index of the current reporter on the view page.
		@see #getReporterIndex()
	*/
	private int reporterIndex = 0;
	/**
		The index of the curren position on the view page.
		@see #getPositionIndex()
	*/
	private int positionIndex = 0;
	
	/**
		Number of reporter that match the reporter search.
	*/
	private int matchingReporters = -1;
	
	/**
		The currently selected annotation on the view page.
	*/
	private int annotationTypeId = 0;
	
	/**
		Create a new experiment explorer.
	*/
	private ExperimentExplorer(BioAssaySet bioAssaySet)
	{
		this.bioAssaySet = bioAssaySet;
		this.bioAssays = new HashMap<Short, Integer>();
		this.cacheSize = DEFAULT_CACHE_SIZE;
		for (BioAssay ba : bioAssaySet.getBioAssays().list(bioAssaySet.getDbControl()))
		{
			bioAssays.put(ba.getDataCubeColumnNo(), ba.getId());
		}
	}

	/**
		Get the bioassay set that is currently beeing explored.
		@param dc The DbControl used for access the database
		@return The <code>BioAssaySet</code> object
		@see BioAssaySet#getById(DbControl, int)
	*/
	public BioAssaySet getBioAssaySet(DbControl dc)
	{
		return BioAssaySet.getById(dc, bioAssaySet.getId());
	}	
	
	/**
		Get the number of bioassays in the bioassayset.
	*/
	public int getNumBioAssays()
	{
		return bioAssays.size();
	}
	
	/**
		Get the bioassay with the specified column number in the
		data cube. This method is useful since spot queries (see 
		{@link #getSpotQuery(DbControl, ReporterData, int, List)})
		can only return the column number, but we usually want to display
		the name, etc.
		
		@param dc The DbControl object used to get the bioassay
		@param columnNo The colum number
		@return The bioassay with the specified column number or null
			if no bioassay is found
	*/
	public BioAssay getBioAssay(DbControl dc, short columnNo)
	{
		Integer baId = bioAssays.get(columnNo);
		return baId == null ? null : BioAssay.getById(dc, baId);
	}

	
	/**
		The number of reporter id'd to cache. For performance reasons
		the id's of the reporters matching the reporter search are cached.
		This means we can step forwards and backwards more quickly on the
		view page since we don't have to execute the reporter search each 
		time. If the cache is empty or we step outside the boundaries the
		cache is reloaded with reporters symmetrically located around the
		index {@link #setReporterIndex(int)} setting.
		@return The size of the cache
	*/
	public int getCacheSize()
	{
		return cacheSize;
	}
	
	/**
		Set the size of the reporter cache.
		@param cacheSize The size
		@see #getCacheSize()
	*/
	public void setCacheSize(int cacheSize)
	{
		this.cacheSize = cacheSize;
	}
	
	/**
		Get the total number of unique reporters found in the bioassayset.
		@see BioAssaySet#getNumReporters()
	*/
	public int getNumReporters()
	{
		return bioAssaySet.getNumReporters();
	}
	
	/**
		Get the total number of spots in the bioassayset.
	 	@see BioAssaySet#getNumSpots()
	*/
	public int getNumSpots()
	{
		return bioAssaySet.getNumSpots();
	}
	
	/**
		Get the subcontext name used for the {@link ItemContext} objects
		related to this explorer. The name is created like this:
		<code>explorer.id</code> where id is replaced with the id of
		the bioassayset we are exploring.
		
		@return The subcontext name
	*/
	public String getSubContext()
	{
		return "explorer." + bioAssaySet.getId();
	}
	
	/**
		Get the default spot context that is used to initialise the actual context.
		@return An <code>ItemContext</code> object
	*/
	public ItemContext getDefaultSpotContext()
	{
		return defaultSpotContexts.get(bioAssaySet.getRawDataType());
	}

	/**
		Get the default reporter context that is used to initialise the actual context.
		@return An <code>ItemContext</code> object
	*/
	public ItemContext getDefaultReporterContext()
	{
		return defaultReporterContext;
	}
	
	/**
		Get and update the current settings for the spot context. The spot context is used
		on the view page for the list with spot data.
		@param sc The SessionControl of the logged in user
		@param pageContext The {@link PageContext} containing the
			updated settings, or null to not update the settings
		@return An <code>ItemContext</code> object
		@see Base#getAndSetCurrentContext(SessionControl, Item, PageContext, ItemContext)
	 */
	public ItemContext getAndSetSpotContext(SessionControl sc, PageContext pageContext)
	{
		ItemContext cc = Base.getAndSetCurrentContext(sc, Item.SPOTDATA, getSubContext(), pageContext, getDefaultSpotContext());
		if (pageContext != null)
		{
			ServletRequest request = pageContext.getRequest();
			// TODO - update other settings (ie. reporterIndex, spotIndex)
		}
		return cc;
	}
	
	/**
		Get and update the current settings for the reporter context. The reporter context
		is used on the reporter search page.
		@param sc The SessionControl of the logged in user
		@param pageContext The {@link PageContext} containing the
			updated settings, or null to not update the settings
		@return An <code>ItemContext</code> object
		@see Base#getAndSetCurrentContext(SessionControl, Item, PageContext, ItemContext)
	*/
	public ItemContext getAndSetReporterContext(SessionControl sc, PageContext pageContext)
	{
		ItemContext cc = Base.getAndSetCurrentContext(sc, Item.REPORTER, getSubContext(), pageContext, getDefaultReporterContext());
		if (pageContext != null)
		{
			// Settings may have changed, clear cache
			firstCachedIndex = 0;
			lastCachedIndex = -1;
			matchingReporters = -1;
		}
		return cc;
	}
	
	/**
		Get the id of the selected annotation type.
		@return The id of the selected annotation type or 0 if no annotation type
			is selected
	*/
	public int getAnnotationTypeId()
	{
		return annotationTypeId;
	}
	
	/**
		Set the id of the selected annotation type.
		@param annotationTypeId The annotation type id or 0 if none is
			selected
	*/
	public void setAnnotationTypeId(int annotationTypeId)
	{
		this.annotationTypeId = annotationTypeId;
	}
	
	/**
		Get the annotation types that are of interest in this experiment.
		Ie. the experimental factors of the experiment the bioassayset we
		are exploring belongs to. Optionally, you can also get annotations
		for bioassays.

		@param dc The DbControl for database access
		@param includeBioAssayAnnotations If bioassay annotation should be included
			even if they are not experimental factors
		@return A list of annotation types, null may be returned if no annotation types
			exists
		@see net.sf.basedb.core.Experiment#getExperimentalFactors()
	*/
	public List<AnnotationType> getAnnotationTypes(DbControl dc, boolean includeBioAssayAnnotations)
	{
		final ItemQuery<AnnotationType> query = AnnotationType.getQuery(null);
		query.join(Hql.leftJoin("experiments", Item.EXPERIMENT.getAlias()));

		// Restriction for experimental factors
		Restriction ef = Restrictions.eq(
			Hql.alias(Item.EXPERIMENT.getAlias()), 
			Hql.entity(getBioAssaySet(dc).getExperiment())
		);
		// Restrictions for BioAssay annotation types
		if (includeBioAssayAnnotations)
		{
			query.join(Hql.innerJoin("itemTypes", "itemType"));
			Restriction baa = Restrictions.eq(
				Hql.alias("itemType"),
				Expressions.integer(Item.BIOASSAY.getValue())
			);
			ef = Restrictions.or(ef, baa);
		}
		query.restrict(ef);
		query.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
		query.order(Orders.asc(Hql.property("name")));
		query.setDistinct(true); // Needed because of the left join
		return query.list(dc);
	}
	
	/**
		Get an AnnotationSummary object which can be used to calculate means
		and other values for each annotation group. When the annotation summary
		object is create each bioassay is placed in one annotation group
		based on which annotatations it have. Each annotation group is unique and
		is the only one with the same combination of annotation values. More
		than one bioassay may be placed in the same group if they are annotated
		with the same values.
		
		@param dc The DbControl for database access
		@param annotationType The annotation type, mustn't be null
		@return An <code>AnnotationSummary</code> object
		@see AnnotationGroup#getAnnotationValues()
	*/
	public AnnotationSummary getAnnotationSummary(DbControl dc, AnnotationType annotationType)
	{
		return new AnnotationSummary(this, dc, annotationType);
	}
	
	/**
	  	Get the number of reporters matching the current reporter
		filter.
		@param dc The DbControl for database access
	  	@return an <code>int</code> of matching reporters 
	*/
	public int getMatchingReporters(DbControl dc)
	{
		if (matchingReporters == -1)
		{
			matchingReporters = (int)getReporterQuery(dc, null).count(dc);
		}
		return matchingReporters;
	}
	
	/**
		Get the index of the current reporter. It should be a value between
		0 and {@link #getMatchingReporters(DbControl)}-1.
		@see #getReporter(DbControl, int)
	*/
	public int getReporterIndex()
	{
		return reporterIndex;
	}
	
	/**
		Set the index of the currently viewed reporter. If the index is below
		zero or greater than or equal to {@link #getMatchingReporters(DbControl)} it is set
		to the nearest index.
		@param reporterIndex The new index
		@see #getReporter(DbControl, int)
	*/
	public void setReporterIndex(int reporterIndex)
	{
		if (reporterIndex >= matchingReporters) reporterIndex = matchingReporters - 1;
		if (reporterIndex < 0) reporterIndex = 0;
		this.reporterIndex = reporterIndex;
	}
	
	/**
		Get the reporter for the specified index. If the index is outside the
		cached range or if the cache is empty the cache is reloaded with new
		reporters symmetrically located around the specified index.
		
		@param dc The DbControl to use for database access
		@param index The index
		@return The reporter or null if no reporter is found at the specified index
		@see #getCacheSize()
	*/
	public ReporterData getReporter(DbControl dc, int index)
	{
		if (reporterIds == null || reporterIds.length != cacheSize)
		{
			reporterIds = new int[cacheSize];
		}
		
		if (index < firstCachedIndex || index > lastCachedIndex)
		{
			// The index is outside the cached range, reload another range: index  (cacheSize / 2)
			DynamicSpotQuery reporterQuery = getReporterQuery(dc, null);
			
			this.firstCachedIndex = Math.max(0, index - cacheSize / 2);
			reporterQuery.setFirstResult(firstCachedIndex);
			reporterQuery.setMaxResults(cacheSize);
			try
			{
				DynamicResultIterator result = reporterQuery.iterate(dc);
				int cacheIndex = 0;
				while (result.hasNext())
				{
					reporterIds[cacheIndex] = result.next().getInt(1);
					cacheIndex++;
				}
				this.lastCachedIndex = firstCachedIndex + cacheIndex - 1;
			}
			catch (SQLException ex)
			{
				throw new BaseException(ex);
			}
		}
		
		if (index < firstCachedIndex || index > lastCachedIndex)
		{
			return null;
		}
		else
		{
			int reporterId = reporterIds[index - firstCachedIndex];
			return Reporter.getById(dc, reporterId);
		}
	}

	/**
		Get the number of unique positions a reporter is present in
		among all bioassays in the bioassayset.
		@param dc The DbControl to use for database access
		@param reporter The reporter, or null to find spots without any reporter
	*/
	public int getNumPositions(DbControl dc, ReporterData reporter)
	{
		BioAssaySet bas = getBioAssaySet(dc);
		DynamicSpotQuery spotQuery = bas.getSpotData();
		spotQuery.joinReporters(JoinType.LEFT);
		spotQuery.select(Selects.expression(
			Aggregations.count(Dynamic.column(VirtualColumn.POSITION), true),
			"posCount"
		));
		spotQuery.restrict(
			Restrictions.eq(
				Dynamic.reporter("id"), 
				reporter == null ? null : Expressions.integer(reporter.getId())
			)
		);
		int posCount;
		try
		{
			posCount = spotQuery.iterate(dc).next().getInt(1);
		}
		catch (SQLException ex)
		{
			throw new BaseException(ex);
		}
		return posCount;
	}

	/**
		Get the index of the current position. It should be a value between
		0 and {@link #getNumPositions(DbControl, ReporterData)}-1, or one
		of the {@link #SPOT_ALL} or {@link #SPOT_AVG} special values (negative).
		@return The current index
		@see #getPosition(DbControl, ReporterData, int)
	*/
	public int getPositionIndex()
	{
		return positionIndex;
	}
	
	/**
		Set the index of the curren position.
		@param positionIndex The new index
		@see #getPositionIndex()
		@see #getPosition(DbControl, ReporterData, int)
	*/
	public void setPositionIndex(int positionIndex)
	{
		this.positionIndex = positionIndex;
	}
	
	/**
		Get the position number for the specified reporter and position index.
		The metod will query the data for all positions with the specified
		reporter, sort them by position and return the value given by the
		<code>positionIndex</code> parameter.
		
		@param dc The DbControl for database access
		@param reporter The reporter to look for, or null to find spots without reporter
		@param positionIndex The index of the position to find
		@return The position number
	*/
	public int getPosition(DbControl dc, ReporterData reporter, int positionIndex)
	{
		BioAssaySet bas = getBioAssaySet(dc);
		DynamicSpotQuery spotQuery = bas.getSpotData();
		spotQuery.joinReporters(JoinType.LEFT);
		spotQuery.setDistinct(true);
		spotQuery.select(Dynamic.select(VirtualColumn.POSITION));
		spotQuery.restrict(
			Restrictions.eq(
				Dynamic.reporter("id"), 
				reporter == null ? null : Expressions.integer(reporter.getId())
			)
		);
		spotQuery.order(Orders.asc(Dynamic.column(VirtualColumn.POSITION)));
		spotQuery.setFirstResult(positionIndex);
		spotQuery.setMaxResults(1);
		int position;
		try
		{
			position = spotQuery.iterate(dc).next().getInt(1);
		}
		catch (SQLException ex)
		{
			throw new BaseException(ex);
		}
		return position;
	}
	
	/**
		Get a query returning the reporter data used by the reporter search page.
		Note that the query is {@link DynamicSpotQuery} and not a 
		{@link net.sf.basedb.core.DataQuery}. This means that we don't get 
		{@link ReporterData} objects as a result and that we must specify the
		properties to select (see {@link ItemContext#configureQuery(DbControl, DynamicQuery, List)}).
		<p>
		The reporter id is always included as the first selected property so it
		doesn't have to be in the selection list.
		
		@param dc The DbControl for database access
		@param selectionList The properties that the query should select
		@return A query
		@see ItemContext#configureQuery(DbControl, DynamicQuery, List)
	*/
	public DynamicSpotQuery getReporterQuery(DbControl dc, List<String> selectionList)
	{
		DynamicSpotQuery reporterQuery = getBioAssaySet(dc).getSpotData();
		reporterQuery.joinReporters(JoinType.INNER);
		reporterQuery.setDistinct(true);
		reporterQuery.select(Selects.expression(Dynamic.reporter("id"), "id", true));
		
		ItemContext cc = dc.getSessionControl().getCurrentContext(Item.REPORTER, getSubContext(), null);
		cc.configureQuery(dc, reporterQuery, selectionList);
		
		return reporterQuery;
	}
	
	/**
		Get a query returning spot data for a given reporter and position. This query is used
		by the view page to list the spot data for each bioassay.
		<p>
		The column number is always included as the first selected property. If the
		{@link #SPOT_AVG} is specified for <code>positionIndex</code> a count of the
		number of positions is included as the second selected property. Otherwise, the 
		position number is included as the second selected property.
		<p>
		Then, all channel intensities are selected.
		
		<p> 
		See {@link ItemContext#getDynamicExpression(DbControl, String)} for more information 
		about how to specify additional selection properties.
		

		@param dc The DbControl for database access
		@param reporter The reporter, or null to get data for spots without a reporter
		@param positionIndex The index of the position, or one the special
			values {@link #SPOT_ALL} or {@link #SPOT_AVG}
		@param selectionList The data that the query should select
		@return A query
	*/
	public DynamicSpotQuery getSpotQuery(DbControl dc, ReporterData reporter, int positionIndex, List<TableColumn> selectionList)
	{
		BioAssaySet bioAssaySet = getBioAssaySet(dc);
		int channels = bioAssaySet.getRawDataType().getChannels();
		DynamicSpotQuery spotQuery = bioAssaySet.getSpotData();

		// Always select the column (ie. bioassay)
		spotQuery.select(Dynamic.select(VirtualColumn.COLUMN));
		
		if (positionIndex == SPOT_ALL)
		{
			// Get all spots for the current reporter	
			// Select the position as well
			spotQuery.select(Dynamic.select(VirtualColumn.POSITION));
			spotQuery.joinReporters(reporter == null ? JoinType.LEFT : JoinType.INNER);
			spotQuery.restrict(
				Restrictions.eq(
					Dynamic.reporter("id"), 
					reporter == null ? null : Expressions.integer(reporter.getId())
				)
			);
		}
		else if (positionIndex == SPOT_AVG)
		{
			// Average over spots in the same bioassay for the current reporter
			// Select the number of spots we are averaging over
			spotQuery.select(
				Selects.expression(
					Aggregations.count(Dynamic.column(VirtualColumn.COLUMN), false), 
					"cnt"
				)
			);
			spotQuery.joinReporters(reporter == null ? JoinType.LEFT : JoinType.INNER);
			spotQuery.restrict(
				Restrictions.eq(
					Dynamic.reporter("id"), 
					reporter == null ? null : Expressions.integer(reporter.getId())
				)
			);
			spotQuery.group(Dynamic.column(VirtualColumn.COLUMN));
		}
		else
		{
			// Get spots for specific position
			int position = getPosition(dc, reporter, positionIndex);
			// Select the position
			spotQuery.select(Dynamic.select(VirtualColumn.POSITION));
			spotQuery.restrict(
				Restrictions.eq(
					Dynamic.column(VirtualColumn.POSITION), 
					Expressions.integer(position)
				)
			);
		}
		
		// Select channel intensities
		for (int ch = 1; ch <= channels; ++ch)
		{
			Expression select = Dynamic.column(VirtualColumn.channel(ch));
			if (positionIndex == SPOT_AVG)
			{
				select = Aggregations.mean(select);
			}
			spotQuery.select(Selects.expression(select, "ch" + ch));
		}
		
		// Set additional selected columns
		int i = 0;
		for (TableColumn tc : selectionList)
		{
			// Skip channel intensities since they are already selected
			if (!tc.getProperty().startsWith("%"))
			{
				Expression select = ItemContext.getDynamicExpression(dc, tc.getProperty());
				if (select != null)
				{
					i++;
					if (positionIndex == SPOT_AVG)
					{
						select = tc.getAverageMethod().getAverageExpression(select);
					}
					spotQuery.select(Selects.expression(select, "e" + i));
				}
			}
		}
		
		// Add sort order information
		ItemContext cc = dc.getSessionControl().getCurrentContext(Item.SPOTDATA, getSubContext());
		String sortProperties = cc.getSortProperty();
		if (sortProperties != null && sortProperties.length() > 0)
		{
			String[] sp = sortProperties.split(",");
			for (String sortProperty : sp)
			{
				Expression sortby = ItemContext.getDynamicExpression(dc, sortProperty);
				if (positionIndex == SPOT_AVG)
				{
					// Find TableColumn to know which average method to use
					boolean found = false;
					for (TableColumn tc : selectionList)
					{
						if (sortProperty.equals(tc.getProperty()))
						{
							sortby = tc.getAverageMethod().getAverageExpression(sortby);
							found = true;
							break;
						}
					}
					if (!found) sortby = null;
				}
				if (sortby != null)
				{
					spotQuery.order(SortDirection.ASC == cc.getSortDirection() ? Orders.asc(sortby) : Orders.desc(sortby));
				}
			}
		}	
		return spotQuery;
	}
	
	
	/**
		An annotation summary object keeps track of intensities and other 
		statistical information based on the annotation group.
	
		@author nicklas
		@version 2.0
		@base.modified $Date: 2008-09-26 14:29:03 +0200 (Fri, 26 Sep 2008) $
	 */
	public static class AnnotationSummary
	{
	
		private final ExperimentExplorer explorer;
		private final Map<Short, AnnotationGroup> assayGroups;
		private final Set<AnnotationGroup> groups;
		
		@SuppressWarnings("unchecked")
		private AnnotationSummary(ExperimentExplorer explorer, DbControl dc, AnnotationType annotationType)
		{
			this.explorer = explorer;
			this.assayGroups = new HashMap<Short, AnnotationGroup>();
			this.groups = new LinkedHashSet<AnnotationGroup>();
			
			Formatter formatter = FormatterFactory.getTypeFormatter(dc.getSessionControl(), annotationType.getValueType());
			if (annotationType.supportUnits()) formatter = annotationType.getDefaultUnit().getFormatter(formatter);
			BioAssaySet bioAssaySet = explorer.getBioAssaySet(dc);
			int channels = bioAssaySet.getRawDataType().getChannels();
			Map<Set<?>, AnnotationGroup> temp = new HashMap<Set<?>, AnnotationGroup>();
			int groupId = 0;
			for (BioAssay ba : bioAssaySet.getBioAssays().list(dc))
			{
				Set<?> annotationValues = BioAssaySetUtil.getAnnotationValues(dc, ba, annotationType);
				AnnotationGroup group = temp.get(annotationValues);
				if (group == null)
				{
					group = new AnnotationGroup(++groupId, annotationValues, channels, formatter);
					temp.put(annotationValues, group);
					groups.add(group);
				}
				assayGroups.put(ba.getDataCubeColumnNo(), group);
			}
			temp.clear();
		}
		
		/**
			Get the annotation group for the bioassay with the 
			give data cube column.
			
			@param column The data cube column of an bioassay
			@return An AnnotationGroup object, or null if none is found
		*/
		public AnnotationGroup getAnnotationGroup(short column)
		{
			return assayGroups.get(column);
		}

		/**
			Get all annotation groups that was found among the bioassays.
			@return The annotations groups.
		*/
		public Set<AnnotationGroup> getAnnotationGroups()
		{
			return groups;
		}
	}
	
	/**
		An annotation group contains statistical information about the spot
		values falling into the group.
	
		@author nicklas
		@version 2.0
		@base.modified $Date: 2008-09-26 14:29:03 +0200 (Fri, 26 Sep 2008) $
	 */
	public static class AnnotationGroup
	{
		private final int id;
		private final Set<?> annotationValues;
		private final String title;
		private final float[] sum;
		private final int[] count;
		private final Map<String, Float> statistics;
		
		private <T> AnnotationGroup(int id, Set<T> annotationValues, int channels, Formatter<T> formatter)
		{
			this.id = id;
			this.annotationValues = annotationValues;
			this.title = Values.getString(annotationValues, ", ", true, formatter);
			this.sum = new float[channels+1]; // Channels are 1-based
			this.count = new int[channels+1];
			this.statistics = new HashMap<String, Float>();
		}
		
		/**
			The ID of the group. The ID is only useful within the objects 
			AnnotationSummary objects lifetime. The first group that is
			created is given an ID of 1, the second 2, and so on.
			@return The ID
		*/
		public int getId()
		{
			return id;
		}
		
		/**
			Get the title of this annotation group. The title is created
			by concatenating all annotation values.
			@return The title
		*/
		public String getTitle()
		{
			return title;
		}
		
		/**
			Get all annotation values in this annotation group.
			@return A set containing all annotation values
		*/
		public Set<?> getAnnotationValues()
		{
			return annotationValues;
		}

		/**
			Get a color code for HTML that is as distinct as possible from
			the color of the other groups. The colors may repeat if there are many
			groups.
			@return The color string
		*/
		public String getColor()
		{
			return COLORS[(id - 1) % COLORS.length];
		}
		
		/**
			Add an intensity value to this group.
			@param channel The number of the channel, starting with 1
			@param intensity The spot's intensity
		*/
		public void addValue(int channel, float intensity)
		{
			sum[channel] += intensity;
			count[channel]++;
		}
		
		/**
			Get the sum of intensities for the specified channel.
			@param channel The number of the channel, starting with 1
			@return The sum of the intensities
			@see #addValue(int, float)
		*/
		public float getSum(int channel)
		{
			return sum[channel];
		}
		
		/**
		 	Get the number of intensity values added to the specified channel.
			@param channel The number of the channel, starting with 1
			@return The number of values
			@see #addValue(int, float)
		*/
		public int getCount(int channel)
		{
			return count[channel];
		}
		
		/**
			Get the mean of the intensity values for the given channel.
			@param channel he number of the channel, starting with 1
			@return The mean of the values, or NaN if no values has been added
			@see #addValue(int, float)
		*/
		public float getMean(int channel)
		{
			int theCount = count[channel];
			float theMean = theCount > 0 ? sum[channel] / theCount : Float.NaN;
			return theMean;
		}
		
		/**
			Add another statisitcal measurement to this group. All values
			with the same key are added together and can be retrieved with 
			the {@link #getStatistics(String)} method.
			
			@param key The key for the measurment
			@param value The value to add
		 */
		public void addStatistics(String key, float value)
		{
			Float current = statistics.get(key);
			if (current == null)
			{
				current = new Float(value);
			}
			else
			{
				current = new Float(current.floatValue() + value);
			}
			statistics.put(key, current);
		}
		
		/**
			Get the sum of all values added with the {@link #addStatistics(String, float)}
			method with the given key.
			@param key The measurement key
			@return The sum of the added values or Float.NaN if no value is found
		*/
		public float getStatistics(String key)
		{
			Float value = statistics.get(key);
			return value == null ? Float.NaN : value.floatValue();
		}
		
	}
}
