/*
	$Id: AbstractEntityQuery.java 4600 2008-10-24 11:43:13Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.EntityQuery;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.QueryParameter;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.QueryType;
import net.sf.basedb.core.signal.SignalException;

import java.util.Set;
import java.util.Map;
import java.util.Collections;
import java.util.Collection;
import java.util.EnumSet;

import org.hibernate.Query;

/**
	An abstract implementation of the {@link EntityQuery} interface. This class
	doesn't support adding selections, groups or having restrictions.
	<p>
	{@link Include} options and permission control are implemented using 
	Hibernate filters. See {@link QueryRuntimeFilterFactory}.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-10-24 13:43:13 +0200 (Fri, 24 Oct 2008) $
*/
abstract class AbstractEntityQuery
	extends AbstractQuery
	implements EntityQuery
{

	/**
		Log all parameter bindings to prepared statement.
	*/
	private static final org.apache.log4j.Logger logParam = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.query.bind.AbstractEntityQuery");
		
	/**
		So we don't always have to call logParam.debug()
	*/
	private static final boolean debugEnabled = logParam.isDebugEnabled();

	/**
		The item type returned by the query, eg.
		SELECT [returnType] FROM ...
	*/
	private final Item returnType;
	
	/**
		The root item type we start with in the query. eq.
		SELECT ... FROM [rootTypeTable]
		@since 2.8
	*/
	private final Item rootType;
	
	/**
		If the query should be executed in a stateless session or not.
	*/
	private final boolean stateless;
	
	/**
		The required runtime filter.
	*/
	private final QueryRuntimeFilter requiredFilter;
	
	/**
		Optional runtime filter.
	*/
	private final QueryRuntimeFilter optionalFilter;

	/**
		The manager of the runtime query filters.
	*/
	private QueryRuntimeFilterManager filterManager;
	
	/**
		The most recent DbControl used to execute the main query.
	*/
	private DbControl lastMainDc;

	/**
		The most recent DbControl used to execute the count query.
	*/
	private DbControl lastCountDc;
	
	/**
		The most recent main query.
	*/
	private org.hibernate.Query lastMainQuery;
	
	/**
		The most recent count query.
	*/
	private org.hibernate.Query lastCountQuery;
	
	/**
		If the results should be cached or not.
	*/
	private boolean cacheResults;
	
	/**
		Flags specifying which items to include.
	*/
	private EnumSet<Include> includes;
	
	/**
		The logged in user's permission on the items.
	*/
	private Permission permission;
	
	/**
		Create a new query returning items of the specified item type
		using the default optional runtime filter.
		@param returnType The type of items that are returned
		@param entityName The name of the type of items as known to
			Hibernate, or null to use the class of the return type
		@param stateless TRUE if the stateless Hibernate session should be used, FALSE
			if the regular Hibernate session should be used
	*/
	AbstractEntityQuery(Item returnType, String entityName, boolean stateless)
	{
		this(returnType, returnType, entityName, null, stateless, 
				QueryRuntimeFilterFactory.getOptionalFilter(returnType));
	}
	
	/**
		Create a new query returning items of the specified type with a non-default
		optional runtime filter.
		@param returnType The type of items that are returned
		@param entityName The name of the type of items as known to
			Hibernate, or null to use the class of the return type
		@param stateless TRUE if the stateless Hibernate session should be used, FALSE
			if the regular Hibernate session should be used
		@param optionalFilter A runtime filter replacing the default optional filter
			or null to not use any optional filter
	*/
	AbstractEntityQuery(Item returnType, String entityName, boolean stateless, QueryRuntimeFilter optionalFilter)
	{
		this(returnType, returnType, entityName, null, stateless, optionalFilter);
	}

	/**
		Create a new query that may have different return type and root item type.

		@see DataQuery#DataQuery(Class, Item, String, String)
		@since 2.8
	*/
	AbstractEntityQuery(Item returnType, Item rootType, String rootName, String select, 
			boolean stateless, QueryRuntimeFilter optionalFilter)
	{
		super(rootName == null ? rootType.getDataClass().getName() : rootName);
		super.selectPermanent(Selects.expression(Hql.property(rootType.getAlias(), select), null, true));
		this.returnType = returnType;
		this.rootType = rootType;
		this.stateless = stateless;
		this.requiredFilter = QueryRuntimeFilterFactory.getRequiredFilter(rootType);
		this.optionalFilter = optionalFilter;
		this.includes = EnumSet.of(Include.NOT_REMOVED, Include.MINE, Include.IN_PROJECT, 
				Include.ANNOTATED, Include.NOT_ANNOTATED);
		this.permission = Permission.READ;
		setAutoJoinType(JoinType.LEFT);
	}
	
	/*
		From the Query interface
		-------------------------------------------
	*/
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void select(Select select)
	{
		throw new UnsupportedOperationException();
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void selectPermanent(Select select)
	{
		throw new UnsupportedOperationException();
	}
	
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void group(Expression expression)
	{
		throw new UnsupportedOperationException();
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void groupPermanent(Expression expression)
	{
		throw new UnsupportedOperationException();
	}

	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void having(Restriction restriction)
	{
		throw new UnsupportedOperationException();
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void havingPermanent(Restriction restriction)
	{
		throw new UnsupportedOperationException();
	}
	
	/**
		@return {@link QueryType#HQL}
	*/
	public QueryType getQueryType()
	{
		return QueryType.HQL;
	}
	
	/**
		The alias of the item that is the root of this query.
		@see Item#getAlias()
	*/
	public String getRootAlias()
	{
		return rootType.getAlias();
	}
	
	public boolean isReadonly()
	{
		return super.isReadonly() || lastMainQuery != null;
	}
	
	/**
		Reset all non-permanent query elements of the query and 
		clear cached queries.
	*/
	public void reset()
	{
		super.reset();
		lastMainQuery = null;
		lastCountQuery = null;
		lastMainDc = null;
		lastCountDc = null;
	}
	public long count(DbControl dc)
		throws BaseException
	{
		long count = -1;
		enableFilters(dc);
		try
		{
			Query query = getCountHqlQuery(dc);
			count = isStateless() ?
				QueryExecutor.loadData(Long.class, query, dc.getStatelessSession(), dc.getSessionControl()) :
				HibernateUtil.loadData(Long.class, query);
		}
		catch (InterruptedException ex)
		{
			throw new SignalException("Aborted by user", ex);
		}
		finally
		{
			disableFilters(dc);
		}
		return count;
	}
	// -------------------------------------------
	/*
		From the HqlQuery interface
		-------------------------------------------
	*/
	public void setCacheResult(boolean flag)
	{
		this.cacheResults = flag;
	}
	public boolean isCachingResult()
	{
		return cacheResults;
	}
	// -------------------------------------------
	
	/*
		From the EntityQuery interface
		-------------------------------------------
	*/
	public Item getItemType()
	{
		return returnType;
	}
	/**
		@since 2.8
	*/
	public Item getRootType()
	{
		return rootType;
	}
	public void include(Include... includes)
	{
		for (Include i : includes)
		{
			this.includes.add(i);
		}
	}
	public void include(Collection<Include> includes)
	{
		this.includes.addAll(includes);
	}
	public void exclude(Include... excludes)
	{
		for (Include i : excludes)
		{
			this.includes.remove(i);
		}
	}
	public void exclude(Collection<Include> includes)
	{
		this.includes.removeAll(includes);
	}
	public boolean isIncluded(Include... includes)
	{
		boolean isIncluded = true;
		for (Include i : includes)
		{
			isIncluded = isIncluded && this.includes.contains(i);
		}
		return isIncluded;
	}
	public boolean isIncluded(Collection<Include> includes)
	{
		return this.includes.containsAll(includes);
	}
	public void setItemPermission(Permission permission)
	{
		this.permission = permission == null ? Permission.READ : permission;
	}
	public Permission getItemPermission()
	{
		return permission;
	}
	// -------------------------------------------
	
	/**
		Specify the join type of automatic joins. The default join type is
		{@link JoinType#LEFT}. This setting doesn't affect joins that has been made
		directly to the the query.
		@param joinType The type of the automatic joins
		@since 2.8
	*/
	public void setAutoJoinType(JoinType joinType)
	{
		addAutoJoiner(HqlQueryJoiners.getReporterListJoiner(joinType));
	}
	
	/**
		Build the main query and set parameter values for it. If the query has been 
		executed with the same <code>DbControl</code> before and hasn't been {@link #reset()} 
		the cached query is returned. If the parameters values have been changed the new
		values are used.
		
		@param dc The DbControl to use for executing the query
		@return A <code>org.hibernate.Query</code> object
	*/
	org.hibernate.Query getMainHqlQuery(DbControl dc)
	{
		if (lastMainQuery == null || lastMainDc != dc)
		{
			// If we are using the same DbControl as before we don't have to rebuild the query
			String mainHql = getMainQuery(dc, true);
			if (stateless)
			{
				lastMainQuery =  HibernateUtil.createQuery(dc.getStatelessSession(), mainHql);
			}
			else
			{
				lastMainQuery =  HibernateUtil.createQuery(dc.getHibernateSession(), mainHql);
			}
		}
		lastMainDc = dc;
		lastMainQuery.setCacheable(isCachingResult());
		setParameters(lastMainQuery, getParameters());
		if (getFirstResult() >= 0) lastMainQuery.setFirstResult(getFirstResult());
		if (getMaxResults() > 0) lastMainQuery.setMaxResults(getMaxResults());
		return lastMainQuery;
	}
	
	/**
		Build the count query and set parameter values for it. If the query has been 
		executed with the same <code>DbControl</code> before and hasn't been {@link #reset()} 
		the cached query is returned. If the parameters values have been changed the new
		values are used.
		
		@param dc The DbControl to use for executing the query
		@return A <code>org.hibernate.Query</code> object
	*/
	org.hibernate.Query getCountHqlQuery(DbControl dc)
	{
		if (lastCountQuery == null || lastCountDc != dc)
		{
			// If we are using the same DbControl as before we don't have to rebuild the query
			String countHql = getCountQuery(dc, true);
			if (stateless)
			{
				lastCountQuery =  HibernateUtil.createQuery(dc.getStatelessSession(), countHql);
			}
			else
			{
				lastCountQuery =  HibernateUtil.createQuery(dc.getHibernateSession(), countHql);
			}
		}
		lastCountDc = dc;
		lastCountQuery.setCacheable(true);
		setParameters(lastCountQuery, getParameters());
		return lastCountQuery;
	}
	
	/**
		If queries are using the stateless Hibernate session or the regular
		session.
		@return TRUE if the stateless session is used, FALSE otherwise
		@since 2.4
	*/
	protected boolean isStateless()
	{
		return stateless;
	}
	
	/**
		Enable runtime query filters for the query.
		@see #disableFilters(DbControl)
	*/
	void enableFilters(DbControl dc)
	{
		if (requiredFilter != null || optionalFilter != null)
		{
			filterManager = new QueryRuntimeFilterManager(dc);
			if (requiredFilter != null) requiredFilter.enableFilters(filterManager, this, dc);
			if (optionalFilter != null) optionalFilter.enableFilters(filterManager, this, dc);
		}
	}
	
	/**
		Disable all enabled runtime query filters for the query.
		@see #enableFilters(DbControl)
	*/
	void disableFilters(DbControl dc)
	{
		if (filterManager != null)
		{
			filterManager.disableAll();
			filterManager = null;
		}
	}
	
	private static final Set<Integer> ZERO_SET = Collections.singleton(0);

	/**
		Set parameter values on a query.
		@param query The Hibernate query object
		@param parameters A map containing parameter names and values
	*/
	private void setParameters(org.hibernate.Query query, Map<String, QueryParameter> parameters)
	{
		if (parameters == null) return;
		if (debugEnabled)
		{
			logParam.debug("Binding parameters to query: " + query.getQueryString());
		}
		for (Map.Entry<String, QueryParameter> entry : parameters.entrySet())
		{
			String name = entry.getKey();
			QueryParameter qp = entry.getValue();
			Type valueType = qp.getType();
			Object value = qp.getValue();
			if (value instanceof Collection)
			{
				Collection c = (Collection)value;
				if (debugEnabled)
				{
					logParam.debug("Binding collection '"+value+"' to parameter :"+name);
				}
				if (valueType == null)
				{
					query.setParameterList(name, c.size() == 0 ? ZERO_SET : c);
				}
				else
				{
					query.setParameterList(name, c.size() == 0 ? ZERO_SET : c, valueType.getHibernateType());
				}
			}
			else
			{
				if (debugEnabled)
				{
					logParam.debug("Binding '"+value+"' to parameter :"+name);
				}
				if (valueType == null)
				{
					query.setParameter(name, value);
				}
				else
				{
					query.setParameter(name, value, valueType.getHibernateType());
				}
			}
		}
	}

}
