/*
	$Id: ClientDefaultSetting.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ClientDefaultSettingData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;


/**
	This class is used to represent a setting for
	specific client application.

	@author Nicklas
	@version 2.0
*/
public class ClientDefaultSetting
	extends Setting<ClientDefaultSettingData>
{
	/**
		The type of item represented by this class.
		@see Item#CLIENTDEFAULTSETTING
		@see #getType()
	*/
	public static final Item TYPE = Item.CLIENTDEFAULTSETTING;

	/**
		This filter will only return items if the logged in user has
		generic read permission to clients.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = 
		new QueryRuntimeFilterFactory.ChildFilter(TYPE, Item.CLIENT);

	/**
		Create a new <code>ClientDefaultSetting</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param client The <code>Client</code> which this should be a default setting for
		@param name The name of the setting
		@param value The value of the setting
		@return The new <code>ClientDefaultSetting</code> item
		@throws BaseException If there is an error
		@see Client#newDefaultSetting(String,String)
	*/
	public static ClientDefaultSetting getNew(DbControl dc, Client client, String name, String value)
		throws BaseException
	{
		ClientDefaultSetting cds = dc.newItem(ClientDefaultSetting.class);
		cds.setClient(client);
		cds.setName(name);
		cds.setValue(value);
		return cds;
	}

	/**
		Get a <code>ClientDefaultSetting</code> item when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>ClientDefaultSetting</code> item
		@throws ItemNotFoundException If an item with the specified 
			ID is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static ClientDefaultSetting getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		ClientDefaultSetting cds = dc.loadItem(ClientDefaultSetting.class, id);
		if (cds == null) throw new ItemNotFoundException("ClientDefaultSetting[id="+id+"]");
		return cds;
	}

	/**
		Get a query object configured to retrieve 
		default settings for the specified client application.
		@param client The client to retreive settings for, null is allowed if
			the logged in user has generic READ permission for clients in which case
			all settings will be returned
		@return An {@link ItemQuery} object
		@see Client#getDefaultSettings()
	*/
	public static ItemQuery<ClientDefaultSetting> getQuery(Client client)
	{
		ItemQuery<ClientDefaultSetting> query = null;
		if (client != null)
		{
			query = new ItemQuery<ClientDefaultSetting>(ClientDefaultSetting.class, null);
			query.restrictPermanent(
				Restrictions.eq(
					Hql.property("client"), 
					Hql.entity(client)
				)
			);
		}
		else
		{
			query = new ItemQuery<ClientDefaultSetting>(ClientDefaultSetting.class, RUNTIME_FILTER);
		}
		return query;
	}

	ClientDefaultSetting(ClientDefaultSettingData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		READ permission is granted if the logged in user has READ permission to the
		client application. WRITE, DELETE and CREATE permissions are granted if the logged 
		in user have WRITE permission to the client application of this setting.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		Client c = getClient();
		if (c.hasPermission(Permission.READ))
		{
			granted |= Permission.grant(Permission.READ);
		}
		if (c.hasPermission(Permission.WRITE))
		{
			granted |= Permission.grant(Permission.WRITE, Permission.DELETE, Permission.CREATE);
		}
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/**
		Get the client application this setting is valid for.
		@return The <code>Client</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission
		@throws BaseException If there is another error
	*/
	public Client getClient()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Client.class, getData().getClient());
	}
	
	/**
		Set the client application this setting is valid for. Can only be set 
		on a new item.
		@throws InvalidDataException If the client is null
	*/
	void setClient(Client client)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (client == null) throw new InvalidUseOfNullException("client");
		getData().setClient(client.getData());
	}
}

