/*
	$Id: LabeledExtract.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.data.LabeledExtractData;

import java.util.Set;
import java.util.HashSet;

/**
	This class is used to represent labeled extract items. A labeled extract
	is an {@link Extract} that has been marked with a {@link Label}.
	It is also possible to create a labled extract by combining
	other labeled extracts, or as a standalone item with no reference to an
	extract or other labeled extracts.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class LabeledExtract
	extends MeasuredBioMaterial<LabeledExtractData>
{

	/**
		The type of item represented by this class.
		@see Item#LABELEDEXTRACT
		@see #getType()
	*/
	public static final Item TYPE = Item.LABELEDEXTRACT;
	
	/**
		Create a new <code>LabeledExtract</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param label The labeling compound used for labeling the extract
		@return The new <code>LabeledExtract</code> item
		@throws BaseException If there is an error
	*/
	public static LabeledExtract getNew(DbControl dc, Label label)
		throws BaseException
	{
		LabeledExtract le = dc.newItem(LabeledExtract.class);
		le.setName("New labeled extract");
		le.setLabel(label);
		return le;
	}

	/**
		Get a <code>LabeledExtract</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>LabeledExtract</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static LabeledExtract getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		LabeledExtract le = dc.loadItem(LabeledExtract.class, id);
		if (le == null) throw new ItemNotFoundException("LabeledExtract[id="+id+"]");
		return le;
	}

	/**
		Get a query that returns labeled extracts.
		@return An {@link ItemQuery} object.
	*/
	public static ItemQuery<LabeledExtract> getQuery()
	{
		return new ItemQuery<LabeledExtract>(LabeledExtract.class);
	}
	
	
	LabeledExtract(LabeledExtractData labeledExtractData)
	{
		super(labeledExtractData);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		Get the extract or pooled labeled extracts.
	*/
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		try
		{
			if (isPooled())
			{
				annotatable.addAll(getCreationEvent().getSources().list(getDbControl()));
			}
			else if (getData().getParent() != null)
			{
				annotatable.add(getExtract());
			}
		}
		catch (PermissionDeniedException ex)
		{}
		return annotatable;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check that:
		<ul>
		<li>no {@link Hybridization}:s has been created from this item
		<li>no {@link LabeledExtract}:s has been created from this item
		</ul>
	*/
	public boolean isUsed()
		throws BaseException
	{
		return super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link Hybridization}:s created from this labeled extract
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_SOURCEEVENTS_FOR_BIOMATERIAL", "bme.hybridization.id");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				JOIN bme.sources src
				WHERE index(src) = :bioMaterial
			*/
		query.setEntity("bioMaterial", this.getData());
		addUsingItems(using, Item.HYBRIDIZATION, query);
		return using;
	}
	// -------------------------------------------

	/**
		Get the {@link Label} compound used for labeling the extract.
		@return A <code>Label</code> object
		@throws PermissionDeniedException If the logged in user doesn't have read
			permission for the label
		@throws BaseException If there is another error
	*/
	public Label getLabel()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Label.class, getData().getLabel());
	}

	/**
		Set the {@link Label} that was used to label the extract.
		@param label The new <code>Label</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this labeled extract or use permission for the label
		@throws InvalidDataException If the label is null
		@throws BaseException If there is another error
	*/
	public void setLabel(Label label)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (label == null) throw new InvalidUseOfNullException("label");
		label.checkPermission(Permission.USE);
		getData().setLabel(label.getData());
	}

	/**
		Get the {@link Extract} that is the parent of this labeled extract.
		@return An <code>Extract</code> object or null if this is a pooled or
			standalone labeled extract
		@throws PermissionDeniedException If the logged in user doesn't have read
			permission for the extract
		@throws BaseException If there is another error
	*/
	public Extract getExtract()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Extract.class, getData().getParent());
	}

	/**
		Set the {@link Extract} item that is the the parent
		of this labeled extract.
		@param extract The new <code>Extract</code> item
	 	@param usedQuantity Used quantity of the extract. 
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this labeled extract or use permission for the extract
		@throws InvalidDataException If this is a pooled labeled extract
		@throws BaseException If there is another error
	*/
	public void setExtract(Extract extract, Float usedQuantity)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (extract != null) extract.checkPermission(Permission.USE);
		if (isPooled())
		{
			throw new InvalidDataException("A pooled labeled extract can't have a parent extract");
		}
		getData().setParent(extract == null ? null : extract.getData());
		getCreationEvent().setSource(extract, usedQuantity);
	}

	/**
		Get a query that returns all hybridizations created from this 
		labeled extract.
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<Hybridization> getHybridizations()
	{
		ItemQuery<Hybridization> query = Hybridization.getQuery();
		query.joinPermanent(Hql.innerJoin("creationEvent.sources", Item.LABELEDEXTRACT.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.index(Item.LABELEDEXTRACT.getAlias(), null),
				Hql.entity(this)
			)
		);
		return query;
	}

}

