/*
	$Id: ParameterValuesImpl.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.plugin.ParameterValues;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

/**
	An implementation of the <code>ParameterValues</code> interface that
	lets a plugin write the job's configuration. 
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
class ParameterValuesImpl
	implements ParameterValues
{
	/**
		The id of the job or configuration.
	*/
	private final int id;

	private final Job job;
	private final PluginConfiguration configuration;
	private final PluginDefinition plugin;
	
	/**
		The configuration should only be writeable when configuring the job,
		not while executing a job. 
	*/
	private final boolean writeProtected;
	/**
		Preloaded parameter values, if the write protected flag is set.
	*/
	private final Map<String, List<?>> parameters;

	private final Map<String, ParameterType> parameterTypes;
	
	/**
		Empty parameter values
	*/
	ParameterValuesImpl(PluginDefinition plugin)
	{
		this.id = 0;
		this.writeProtected = true;
		this.parameters = new HashMap<String, List<?>>();
		this.parameterTypes = new HashMap<String, ParameterType>();
		this.job = null;
		this.configuration = null;
		this.plugin = plugin;
	}
	
	ParameterValuesImpl(Job job, boolean writeProtected)
		throws BaseException
	{
		this.writeProtected = writeProtected;
		// Preload all parameter values, so the client may close the connection
		// while executing jobs
		parameters = new HashMap<String, List<?>>();
		parameterTypes = new HashMap<String, ParameterType>();
		for (String name : job.getParameterNames())
		{
			parameters.put(name, job.getParameterValues(name));
		}
		this.id = job.getId();
		this.configuration = job.getPluginConfiguration();
		this.plugin = job.getPluginDefinition();
		this.job = job;
	}
	
	ParameterValuesImpl(PluginConfiguration config, int version, boolean writeProtected)
		throws BaseException
	{
		this.writeProtected = writeProtected;
		// Preload all parameter values, so the client may close the connection
		// while executing jobs
		parameters = new HashMap<String, List<?>>();
		parameterTypes = new HashMap<String, ParameterType>();
		for (String name : config.getParameterNames(version))
		{
			parameters.put(name, config.getParameterValues(name, version));
		}
		this.id = config.getId();
		this.configuration = config;
		this.plugin = config.getPluginDefinition();
		this.job = null;
	}
		
	/*
		From the ParameterValues interface (plugin package)
		---------------------------------------------------
	*/
	public Job getJob()
	{
		return job;
	}
	public PluginDefinition getPluginDefinition()
	{
		return plugin;
	}
	public PluginConfiguration getPluginConfiguration()
	{
		return configuration;
	}
	public int getId()
	{
		return id;
	}
	
	public Object getValue(String name)
		throws PermissionDeniedException, BaseException
	{
		List<?> values = parameters.get(name);
		if (values != null && values.size() >= 1)
		{
			return values.get(0);
		}
		else
		{
			return null;
		}
	}
		
	public List<?> getValues(String name)
		throws PermissionDeniedException, BaseException
	{
		return parameters.get(name);
	}
	
	public <T> void setValue(String name, ParameterType<T> type, T value)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (writeProtected) 
		{
			throw new PermissionDeniedException(Permission.WRITE, "Parameter: "+name);
		}
		if (name == null) throw new InvalidUseOfNullException("name");
		if (type == null) throw new InvalidUseOfNullException("type[name=" + name + "]");
		if (value == null && type.getNotNull()) value = type.getDefaultValue();
		type.validate(name, value);
		List<T> l = new ArrayList<T>(1);
		l.add(value);
		parameters.put(name, l);
		parameterTypes.put(name, type);
	}
		
	@SuppressWarnings("unchecked")
	public <T> void setValues(String name, ParameterType<T> type, List<T> values)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (writeProtected) 
		{
			throw new PermissionDeniedException(Permission.WRITE, "Parameter: "+name);
		}
		if (name == null) throw new InvalidUseOfNullException("name");
		if (type == null) throw new InvalidUseOfNullException("type[name=" + name + "]");
		if ((values == null || values.size() == 0) && type.getNotNull() && type.getDefaultValue() != null)
		{
			values = Arrays.asList(type.getDefaultValue());
		}
		type.validate(name, values);
		parameters.put(name, values);
		parameterTypes.put(name, type);
	}
	// -------------------------------------------
	
	boolean isWriteProtected()
	{
		return writeProtected;
	}
	
	void saveParameters(Job job, Map<String, PluginParameter<?>> moreInfo)
	{
		if (writeProtected) 
		{
			throw new PermissionDeniedException(Permission.WRITE, "Job parameters");
		}
		for (String name : parameterTypes.keySet())
		{
			PluginParameter<?> pp = moreInfo == null ? null : moreInfo.get(name);
			if (pp == null)
			{
				job.setParameterValuesInternal(name, null, null,
					parameterTypes.get(name), parameters.get(name), false);
			}
			else
			{
				job.setParameterValuesInternal(name, pp.getLabel(), pp.getDescription(), 
					parameterTypes.get(name), parameters.get(name), false);
			}
		}
	}

	void saveParameters(PluginConfiguration config, Map<String, PluginParameter<?>> moreInfo)
	{
		if (writeProtected) 
		{
			throw new PermissionDeniedException(Permission.WRITE, "Configuration parameters");
		}
		for (String name : parameterTypes.keySet())
		{
			PluginParameter<?> pp = moreInfo == null ? null : moreInfo.get(name);
			if (pp == null)
			{
				config.setParameterValuesInternal(name, null, null, 
					parameterTypes.get(name), parameters.get(name), false);
			}
			else
			{
				config.setParameterValuesInternal(name, pp.getLabel(), pp.getDescription(), 
					parameterTypes.get(name), parameters.get(name), false);
			}
		}
	}
	
}
