/*
  $Id: PluginType.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.PluginTypeData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.util.JarClassLoader;

/**

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class PluginType
	extends BasicItem<PluginTypeData>
	implements Nameable, Removable
{

	/**
		The type of item represented by this class.
		@see Item#PLUGINTYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.PLUGINTYPE;

	/**
		The maximum length of the interface name that can be stored in the
		database. Check the length against this value before calling the
		{@link #setInterfaceName(String)} method to avoid exceptions.
	*/
	public static final int MAX_INTERFACENAME_LENGTH = PluginTypeData.MAX_INTERFACENAME_LENGTH;
	
	/**
		The maximum length of the jar path name that can be stored in the
		database.
	*/
	public static final int MAX_JARPATH_LENGTH = PluginTypeData.MAX_JARPATH_LENGTH;

	/**
		Create a new <code>PluginType</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param interfaceName The name of an interface that a plugin
			must implement to be of this type
		@param jarPath Optional path to the jar file which contains the
			interface class, if missing the interface must be in the classpath
		@return The new <code>PluginDefinition</code> item
		@throws BaseException If there is an error
	*/
	public static PluginType getNew(DbControl dc, String interfaceName, String jarPath)
		throws BaseException
	{
		PluginType pt = dc.newItem(PluginType.class);
		pt.setName(interfaceName);
		pt.loadInterfaceInformation(jarPath, interfaceName);
		return pt;
	}

	/**
		Get a <code>PluginType</code> object when you know the ID.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The id of the item to load.
		@return The <code>PluginType</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static PluginType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		PluginType pt = dc.loadItem(PluginType.class, id);
		if (pt == null) throw new ItemNotFoundException("PluginType[id="+id+"]");
		return pt;
	}
	
	/**
		Get a <code>PluginType</code> object when you know the interface
		name it represents.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param interfaceName The interface name of the plugin type
		@return The <code>PluginType</code> item
		@throws ItemNotFoundException If an item with the specified 
			interface name is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static PluginType getByInterfaceName(DbControl dc, String interfaceName)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		if (interfaceName != null) interfaceName = interfaceName.trim();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(
			dc.getHibernateSession(), "GET_PLUGINTYPE_FOR_INTERFACENAME");
		query.setParameter("interfaceName", interfaceName);
		PluginTypeData pt = HibernateUtil.loadData(PluginTypeData.class, query);
		if (pt == null) throw new ItemNotFoundException("PluginType[interfaceName="+interfaceName+"]");
		return dc.getItem(PluginType.class, pt);
	}
	
	/**
		Get a query configured to retrieve plugin types.
		
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<PluginType> getQuery()
	{
		return new ItemQuery<PluginType>(PluginType.class);
	}
	
	/**
		Creates a plugin type item.
		
		@param data the data
	*/
	PluginType(PluginTypeData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}

	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}

	public String getDescription()
	{
		return getData().getDescription();
	}

	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		All users get READ access.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		granted |= Permission.grant(Permission.READ);
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/*
		From the Removable interface
		-------------------------------------------
	*/
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	// -------------------------------------------

	/**
		Get the class name of the Java class that implements this plugin. The
		specified class must implement the {@link net.sf.basedb.core.plugin.Plugin}
		interface.
		@return The class name for this plugin
	*/
	public String getInterfaceName()
	{
		return getData().getInterfaceName();
	}
	/**
		The class name cannot be changed. Used internally only.
	*/
	private void setInterfaceName(String interfaceName)
		throws InvalidDataException
	{
		getData().setInterfaceName(StringUtil.setNotNullString(interfaceName, "interfaceName", MAX_INTERFACENAME_LENGTH));
	}

	/**
		Get the path to the JAR file that contains the plugin class file.
		If this value is null, the plugin must be on the classpath.
	*/
	public String getJarPath()
	{
		return getData().getJarPath();
	}
	/**
		The jar path cannot be changed. Used internally only.
	*/
	private void setJarPath(String jarPath)
		throws InvalidDataException
	{
		getData().setJarPath(StringUtil.setNullableString(jarPath, "jarPath", MAX_JARPATH_LENGTH));
	}
	
	/**
	 	@param jarPath Path to the jar file where the interface is located. 
	 		Null can be used only if the interface is located in the server's classpath. 
	 	@param interfaceName The name of an interface that a plugin
			must implement to be of this type. 
	 	@throws InvalidDataException If the found class not really is an interface, or 
	 		if the found interface doesn't extend from {@link net.sf.basedb.core.plugin.Plugin}
	 	@throws BaseException If there is some other error.

	*/
	public void loadInterfaceInformation(String jarPath, String interfaceName)
		throws InvalidDataException, BaseException
	{
		try
		{
			Class c = null;
			if (jarPath == null)
			{
				c = Class.forName(interfaceName);
			}
			else
			{
				JarClassLoader.unload(jarPath);
				ClassLoader cl = JarClassLoader.getInstance(jarPath, Application.autoUnloadPlugins());
				c = cl.loadClass(interfaceName);
			}
			if (!c.isInterface())
			{
				throw new InvalidDataException("Class is not an interface: "+interfaceName);
			}
			if (!Plugin.class.isAssignableFrom(c))
			{
				throw new InvalidDataException("Interface does not extend net.sf.basedb.core.plugin.Plugin: "+interfaceName);
			}
			setInterfaceName(interfaceName);
			setJarPath(jarPath);
		}
		catch (Exception ex)
		{
			throw new BaseException("Could not instantiate plugin interface: "+interfaceName+"", ex);
		}
	}
	
	/**
		Get a query that returns the plugin definitions which implements
		this type.
		@see PluginDefinition#getQuery()
	*/
	public ItemQuery<PluginDefinition> getPlugins()
	{
		ItemQuery<PluginDefinition> query = PluginDefinition.getQuery();
		query.joinPermanent(Hql.innerJoin("pluginTypes", Item.PLUGINTYPE.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.PLUGINTYPE.getAlias()),
				Hql.entity(this)
			)
		);
		return query;
	}
}
