/*
	$Id: PositionExtraValueBatcher.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.hibernate.ExecuteUpdateWork;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Selects;

import java.util.List;
import java.util.LinkedList;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
	Batcher class for per-position extra values. This batcher can only be used
	for a new bioassayset that hasn't yet been committed to the database.
	<p>
	Code example: Multiply the mean of intensities of channel 1 and 2
<pre class="code">
// Get source bioassayset and extra value type
DbControl dc = ...
BioAssaySet source = ...
ExtraValueType type = ...

// Create new transformation and bioassayset to hold the
// new intensities, we use the same data cube and layer as the source
Transformation t = source.newTransformation();
dc.saveItem(t);
BioAssaySet bas = t.newProduct(null, null, true);
dc.saveItem(bas);

// Create a query that selects the data of
// the source bioassayset
DynamicSpotQuery query = source.getSpotData();

// Multiply the mean of the intensities
Expression meanCh1 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(1)));
Expression meanCh2 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(2)));
Expression product = Expressions.multiply(meanCh1, meanCh2);

// Select the position and product, and group by position
query.select(Dynamic.select(VirtualColumn.POSITION));
query.select(Selects.expression(product, "product"));
query.group(Dynamic.column(VirtualColumn.POSITION));

// Execute the query, and use the batcher to insert the new values
SpotExtraValueBatcher&lt;Float&gt; batcher = bas.getSpotExtraValueBatcher(Float.class, type);
DynamicResultIterator spots = query.iterate(dc);
int position = spots.getIndex(VirtualColumn.POSITION.getName());
int prod = spots.getIndex("product");
while (spots.hasNext())
{
   SqlResult r = spots.next();
   batcher.insert(r.getInt(position), r.getFloat(prod));
}

// Close and commit
batcher.close();
dc.commit();
</pre>

	
	@author Nicklas
	@version 2.0
	@see BioAssaySet#getPositionExtraValueBatcher(Class, ExtraValueType, Job)
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class PositionExtraValueBatcher<I>
	extends AbstractBatcher
{

	/**
		The extra value object which links the the extra value type.
	*/
	private final ExtraValue extraValue;
	
	/**
		The type of values to insert.
	*/
	private final Type valueType;

	/**
		The sql type code of the value to insert.
		@see java.sql.Types
	*/
	private final int sqlType;
	
	/**
		The number of bytes used by a single row.
	*/
	private final long bytesPerRow;
	
	/**
		The number of queued inserts. Used to check when a flush
		is needed.
	*/
	private int queuedInsertCount = 0;
	
	/**
		The total number of inserts done by this batcher.
	*/
	private int totalInsertCount = 0;

	/**
		The number of bytes this batcher has added to the experiment.
	*/
	private long bytes = 0;


	/**
		The statement that inserts spot data.
<pre class="code">
INSERT INTO Dynamic#PositionExtraXxx (cube, extra, position, value)
VALUES (cube, extra, ?, ?)
</pre>
	*/
	private PreparedStatement extraSql = null;
	
	/**
	 	The SQL string for the {@link #extraSql} statement.
	*/
	private String extraSqlStatement;


	/**
		Create a new spot batcher for a bioassayset. It is assumed that
		the calling code has checked that the bioassayset is allowed
		to receive more data (ie. it's data cube layer hasn't been committed 
		to the database).
	*/
	PositionExtraValueBatcher(DbControl dc, ExtraValue extraValue)
	{
		super();
		this.extraValue = extraValue;
		this.bytesPerRow = extraValue.getVirtualTable().getBytesPerRow(extraValue.getRawDataType());
		this.valueType = extraValue.getExtraValueType().getValueType();
		this.sqlType = valueType.getSQLType();
		setDbControl(dc);
	}

	/*
		From the Batcher interface
		-------------------------------------------
	*/
	public void flush()
		throws BaseException
	{
		try
		{
			if (extraSql != null) extraSql.executeBatch();
		}
		catch (SQLException ex)
		{
			logSql.error(extraSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		queuedInsertCount = 0;
	}
	// -------------------------------------------
	/*
		From the AbstractBatcher class
		-------------------------------------------
	*/
	/**
		Update disk usage and close open SQL statements
	*/
	void onBeforeClose()
		throws BaseException
	{
		extraValue.getDataCubeExtraValue().addBytes(bytes);
		extraValue.setNumValues(totalInsertCount);
		try
		{
			if (extraSql != null) 
			{
				extraSql.close();
				extraSql = null;
			}
		}
		catch (SQLException ex)
		{
			logSql.error(ex.getMessage(), ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
		}
		if (totalInsertCount > 0 && !getDbControl().isRolledBack())
		{
			String catalog = Application.getDynamicCatalog();
			String schema = Application.getDynamicSchema();
			String table = extraValue.getVirtualTable().getTableName(extraValue.getVirtualDb());
			analyzeTable(catalog, schema, table);
		}
	}
	// -------------------------------------------

	/**
		Insert an extra value for a given position coordinate.
		
		@param position The position coordinate
		@param value The extra value
	 	@throws InvalidDataException If some data is corrupted
		@throws BaseException If there is another error
	*/
	public void insert(int position, I value)
		throws InvalidDataException, BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		
		if (sqlType == Types.FLOAT && value != null && ((Float)value).isNaN())
		{
			// Convert Float.NaN to NULL
			value = null;
		}

		// TODO - should we verify max position?
		try
		{
			if (extraSql == null) buildSpotSql();
			extraSql.setInt(1, position);
			extraSql.setObject(2, value, sqlType);
			extraSql.addBatch();
			bytes += bytesPerRow + (valueType == Type.STRING && value != null ? value.toString().length() : 0);
			totalInsertCount++;
		}
		catch (SQLException ex)
		{
			logSql.error(extraSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		if (++queuedInsertCount == getBatchSize())
		{
			flush();
		}
	}
	
	/**
		Insert position extra values using a query. The query MUST select data in
		the following order: position, extraValue
		<p>
		Internally this method converts the query into a 
		<code>INSERT INTO ... SELECT ...</code> query, making it very fast,
		since the data for each position doesn't have to be retreived, processed and
		then inserted one row at a time. It is possible to call
		this method multiple times with the same or different query. 
		<p>
		Note! This method can't be called for string extra values. This is
		because the length of each string must be added to the disk usage. So 
		they must be processed individually in any case. Use the other 
		{@link #insert(int, Object)} method. 
		<p>
		Here is an alternative to the example above:
		<pre class="code">
// Get source bioassayset and extra value type
DbControl dc = ...
BioAssaySet source = ...
ExtraValueType type = ...

// Create new transformation and bioassayset to hold the
// new intensities, we use the same data cube and layer as the source
Transformation t = source.newTransformation();
dc.saveItem(t);
BioAssaySet bas = t.newProduct(null, null, true);
dc.saveItem(bas);


// Create a query that selects the data of
// the source bioassayset
DynamicQuery query = source.getSpotData();

// Multiply the mean of the intensities
Expression meanCh1 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(1)));
Expression meanCh2 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(2)));
Expression product = Expressions.multiply(meanCh1, meanCh2);

// Select the position and product, and group by position
query.select(Dynamic.select(VirtualColumn.POSITION));
query.select(Selects.expression(product, "product"));
query.group(Dynamic.column(VirtualColumn.POSITION));

// Execute the query, and use the batcher to insert the new values
SpotExtraValueBatcher&lt;Float&gt; batcher = bas.getSpotExtraValueBatcher(Float.class, type);
batcher.insert(query);

// Close and commit
batcher.close();
dc.commit();
</pre>
		
		@param query The query that selects the data to be inserted
		@return int The number of rows inserted
		@throws BaseException If there is an error
	*/
	public int insert(final AbstractSqlQuery query)
		throws BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		if (query == null) throw new InvalidUseOfNullException("query");
		
		if (valueType == Type.STRING) 
		{
			throw new BaseException("Operation not supported for string extra value: " + extraValue);
			// TODO - query.iterate() and then insert(position, value) instead ??
		}
		
		query.resetTemporary();
		
		// Add static values cubeNo and extraNo to the selection
		query.temporarySelect(
			Selects.expression(
				Expressions.integer(extraValue.getDataCubeNo()), 
				VirtualColumn.CUBE.getName()
			)
		);
		query.temporarySelect(
			Selects.expression(
				Expressions.integer(extraValue.getExtraNo()), 
				VirtualColumn.EXTRA.getName()
			)
		);
		
		String selectSql = query.getMainQuery(getDbControl(), true);
		
		// Holds the names of the parameters in the query in the order they appear
		final List<String> parameterOrder = new LinkedList<String>();
		
		// Parse out named parameters
		selectSql = query.parseParameters(selectSql, parameterOrder);
		String insertSql = buildInsertSelectSql(selectSql);

		int rowsInserted = 0;
		try
		{
			rowsInserted = HibernateUtil.doJdbcWork(getDbControl().getHibernateSession(), 
					new ExecuteUpdateWork(getSessionControl(), insertSql)
					{
						@Override
						public void setParameters(PreparedStatement ps)
							throws SQLException
						{
							query.setParameters(ps, parameterOrder);
						}
					}
				);
			bytes += rowsInserted * bytesPerRow;
			totalInsertCount += rowsInserted;
		}
		catch (SQLException ex)
		{
			logSql.error(insertSql, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		return rowsInserted;

	}
	/**
		Builds the insert SQL statement. Also creates the Dynamic#PositionExtraXxx table
		if it doesn't exists.
		@see BatchUtil#buildInsertSql(VirtualDb, VirtualTable, VirtualColumn[], Object[])
	*/
	private void buildSpotSql()
		throws SQLException, BaseException
	{
		RawDataType rdt = extraValue.getRawDataType();
		VirtualDb vdb = extraValue.getVirtualDb();
		VirtualTable vt = extraValue.getVirtualTable();
		if (!vdb.hasTable(vt))
		{
			vdb.createTables(vt);
		}
		short cube = extraValue.getDataCubeNo();
		short extra = extraValue.getExtraNo();
		extraSqlStatement = BatchUtil.buildInsertSql(
			vdb,
			vt,
			vt.getColumns(rdt), 
			new Object[] {cube, extra}
		);
		extraSql = HibernateUtil.getConnection(getDbControl().getHibernateSession()).prepareStatement(extraSqlStatement);
	}

	/**
		Build the INSERT INTO ... SELECT ... statment. Also creates the 
		Dynamic#PositionExtraXxx table if it doesn't exists.
		@param selectSql The SELECT part of the query
		@see BatchUtil#buildInsertSelectSql(VirtualDb, VirtualTable, VirtualColumn[], String)
	*/
	private String buildInsertSelectSql(String selectSql)
		throws BaseException
	{
		RawDataType rdt = extraValue.getRawDataType();
		VirtualDb vdb = extraValue.getVirtualDb();
		VirtualTable vt = extraValue.getVirtualTable();
		if (!vdb.hasTable(vt))
		{
			vdb.createTables(vt);
		}
		String sql = BatchUtil.buildInsertSelectSql(vdb, vt, vt.getColumns(rdt), selectSql);
		return sql;
	}
}
