/*
	$Id: ReporterBatcher.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ReporterData;

import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.TreeSet;

/**
	Batcher class for reporters.

	@author Samuel, Nicklas
	@version 2.0
*/
public class ReporterBatcher
	extends BasicBatcher<ReporterData>
{
	
	/**
	 	A string to hold the name of the source, 
	 	used to update the reporters.
	 */
	private String updateSource = null;

	/**
		Create a new batcher for reporters.
		
		@param dc The DbControl to use for database access and 
			permission checking
		@return A new instance of ReporterBatcher.
		@throws BaseException If there is an error
	*/
	public static ReporterBatcher getNew(DbControl dc)
		throws BaseException
	{
		ReporterBatcher rb = new ReporterBatcher(dc);
		rb.initPermissions(0, 0);
		rb.checkPermission(Permission.READ);		
		return rb;
	}
	
	/**
		The extended properties for reporters
		@see ExtendedProperties
	*/
	private final List<ExtendedProperty> extendedProperties;

	/**
		A set containing all batched but not yet flushed
		external reporter id:s. Used by the exists method so
		a client application can avoid exception if trying to 
		insert duplicate records.
	*/
	private final Set<String> batchedExternalIds;
	
	ReporterBatcher(DbControl dc)
		throws BaseException
	{
		super(dc.getHibernateSession(), ReporterData.class, null);
		extendedProperties = ExtendedProperties.getProperties("ReporterData");
		batchedExternalIds = HibernateUtil.getDbEngine().caseInsensitiveComparison() ?
			new TreeSet<String>(String.CASE_INSENSITIVE_ORDER) : new HashSet<String>();
		setDbControl(dc);
	}

	/*
		From the BasicBatcher class
		-------------------------------------------
	*/
	public Item getType()
	{
		return Item.REPORTER;
	}

	/**
		Insert and add the external reporter id to the internal cache.
	*/
	public void insert(ReporterData data)
		throws BaseException
	{
		super.insert(data);
		batchedExternalIds.add(data.getExternalId());
	}
	
	/**
		Flush and clear the internal cache of external reporter id:s.
	*/
	public void flushInsert()
		throws DatabaseException
	{
		super.flushInsert();
		batchedExternalIds.clear();
	}
	
	/**
		Validate all properties for the reporter object:
		<ul>
		<li>The data object is not null
		<li>The name or external id is not null or too long
		<li>The description or symbol is not too long
		<li>All of the extendable properties using the {@link ExtendedProperty#validateValue(Object)}
			method
		</ul>
	*/
	public void validate(ReporterData data)
		throws InvalidDataException, InvalidUseOfNullException
	{
		if (data == null) throw new InvalidUseOfNullException("data");
		
		data.setName(StringUtil.setNotNullString(data.getName(), 
			"name", ReporterData.MAX_NAME_LENGTH));
		data.setDescription(StringUtil.setNullableString(data.getDescription(), 
			"description", ReporterData.MAX_DESCRIPTION_LENGTH));
		data.setExternalId(StringUtil.setNotNullString(data.getExternalId(), 
			"externalId", ReporterData.MAX_EXTERNAL_ID_LENGTH));
		data.setSymbol(StringUtil.setNullableString(data.getSymbol(), 
			"symbol", ReporterData.MAX_SYMBOL_LENGTH));

		// Validate ExtendableData interface
		if (extendedProperties != null)
		{
			for (ExtendedProperty ep : extendedProperties)
			{
				ep.validateValue(data.getExtended(ep.getName()));
			}
		}
	}
	
	/**
		Set the last update date to today's date and 
		change the last source of update.
	*/
	void onBeforeCommit(ReporterData data, Transactional.Action action)
		throws BaseException
	{
		setPropertyValue(data, "lastUpdate", new Date());
		if (updateSource == null)
		{
			int editorId = this.getDbControl().getSessionControl().getLoggedInUserId();
			User editor = User.getById(this.getDbControl(), editorId);
			updateSource = "User: " + editor.getName();
		}
		setPropertyValue(data, "lastSource", updateSource);
	}
	// -------------------------------------------

	/**
		A query to find reporters by the external id.
	*/
	private org.hibernate.Query findReporter;
	
	/**
		Check if a reporter with a given external id exists in the database or
		has been added to the batch queue of insertion.
		@param externalId The external id of the reporter
		@param checkInsertQueue If the insert queue should be checked or not
		@param checkDb If the database should be checked or not
		@throws BaseException If there is an error
	*/
	public boolean exists(String externalId, boolean checkInsertQueue, boolean checkDb)
		throws BaseException
	{
		updateLastAccess();
		if (externalId == null) return false;
		externalId = externalId.trim();
		if (checkInsertQueue && batchedExternalIds.contains(externalId)) 
		{
			return true;
		}
		if (checkDb)
		{
			org.hibernate.StatelessSession session = getDbControl().getStatelessSession();
			if (findReporter == null)
			{
				findReporter = HibernateUtil.getPredefinedQuery(session, "GET_REPORTER_FOR_EXTERNAL_ID");
			}
			findReporter.setString("externalId", externalId);
			ReporterData r = HibernateUtil.loadData(ReporterData.class, findReporter);
			return r != null;
		}
		else
		{
			return false;
		}
	}
	
	public boolean isInInsertQueue(String externalId)
	{
		if (externalId == null) return false;
		return batchedExternalIds.contains(externalId.trim());
	}
	
	/**
		Get a reporter when you know the external id. This method only checks the database,
		not the insertion batch queue. It is possible that the {@link #exists(String, boolean, boolean)}
		method returns true, but this method throws an {@link ItemNotFoundException}.

		@param externalId The external id of the reporter
		@return A <code>ReporterData</code> object
		@throws ItemNotFoundException If a reporter with the given id
			isn't found
		@throws BaseException If there is another error
	*/
	public ReporterData getByExternalId(String externalId)
		throws ItemNotFoundException, BaseException
	{
		return getByExternalId(externalId, true);
	}
	
	
	/**
		Get a reporter when you know the external id. This method only checks the database,
		not the insertion batch queue. It is possible that the {@link #exists(String, boolean, boolean)}
		method returns true, but this method throws an {@link ItemNotFoundException}.
	
		@param externalId The external id of the reporter
		@param errorIfNotFound If an ItemNotFoundException should be thrown if the
			reporter isn't found
		@return A <code>ReporterData</code> object, or null if not found
		@throws ItemNotFoundException If a reporter with the given id
			isn't found and the errorIfNotFound parameter is true
		@throws BaseException If there is another error
	*/
	public ReporterData getByExternalId(String externalId, boolean errorIfNotFound)
	{
		updateLastAccess();
		if (externalId != null) externalId = externalId.trim();
		if (findReporter == null)
		{
			org.hibernate.StatelessSession session = getDbControl().getStatelessSession();
			findReporter = HibernateUtil.getPredefinedQuery(session, "GET_REPORTER_FOR_EXTERNAL_ID");
		}
		findReporter.setString("externalId", externalId);
		ReporterData r = HibernateUtil.loadData(ReporterData.class, findReporter);
		if (r == null && errorIfNotFound) throw new ItemNotFoundException("Reporter[externalId="+externalId+"]");
		return r;
	}

	/**
	 	Set the source that was used when updating the reporters. 
	 	This must be set before committing this batcher otherwise the column 
	 	for last update source will be set to n/a. 	 	   
	 	@param source The source to be set. The format of the source string 
	 		should be like: "[item_type]:[item_name]"	 
	 	@throws StringTooLongException If the source is longer that
	 		{@link ReporterData#MAX_LAST_SOURCE_LENGTH}
	 */
	public void setUpdateSource(String source)
	{
		this.updateSource = StringUtil.setNullableString(source, "source", ReporterData.MAX_LAST_SOURCE_LENGTH); 
	}
}
