/*
	$Id: RequestInformation.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.List;
import java.util.Map;
import java.util.HashMap;


/**
	Get information about the parameters the plugin needs for a particular request.
	See {@link net.sf.basedb.core.PluginRequest} for more information.

	@author Enell, Nicklas
	@version 2.0
	@see net.sf.basedb.core.PluginRequest
	@base.modified $Date$
*/
public class RequestInformation
{
	private final String command;
	
	private final String description;
	
	private final String title;
	
	private final List<PluginParameter<?>> parameters;

	private final String jspPage;
	
	private Map<String, PluginParameter<?>> parameterMap;
	
	public RequestInformation(String command, String title, String description, List<PluginParameter<?>> parameters)
	{
		this(command, title, description, parameters, null);
	}
	
	public RequestInformation(String command, String title, String description, List<PluginParameter<?>> parameters, String jspPage)
	{
		this.command = command;
		this.description = description;
		this.parameters = parameters;
		this.title = title;
		this.jspPage = jspPage;
	}

	/**
		Get the command this request information object is valid for.
		@return The name of the command
	*/
	public String getCommand()
	{
		return command;	
	}

	/**
		Get the title of the request. A client application can use this text as
		a page header.
	 	@return A String object or null if not specified. 
	*/
	public String getTitle()
	{
		return title;
	}
	
	/**
		Get a description of the request. A client application can use this information
		as a help text describing the current configuration page.
		@return A String object or null if not specified.
	*/
	public String getDescription()
	{
		return description;
	}

	/**
		Get a list containing the information about the parameters
		needed for the current request.
		@return A {@link java.util.List} with {@link net.sf.basedb.core.PluginParameter}:s
	*/
	public List<PluginParameter<?>> getParameters()
	{
		return parameters;
	}
	
	/**
		Get the plugin parameter with the specified name.
	 	@param name Name of the plugin parameter. 
		@return The PluginParameter objec or null if no parameter with that name
			exists
	*/
	public PluginParameter<?> getParameter(String name)
	{
		if (parameterMap == null)
		{
			createParameterMap();
		}
		return parameterMap.get(name);
	}
	
	/**
		The name of the jsp page to use for displaying the input form for
		requesting parameters. This is only used by the web client application 
		(or compatible). It is recommended that a plugin do not depend on the JSP
		page for parameter input.
		<p>
		This method should return the name of the JSP page only, not the path.
		It must be located in a subdirectory to the <code>/plugins</code> directory,
		where the subdirectory is given by the package name of the plugin.
		Ie. the plugin <code>net.sf.basedb.plugins.APlugin</code> would
		have all it's JSP pages located in the <code>/plugins/net/sf/basedb/plugins</code>
		directory.
		<p>
		TODO - the JSP page must conform to certain rules, ie. names of input fields, etc.
		create other documentation for that and link to it from here
		
		@return The name of the JSP page or null if this request doesn't 
			require a JSP page.
	*/
	public String getJspPage()
	{
		return jspPage;
	}

	/**
		Create a hash from parameter name to PluginParameter objects
		so that the {@link #getParameter(String)} method doesn't have
		to look in the list.
	*/
	private void createParameterMap()
	{
		parameterMap = new HashMap<String, PluginParameter<?>>();
		if (parameters != null)
		{
			for (PluginParameter<?> param : parameters)
			{
				parameterMap.put(param.getName(), param);
			}
		}
	}
	
}
