/*
	$Id: Software.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

import net.sf.basedb.core.data.SoftwareData;

/**
	This class is used to represent individual software items
	and information about them. In the current version of BASE
	the only type of software we keep information about is feature extraction software.

	@author enell
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class Software
	extends CommonItem<SoftwareData>
{
	/**
		The type of item represented by this class.
		
		@see Item#SOFTWARE
		@see #getType()
		@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	*/
	private static final Item TYPE = Item.SOFTWARE;

	/**
		The maximum length of the version variable that can be stored in the
		database. Check the length against this value before calling the
		{@link #setVersionString(String)} method to avoid
		exceptions.
	*/
	public static final int MAX_VERSIONSTRING_LENGTH = SoftwareData.MAX_VERSIONSTRING_LENGTH;
	/**
		Create a new <code>Software</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param softwareType The type of this software
		@return The new <code>Software</code> item
		@throws BaseException This exception is thrown if there is another
			error
	*/
	public static Software getNew(DbControl dc, SoftwareType softwareType)
		throws BaseException
	{
		Software software = dc.newItem(Software.class);
		software.setName("New software");
		software.setSoftwareType(softwareType);
		return software;
	}

	/**
		Get a <code>Software</code> item when you know the ID.
	
		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>Software</code> item
		@throws ItemNotFoundException If an item with the specified ID is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static Software getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Software sw = dc.loadItem(Software.class, id);
		if (sw == null) throw new ItemNotFoundException("Software[ID="+id+"]");
		return sw;
	}
	
	/**
		Get a query configured to retrieve software.
		
		@return An {@link ItemQuery} object
	 	@throws BaseException If getting the query fails. 
	*/
	public static ItemQuery<Software> getQuery() 
		throws BaseException
	{
		return new ItemQuery<Software>(Software.class);
	}
	
	/**
		Creates a new software item.
		
		@param data the data
	*/
	Software(SoftwareData data)
	{
		super(data);
	}
	
	/*
		From the Identibiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>a RawBioAssay was created with this software
		</ul>
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_SOFTWARE", "count(*)");
		/*
			SELECT {1}
			FROM RawBioAssay rba
			WHERE rba.software = :software
		*/
		query.setEntity("software", this.getData());
		return HibernateUtil.loadData(Long.class, query) > 0 || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link RawBioAssay}:s created with this software
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_SOFTWARE", "rba.id");
			/*
				SELECT {1}
				FROM RawBioAssay rba
				WHERE rba.software = :software
			*/
		query.setEntity("software", this.getData());
		addUsingItems(using, Item.RAWBIOASSAY, query);
		return using;
	}
	// -------------------------------------------

	/**
		Get the associated {@link SoftwareType} item.
		All <code>Software</code> items must have a type.
		@return The <code>SoftwareType</code> item
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#READ} permission for the software type
		@throws BaseException If there is another error
	*/
	public SoftwareType getSoftwareType()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(SoftwareType.class, getData().getSoftwareType());
	}

	/**
		Set the {@link SoftwareType} of this <code>Software</code> item.
		This parameter mustn't be <code>null</code>.
		@param softwareType The new <code>SoftwareType</code>
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#WRITE} permission for the software or
			{@link Permission#USE} permission for the software type
		@throws InvalidUseOfNullException If software type is null
	*/
	public void setSoftwareType(SoftwareType softwareType)
		throws PermissionDeniedException, InvalidUseOfNullException
	{
		checkPermission(Permission.WRITE);
		if (softwareType == null) throw new InvalidUseOfNullException("softwareType");
		softwareType.checkPermission(Permission.USE);
		getData().setSoftwareType(softwareType.getData());
	}
	
	/**
		Get the versionstring of this <code>Software</code> item.
		@return A string with the version of this item, or null if not known
	*/
	public String getVersionString()
	{
		return getData().getVersionString();
	}

	/**
		Set the versionstring for this <code>Software</code> item. The value
		must not be longer than the value specified by
		the {@link #MAX_VERSIONSTRING_LENGTH} constant.
		
		@param versionString The new version for this item, or null if not known
		@throws StringTooLongException If the version is longer then {@link #MAX_VERSIONSTRING_LENGTH}
		@throws PermissionDeniedException If the logged in user don't have write permission on this item
	*/
	public void setVersionString(String versionString) 
		throws StringTooLongException, PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setVersionString(StringUtil.setNullableString(versionString, "versionString", MAX_VERSIONSTRING_LENGTH));
	}
}
