/**
	$Id: MySQLEngine.java 4557 2008-10-03 10:28:40Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.dbengine;

import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
	Database engine for My SQL.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-10-03 12:28:40 +0200 (Fri, 03 Oct 2008) $
*/
public class MySQLEngine
	extends AbstractDbEngine
	implements DbEngine
{
	/**
		Create MySQLEngine.
	*/
	public MySQLEngine()
	{}
	
	/*
		From the DbEngine interface
		-------------------------------------------
	*/
	/**
		Always false.
	*/
	public boolean useThetaJoin()
	{
		return false;
	}
	
	/**
	 	Generate <code>ALTER TABLE &lt;catalog&gt;.&lt;table&gt; ADD [UNIQUE] 
	 	INDEX &lt;name&gt; (&lt;columns&gt;)</code>.
		The schema parameter is not supported.
	*/
	public String getCreateIndexSql(String catalog, String schema, String table, String name, Set<String> columns, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("ALTER TABLE ");
		if (catalog != null) sql.append(getQuotedName(catalog)).append(".");
		sql.append(getQuotedName(table)).append(" ADD ");
		if (unique) sql.append("UNIQUE ");
		sql.append("INDEX ").append(getQuotedName(name)).append(" (");
		int i = 0;
		for (String column : columns)
		{
			if (i > 0) sql.append(", ");
			sql.append(getQuotedName(column));
			i++;
		}
		sql.append(")");
		return sql.toString();
	}
	
	/**
	 	Generate <code>DROP INDEX &lt;name&gt; ON &lt;catalog&gt;.&lt;table&gt;</code>.
		The schema parameter is not supported.
	*/
	public String getDropIndexSql(String catalog, String schema, String table, String name, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("DROP INDEX ").append(getQuotedName(name)).append(" ON ");
		if (catalog != null) sql.append(getQuotedName(catalog)).append(".");
		sql.append(getQuotedName(table));
		return sql.toString();
	}

	/**
		Return true. 
		TODO - we should verify that this does no harm -- switch back to false if
		it does
	*/
	public boolean analyzeAfterBatchOperation()
	{
		return true;
	}

	/**
		Generate <code>OPTIMIZE TABLE &lt;catalog&gt;.&lt;table&gt;</code>.
		The schema parameter is not supported.
	*/
	public String getOptimizeTableSql(String catalog, String schema, String table)
	{
		catalog = catalog == null ? "" : getQuotedName(catalog) + ".";
		return "OPTIMIZE TABLE " + catalog + getQuotedName(table);
	}
	
	/**
		Generate <code>ANALYZE TABLE &lt;catalog&gt;.&lt;table&gt;</code>.
		The schema parameter is not supported.
	*/
	public String getAnalyzeTableSql(String catalog, String schema, String table)
	{
		catalog = catalog == null ? "" : getQuotedName(catalog) + ".";
		return "ANALYZE TABLE " + catalog + getQuotedName(table);
	}

	/**
		Replace CREATE TABLE with CREATE TABLE IF NOT EXISTS...
		@since 2.6
	*/
	public String makeSafeCreateTable(String sql, String catalog, String schema, String table)
	{
		Pattern p = Pattern.compile("CREATE TABLE", Pattern.CASE_INSENSITIVE);
		Matcher m = p.matcher(sql);
		sql = m.replaceFirst("CREATE TABLE IF NOT EXISTS");
		return sql;
	}
	
	/**
		Returns TRUE.
		@since 2.4
	*/
	public boolean caseInsensitiveComparison()
	{
		return true;
	}
		
	
	/**
		Returns FALSE.
	*/
	public boolean supportsColumnAliasInWhere()
	{
		return false;
	}
	/**
		Returns TRUE.
	*/
	public boolean supportColumnAliasInOrderBy()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	public boolean supportColumnAliasInGroupBy()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	public boolean supportColumnAliasInHaving()
	{
		return true;
	}
	/**
		Return <code>LOG(&lt;value&gt;)</code>.
	*/
	public String ln(String value)
	{
		return "LOG("+ value + ")";
	}
	/**
		Return <code>&lt;value&gt; RLIKE &lt;regexp&gt;</code>.
		@since 2.8
	*/
	public String rlike(String value, String regexp)
	{
		return value + " RLIKE " + regexp;
	}
	
	/**
		Return <code>varchar(length) CHARACTER SET utf8 COLLATE utf8_bin</code>.
		@since 2.9
	 */
	public String getCaseSensitiveVarchar(int length)
	{
		return "varchar(" + length + ") CHARACTER SET utf8 COLLATE utf8_bin";
	}
	// -------------------------------------------
	
	/**
		Put quotes (`) around the name.
	*/
	private String getQuotedName(String name)
	{
		return "`" + name + "`";
	}
}
