/*
	$Id: InteractivePlugin.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.plugin;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.SessionControl;

import java.util.Set;

/**
	An interactive plugin is a {@link Plugin} that supports interactive
	configuration in a user session. What this means is that a client application
	may ask the plugin for the parameters it needs using the 
	{@link #getRequestInformation(GuiContext, String)} method. The client then displays some sort
	of interactive interface that asks the user to supply values to those
	parameters and then calls the {@link #configure(GuiContext, Request, Response)} method to
	let the plugin verify and store those parameters in the database.

	<p>
	When configuring a plugin or job the following sequence
	is followed:

	<ol>
	<li>Create a plugin object, using the default constructor
	<li>Call the {@link #init(SessionControl, ParameterValues, ParameterValues)} method
		which gives the plugin access to it's current configuration
		and job-specific parameters
	<li>Call the {@link #getRequestInformation(GuiContext, String)} method with a command
		value of {@link Request#COMMAND_CONFIGURE_PLUGIN} or {@link Request#COMMAND_CONFIGURE_JOB}
	<li>Call the {@link #configure(GuiContext, Request, Response)} method
	<li>If the return status is {@link Response.Status#CONTINUE} the previous
		two steps will be repeated with the a command
	<li>Call the {@link #done()} method
	<li>After <code>done</code> has been called, the plugin instance is not
	</ol>

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public interface InteractivePlugin
	extends Plugin
{
	
	/**
		Get a set containing all items that the plugin handles. Ie. if the 
		plugin imports reporters, return a set containing {@link Item#REPORTER}.
		This information is used by client applications to put the plugin
		in the proper place in the user interface.
		@return A <code>Set</code> containing {@link Item}:s, or null
			if the plugin is not concerned about items
	*/
	public Set<GuiContext> getGuiContexts();

	/**
		Check if the plugin can be used on the specified object. The item is either
		a <code>BasicItem</code> or <code>BasicData</code> object, or null if called from
		a list context. A client application should only call this method for 
		contexts matching one of
		the contexts in the set returned by the {@link #getGuiContexts()}
		method. For {@link GuiContext.Type#ITEM} contexts an item
		of the correct type should be passed as a parameter.
		<p>
		The plug-in can response with a null value indicating success,
		a warning-level message or throw an exception to indicate an
		error-level message. Client application should normally only
		show error-level messages to users, unless the user has 
		requested to see warning messages.
		<p>
		NOTE! Before BASE 2.4 all messages were treated as error-level
		messages.
		
		@param context The current context of the client application,
			it is one of the values found in set returned by {@link #getGuiContexts()}
		@param item The currently active item, it's type should match the 
			{@link GuiContext#getItem()} type, or null if the context is a list
			context
		@return <code>Null</code> if the plugin can use that item, or 
			a warning-level message explaining why the plugin can't be used
		@throws RuntimeException With an error-leve message explaining
			why the plug-in can't be used
	*/
	public String isInContext(GuiContext context, Object item);
	
	/**
		This method will return the {@link RequestInformation} for a given command, i.e. 
		the list of parameters and some nice help text.
		
		@param context The current context of the client application,
			it is one of the values found in set returned by {@link #getGuiContexts()}
		@param command The command
		@return The <code>RequestInformation</code> for the command
		@throws BaseException if there is an error
	*/
	public RequestInformation getRequestInformation(GuiContext context, String command)
		throws BaseException;

	/**
		Configure the plugin. Hopefully the client is supplying values for the
		parameters specified by {@link #getRequestInformation(GuiContext, String)}.

		@param context The current context of the client application,
			it is one of the values found in set returned by {@link #getGuiContexts()}
		@param request Request object with the command and parameters
		@param response Response object in for the plugin to response
			through
	*/
	public void configure(GuiContext context, Request request, Response response);

}
