/**
	$Id: LocalSignalReceiver.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
	A signal receiver implementation that can receive signals from the
	local virtual machine only. The corresponding transporter class
	is {@link LocalSignalTransporter}.
	
	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class LocalSignalReceiver
	extends AbstractSignalReceiver
{
	/**
		Log signals processing.
	*/
	private static final org.apache.log4j.Logger logger = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.signal.LocalSignalReceiver");

	/**
		Holds all registered signal receivers.
	*/
	private static Map<String, LocalSignalReceiver> receivers =
		Collections.synchronizedMap(new HashMap<String, LocalSignalReceiver>());

	/**
		Get a signal receiver with a given ID.
		@param id The ID of the signal receiver
		@return A signal receiver, or null if no receiver with the given
			ID exists.
	*/
	public static LocalSignalReceiver getSignalReceiver(String id)
	{
		return receivers.get(id);
	}
	
	private String receiverId;
	
	/**
		Create a new local signal receiver. Before it can be used it must
		be initialised with {@link #init(String)}.
	*/
	public LocalSignalReceiver()
	{}

	/*
		From the SignalReceiver interface
		-------------------------------------------
	*/
	/**
		Initialise the signal receiver. The given parameter becomes the ID of
		the receiver. When this method return the signal receiver is ready to 
		register signal handlers and receive signals.
	*/
	@Override
	public void init(String params)
	{
		super.init(params);
		this.receiverId = params;
		receivers.put(receiverId, this);
	}
	
	@Override
	public void close(int wait)
	{
		super.close(wait);
		receivers.remove(receiverId);
	}
	
	/**
		@return {@link LocalSignalTransporter}
	*/
	public Class<? extends SignalTransporter> getSignalTransporterClass()
	{
		return LocalSignalTransporter.class;
	}
	// -------------------------------------------
	
	/**
		Send the signal to a registered handler. If no handler with the
		given ID is found this method does nothing. The signal will be
		processed by the handler in the current thread.
		@param handlerId The ID of a registered handler
		@param signal The signal to send
	*/
	public void send(String handlerId, Signal signal)
	{
		logger.info("Receiving signal " + signal.getId() + " to " + handlerId);
		SignalHandler handler = super.getSignalHandler(handlerId);
		if (handler != null) handler.handleSignal(signal);
	}

}
