/**
	$Id: AnnotationUtil.java 4630 2008-11-07 08:36:37Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.Scan;
import net.sf.basedb.util.filter.Filter;

/**
	Utility class for working with annotations.

	@author Nicklas
	@version 2.9
	@base.modified $Date: 2008-11-07 09:36:37 +0100 (Fri, 07 Nov 2008) $
*/
public class AnnotationUtil
{
	
	/**
		Get all annotatable parent items for a given item. A {@link Filter}
		may be used to filter the returned collection. If a filter
		is specified only parent items for which the {@link Filter#evaluate(Object)}
		method returns true are included in the returned collection. Note
		that all parent items are loaded before the filter is evaluated
		and that the evaluation order is undefined.
		
		@param dc The DbControl to use for database access
		@param item The item to find the parents for
		@param filter An optional filter that can be used to filter 
			which parents to include
		@return A set containing the parent items
	*/
	public static Set<Annotatable> getAllAnnotatableParentItems(DbControl dc, 
		Annotatable item, Filter<Annotatable> filter)
	{
		Set<AnnotatableWrapper> processed = new HashSet<AnnotatableWrapper>();
		Queue<AnnotatableWrapper> unprocessed = new LinkedList<AnnotatableWrapper>();
		unprocessed.add(new AnnotatableWrapper(item, 0));

		// As long as we have unprocessed entries...
		while (!unprocessed.isEmpty())
		{
			AnnotatableWrapper toProcess = unprocessed.remove();
			// ...mark it as processed...
			if (processed.add(toProcess))
			{
				// ...and queue the parents items for processing...
				unprocessed.addAll(toProcess.getAnnotatableParentWrappers());
			}
		}
		
		// All parents are now loaded...
		Set<Annotatable> allParents = new HashSet<Annotatable>(processed.size());
		for (AnnotatableWrapper wrapper : processed)
		{
			// ...copy all parents that passes the filter
			Annotatable parent = wrapper.getAnnotatable();
			if (filter == null || filter.evaluate(parent)) allParents.add(parent);
		}
		// Do not include the root item in the result
		allParents.remove(item);
		return allParents;
	}

	
	/**
		This wrapper class is needed because we need to keep
		track of the {@link RawBioAssay#getArrayNum()} property 
		until we have loaded the Labeled Extracts of a Hybridization.
		<p>
		The 'normal' value of arrayNum is 0, which means that it is ignored.
		When we reach a {@link RawBioAssay} item, we use the arrayNum on
		that. The array num is kept as we move up to Scan and Hybridization.
		When the Hybridization is reached we use the arrayNum to call
		{@link Hybridization#getAnnotatableParents(int)} so that we only
		get labeled extracts that are linked with the correct array num.
	*/
	static class AnnotatableWrapper
	{
		/**
	 		Get the array num to use. This method returns:
	 		
	 		<ul>
	 		<li>{@link RawBioAssay#getArrayNum()} if the item is a raw bioassay
	 		<li>'arrayNum' if the items is a Scan or Hybridization
	 		<li>0 in all other cases.
		*/
		private static int getArrayNum(Annotatable item, int arrayNum)
		{
			if (item instanceof RawBioAssay)
			{
				return ((RawBioAssay)item).getArrayNum();
			}
			else if (item instanceof Scan || item instanceof Hybridization)
			{
				return arrayNum;
			}
			return 0;
		}
		
		private final Annotatable item;
		private final int arrayNum;

		AnnotatableWrapper(Annotatable item, int arrayNum)
		{
			this.item = item;
			this.arrayNum = getArrayNum(item, arrayNum);
		}
		
		/**
			The annotatable that we are wrapping.
		*/
		Annotatable getAnnotatable()
		{
			return item;
		}
		
		/**
			Get all annotatable parents wrapped inside AnnotatableWrapper:s
		*/
		Set<AnnotatableWrapper> getAnnotatableParentWrappers() 
			throws BaseException
		{
			Set<Annotatable> parents = null;
			Set<AnnotatableWrapper> wrappedParents = Collections.emptySet();
			if (item instanceof Hybridization)
			{
				parents = ((Hybridization)item).getAnnotatableParents(arrayNum);
			}
			else
			{
				parents = item.getAnnotatableParents();
			}
			if (parents != null)
			{
				wrappedParents = new HashSet<AnnotatableWrapper>(parents.size());
				for (Annotatable parent : parents)
				{
					wrappedParents.add(new AnnotatableWrapper(parent, arrayNum));
				}
			}
			return wrappedParents;
		}
		
		/**
			A wrapper is equal to another if they references the same item
			and have the same arrayNum.
		*/
		@Override
		public boolean equals(Object o)
		{
			if (o == this) return true;
			AnnotatableWrapper other = (AnnotatableWrapper)o;
			return this.item.equals(other.item) && this.arrayNum == other.arrayNum;
		}

		@Override
		public int hashCode()
		{
			return item.hashCode() + arrayNum;
		}
	}
}
