/*
	$Id: ClassLocal.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Map;
import java.util.HashMap;
import java.util.Collections;

/**

	This class is a similar to the JDK {@link ThreadLocal} class,
	but stores one object per class instead of one object per thread.
	<p>
	The main usage for this class is to allow a superclass
	to hold one object for each subclass that is extending it.

	@author Nicklas
	@version 2.0
*/
public class ClassLocal<E>
{

	private Map<Class,E> objects;

	/**
		Create a new <code>ClassLocal</code> object.
	*/
	public ClassLocal()
	{
		objects = Collections.synchronizedMap(new HashMap<Class,E>());
	}

	/**
		Get the object associated with the class of
		the specified instance.
		@param instance An object instance of the class
		@see ThreadLocal#get()
	*/
	public E get(Object instance)
	{
		Class clazz = instance.getClass();
		E object = null;
		synchronized (objects)
		{
			object = objects.get(clazz);
			if ((object == null) && !objects.containsKey(clazz))
			{
				object = initialValue(instance);
				set(clazz, object);
			}
		}
		return object;
	}
	/**
		Store an object and associate it with the class
		of the specified instance.
		@param instance An object instance of the class
		@param object The object to store
		@see ThreadLocal#set(Object)
	*/
	public void set(Object instance, E object)
	{
		objects.put(instance.getClass(), object);
	}
	/**
		Subclasses may override this to initialize the object
		stored for that class.
		@param instance An object instance of the class
		@return The object to associate with the class
	*/
	protected E initialValue(Object instance)
	{
		return null;
	}
}
