/*
	$Id: IncludeExcludeFilter.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicResultIterator;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.query.Aggregations;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.SqlQuery;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.core.query.WhenStatement;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**

	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class IncludeExcludeFilter
	implements DynamicFilter
{
	private final Restriction filter;
	private final int includeLimit;
	private final int excludeLimit;
	private final Map<Integer, Integer> positionCount; 
	
	public IncludeExcludeFilter(DbControl dc, BioAssaySet bas, List<BioAssay> bioAssays,
		Restriction filter, Integer includeLimit, Integer excludeLimit)
	{
		this.filter = filter;
		this.includeLimit = includeLimit == null ? Integer.MAX_VALUE : includeLimit.intValue();
		this.excludeLimit = excludeLimit== null ? 0 : excludeLimit.intValue();
		this.positionCount = new HashMap<Integer, Integer>();
		
		// Select position and count of columns for that position
		Select cnt = Selects.expression(
			Aggregations.count(Dynamic.column(VirtualColumn.COLUMN), true), 
			"cnt"
		);
		
		DynamicSpotQuery query = bas.getSpotData();
		query.select(Dynamic.select(VirtualColumn.POSITION));
		query.select(cnt);
		
		// Restrict on selected bioassays
		// Create child bioassays and filter
		if (bioAssays != null && bioAssays.size() > 0)
		{
			Expression[] selectedColumns = new Expression[bioAssays.size()];
			for (int i = 0; i < selectedColumns.length; ++i)
			{
				BioAssay ba = bioAssays.get(i);
				selectedColumns[i] = Expressions.integer(ba.getDataCubeColumnNo());
			}
			Expression column = Dynamic.column(VirtualColumn.COLUMN);
			query.restrict(Restrictions.in(column, selectedColumns));
		}
		
		// Add restrictions and grouping
		query.restrict(filter);
		query.group(Dynamic.column(VirtualColumn.POSITION));
		if (this.excludeLimit > 0)
		{
			query.having(
				Restrictions.gteq(
						Expressions.selected(cnt),
						Expressions.integer(this.excludeLimit)
				)
			);
		}

		// Execure query and fill hashmap
		DynamicResultIterator result = query.iterate(dc);
		try
		{
			while (result.hasNext())
			{
				SqlResult data = result.next();
				int position = data.getInt(1);
				int count = data.getInt(2);
				positionCount.put(position, count);
			}
		}
		catch (SQLException ex)
		{
			throw new BaseException(ex);
		}
	}
	
	/**
		We need the position which is in column 2 (already selected). If the limits
		leave a gap we also need to know if a spot matches the filter or not.
	*/
	public void configureQuery(SqlQuery query)
	{
		if (includeLimit > excludeLimit)
		{
			// Case statment that returns 1 if spot matches filter, 0 otherwise
			WhenStatement matches = new WhenStatement(filter, Expressions.integer(1));
			query.select(
				Selects.expression(
					Expressions.caseWhen(
						Expressions.integer(0), 
						new WhenStatement[] { matches } 
					)
				, "matches")
			);
		}
	}
	
	/**
		Always true.
	*/
	public boolean useIncludeSpot()
	{
		return true;
	}
	
	/**
		Check if a spot should be included in the filtered output or not.
		
		@param data The current data row to check
		@return TRUE if the spot passed the filter, FALSE otherwise
	*/
	public boolean includeSpot(SqlResult data)
		throws SQLException
	{
		int position = data.getInt(2);
		Integer count = positionCount.get(position);
		boolean match = false;
		if (count != null)
		{
			if (count < excludeLimit)
			{
				match = false;
			}
			else if (count >= includeLimit)
			{
				match = true;
			}
			else // excludeLimit <= count < includeLimit
			{
				match = data.getInt(3) == 1;
			}
		}
		return match;
	}
}
