/*
	$Id: XMLUtil.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.BaseException;

import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.URL;
import java.nio.charset.Charset;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.DocType;
import org.jdom.input.SAXBuilder;
import org.jdom.JDOMException;

import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
	This class contains some useful methods for handling
	XML documents.

	@author Nicklas
	@version 2.0
	@base.modified $Date$
*/
public class XMLUtil
{

	/**
		Log core events.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core");

	/**
		The parser class we are using.
	*/
	private static final String xmlParserClass = "org.apache.xerces.parsers.SAXParser";

	/**
		Convert a <code>Document</code> to xml. The encoding is set to UTF-8.
		@param dom The document
		@return The XML
	*/
	public static String toXml(Document dom)
	{
		XMLOutputter out = new XMLOutputter(Format.getCompactFormat().setEncoding("UTF-8"));
		return out.outputString(dom);
	}
	
	/**
		Create a new <code>Document</code> with root element and System id
		declaration.
		@param rootElement The name of the root element tag
		@param systemId The system id in the DOCTYPE section
	 	@return a {@link org.jdom.Document} object  
	*/
	public static Document createDom(String rootElement, String systemId)
	{
		return new Document(new Element(rootElement), new DocType(rootElement, systemId));
	}
	
	/**
	 	@deprecated Use {@link #getValidatedXML(InputStream, URL, String)} instead
	*/
	public static Document getValidatedXML(InputStream is, URL dtdFile)
			throws JDOMException, IOException
	{
		return getValidatedXML(is, dtdFile, null);
	}
	
	/**
		Load and validate an XML file against a DTD, and return it as a
		<code>Document</code> object. The path to the files should
		be given as a URL, not a filesystem path. For a file on the classpath
		it is easiest done by the {@link Class#getResource(String)} method, ie.
		<code>XMLUtil.class.getResource("/net/sf/basedb/core/the-xml-file.xml")</code>.
		
		@param xmlFile The URL to the XML file
		@param dtdFile The URL to the DTD used for validation, the DTD must be encoded in ISO-8859-1
		@return A <code>Document</code> object
	 	@throws JDOMException If validation of XML fails. 
	 	@throws IOException If reading the input stream fails.
	*/
	public static Document getValidatedXml(URL xmlFile, URL dtdFile)	
			throws JDOMException, IOException
	{		
		InputStream is = xmlFile.openStream();		
		Document xmlDoc = getValidatedXML(is, dtdFile, xmlFile.toString()); 
		return xmlDoc;
	}
	
	/**
		Parse and validate a string containing XML against a DTD, and return it as a
		<code>Document</code> object. The path to the DTD should
		be given as a URL, not a filesystem path. For a file on the classpath
		it is easiest done by the {@link Class#getResource(String)} method, ie.
		<code>XMLUtil.class.getResource("/net/sf/basedb/core/the-dtd-file.dtd")</code>.
		@param xml The string containing the XML
		@param dtdFile The URL to the DTD used for validation, the DTD must be encoded in ISO-8859-1
		@return A <code>Document</code> object
	 	@throws JDOMException If validation of XML fails. 
	 	@throws IOException If reading the string fails.
	*/
	public static Document getValidatedXml(String xml, URL dtdFile)
		throws JDOMException, IOException
	{
		SAXBuilder sax = new SAXBuilder(xmlParserClass, true);
		Validator validator = new Validator(null, dtdFile);
		// The entity resolver lets us find the correct DTD for validating
		sax.setEntityResolver(validator);
		// So we can throw our own exception messages if there is a parse error
		sax.setErrorHandler(validator);
		StringReader sr = new StringReader(xml);
		Document dom = sax.build(sr);
		sr.close();
		return dom;
	}

	/**
	 	Validate an <code>InputStream</code> against a DTD file 
	 	and return it as a <code>Document</code>
	 	
	 	@param is The inputstream to the XML
	 	@param dtdFile The URL to the DTD used for validation, the DTD must be encoded in ISO-8859-1
	 	@param filename The filename of the original XML data, used for error reporting only;
	 		use null if the filename is not known
	 	@return A <code>Document</code> object
	 	@throws JDOMException If validation of the XML fails. 
	 	@throws IOException If reading the input stream fails.
	 	@since 2.4
	*/
	public static Document getValidatedXML(InputStream is, URL dtdFile, String filename)
			throws JDOMException, IOException
	{
		SAXBuilder sax = new SAXBuilder(xmlParserClass, true);
		Validator validator = new Validator(filename, dtdFile);
		// The entity resolver lets us find the correct DTD for validating
		sax.setEntityResolver(validator);
		// So we can throw our own exception messages if there is a parse error
		sax.setErrorHandler(validator);
		Document dom = sax.build(is);
		is.close();
		return dom;
	}
	
	/**
		Validate an XML file using XML schemas. 
		
		@param xmlFile The URL pointing to the XML file to load
		@param schemaFiles An array of schema file declarations. The array must
			contain pairs of elements, the first one is the namespace name, and the
			second element is the URL to the schema definition file
	 	@return A <code>Document</code> object
	 	@throws JDOMException If validation of the XML fails
	 	@throws IOException If reading the input stream fails
		@since 2.7
	*/
	public static Document getSchemaValidatedXML(URL xmlFile, String... schemaFiles)	
		throws JDOMException, IOException
	{		
		InputStream is = xmlFile.openStream();
		Document xmlDoc = getSchemaValidatedXML(is, xmlFile.toString(), schemaFiles); 
		return xmlDoc;
	}
	
	/**
		Validate an XML input stream using XML schemas. 
		
		@param in The inputstream to the XML
		@param filename The filename of the original XML data, used for error reporting only;
	 		use null if the filename is not known
		@param schemaFiles An array of schema file declarations. The array must
			contain pairs of elements, the first one is the namespace name, and the
			second element is the URL to the schema definition file
	 	@return A <code>Document</code> object
	 	@throws JDOMException If validation of the XML fails
	 	@throws IOException If reading the input stream fails
		@since 2.7
	*/
	public static Document getSchemaValidatedXML(InputStream in, String filename, String... schemaFiles)
		throws JDOMException, IOException
	{
		SAXBuilder sax = new SAXBuilder(xmlParserClass, true);
		Validator validator = new Validator(filename, null);
		// So we can throw our own exception messages if there is a parse error
		sax.setErrorHandler(validator);
		
		sax.setFeature("http://apache.org/xml/features/validation/schema", true);
		sax.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);
		if (schemaFiles != null)
		{
			if (schemaFiles.length % 2 != 0)
			{
				throw new ArrayIndexOutOfBoundsException("schemeFiles must contain an even number of elements");
			}
			StringBuilder sb = new StringBuilder();
			for (String schemaFile : schemaFiles)
			{
				sb.append(schemaFile).append(" ");
			}
			sax.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", 
					sb.toString());
		}
		
		Document dom = sax.build(in);
		in.close();
		return dom;

	}
	
	/**
		Get the value of an attribute as an integer. If the attribute is missing or
		doesn't contain an integer, the default value is returned.
		@param el The node element which contains the attribute
		@param attribute The name of the attribute
		@param defaultValue The default value to return
	 	@return an integer. 
	*/
	public static int getIntAttribute(Element el, String attribute, int defaultValue)
	{
		Attribute attr = el.getAttribute(attribute);
		int value = defaultValue;
		if (attr != null)
		{
			try
			{
				value = attr.getIntValue();
			}
			catch (Exception ex)
			{}
		}
		return value;
	}

	/**
		Get the value of an attribute as a boolean. If the attribute is missing or
		doesn't contain an boolean value, the default value is returned.
		@param el The node element which contains the attribute
		@param attribute The name of the attribute
		@param defaultValue The default value to return
	 	@return TRUE if the attribute has a true value, FALSE if it has a false value
	*/
	public static boolean getBooleanAttribute(Element el, String attribute, boolean defaultValue)
	{
		Attribute attr = el.getAttribute(attribute);
		boolean value = defaultValue;
		if (attr != null)
		{
			try
			{
				value = attr.getBooleanValue();
			}
			catch (Exception ex)
			{}
		}
		return value;
	}

	/**
		This class is used to handle callbacks from the XML parser.
		We need this for throwing better error messages and to
		find the DTD.
	*/
	private static class Validator
		implements ErrorHandler, EntityResolver
	{

		private final String parsedFile;
		private final URL dtdFile;
		
		/**
			Create a new instance of this class.
		*/
		private Validator(String parsedFile, URL dtdFile)
		{
			this.parsedFile = parsedFile;
			this.dtdFile = dtdFile;
		}

		/*
			From the org.xml.sax.ErrorHandler class
			-------------------------------------------
		*/
		/**
			Errors thrown with a detailed error message.
		*/
		public void error(SAXParseException exception)
			throws SAXException
		{
			fatalError(exception);
		}
		/**
			Fatal errors thrown with a detailed error message.
		*/
		public void fatalError(SAXParseException exception)
			throws SAXException
		{
			throw new SAXException(exception.getMessage()+" at line "+
				exception.getLineNumber()+" in file '"+parsedFile+"'");
		}
		/**
			Ignore warnings
		*/
		public void warning(SAXParseException exception)
			throws SAXException
		{}

		/*
			From the org.xml.sax.EntityResolver class
			-------------------------------------------
		*/
		/**
			This method is called when the parser wants a reference to the
			DTD file. If the DTD cannot be loaded null is returned.
			<p>
			Actually, we would like to throw an exception but the interface
			doesn't allow us to!
		*/
		public InputSource resolveEntity(String publicId, String systemId)
		{
			InputStream is = null;
			try
			{
				is = dtdFile == null ? null : dtdFile.openStream();
			}
			catch (IOException ex)
			{
				log.info("Exception while resolving XML entity: publicId = '"+
					publicId+"'; systemId = '"+systemId+"'; dtdFile = '"+dtdFile+"'");
				throw new BaseException(ex);
			}
			return is == null ? null : new InputSource(new InputStreamReader(is, Charset.forName("ISO-8859-1")));
		}
	}

}
