/**
	$Id: ExperimentOverview.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.ArrayBatch;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.ArraySlide;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSet;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.Image;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Label;
import net.sf.basedb.core.LabeledExtract;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformFileType;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.Presets;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Scan;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;

/**
 	Get a complete overview of an experiment including the raw data, biomaterials,
 	array designs and more that have been used in it. The structure will also be
 	validated to make sure the experiment makes sense. Validation rules include:
 	<ul>
 	<li>Checking for missing items
 	<li>Checking that annotations doesn't have invalid values or are inherited from
 		non-parent items
 	<li>Checking that the project default values are used
 	<li>And a lot more...
 	</ul>

	Use {@link #getValidators()} to get all validators and {@link #getValidationOptions()}
	to specify if a certain validation rule should be ignored, cause a warning or an 
	error.

	@author Nicklas
	@version 2.2
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class ExperimentOverview
{

	private static Map<String, List<Validator>> validators;

	public static Map<String, List<Validator>> getValidators()
	{
		if (validators == null)
		{
			Map<String, List<Validator>> temp = new LinkedHashMap<String, List<Validator>>();
			temp.put("Project defaults", 
					Arrays.asList(new Validator[] 
					{
						Validator.NONDEFAULT_PLATFORM,
						Validator.NONDEFAULT_RAWDATATYPE, Validator.NONDEFAULT_ARRAYDESIGN, 
						Validator.NONDEFAULT_PROTOCOL, Validator.NONDEFAULT_HARDWARE, 
						Validator.NONDEFAULT_SOFTWARE
					})
				);
			temp.put("Missing items", 
					Arrays.asList(new Validator[] 
					{
						Validator.MISSING_BIOSOURCE, Validator.MISSING_SAMPLE, Validator.MISSING_EXTRACT,
						Validator.MISSING_LABELED_EXTRACT, Validator.MISSING_HYBRIDIZATION, 
						Validator.MISSING_SCAN, Validator.MISSING_ARRAYDESIGN, Validator.MISSING_PROTOCOL, 
						Validator.MISSING_HARDWARE, Validator.MISSING_SOFTWARE,
						Validator.MISSING_ARRAYSLIDE, Validator.MISSING_ARRAYBATCH
					})
				);
			temp.put("Annotations",
					Arrays.asList(new Validator[]
					{
						Validator.MISSING_MIAME_ANNOTATION, Validator.MISSING_FACTORVALUE, 
						Validator.MISSING_PARAMETER, Validator.ANNOTATION_IS_PARAMETER, 
						Validator.ANNOTATION_INVALID_VALUE, Validator.ANNOTATION_INVALID_ITEM,
						Validator.ANNOTATION_INHERIT_FROM_NONPARENT, Validator.ANNOTATION_INHERIT_MULTIPLE
					})
				);
			
			temp.put("Files", 
					Arrays.asList(new Validator[]
					{
						Validator.DATAFILE_INVALID, Validator.DATAFILE_NOTVALIDATED,
						Validator.MISSING_DATAFILE, Validator.DATAFILE_NOTPLATFORM,
						Validator.DATAFILE_INVALID_ITEM
					})
				);
			
			temp.put("Denied access",
					Arrays.asList(new Validator[]
				   {
						Validator.DENIED_BIOSOURCE, Validator.DENIED_SAMPLE, Validator.DENIED_EXTRACT,
						Validator.DENIED_LABEL, Validator.DENIED_HYBRIDIZATION, Validator.DENIED_SCAN,
						Validator.DENIED_ARRAYDESIGN, Validator.DENIED_PLATFORM, Validator.DENIED_PROTOCOL, Validator.DENIED_HARDWARE,
						Validator.DENIED_SOFTWARE, Validator.DENIED_ARRAYSLIDE, Validator.DENIED_ARRAYBATCH
				   })
				);
			
			temp.put("Other",
					Arrays.asList(new Validator[]
					{
						Validator.NONMATHCING_ARRAYDESIGN, Validator.MULTIPLE_ARRAYDESIGNS, 
						Validator.INCORRECT_PROTOCOLTYPE, Validator.INCORRECT_SOFTWARETYPE,
						Validator.INCORRECT_HARDWARETYPE, Validator.INCORRECT_ARRAY_NUM,
						Validator.DUPLICATE_ARRAY_NUM, Validator.INCORRECT_NUM_LABELEDEXTRACTS,
						Validator.NONUNIQUE_LABEL, Validator.DIFFERENT_LABEL, 
						Validator.NONUNIQUE_NAME_FOR_TYPE, Validator.NONUNIQUE_NAME_GLOBAL,
						Validator.CIRCULAR_REFERENCE
					})
				);
			
			validators = Collections.unmodifiableMap(temp);
		}
		return validators;
	}
	
	
	/**
		Get the existing experiment overview for an experiment or create a new
		one if none exist yet. The returned object is cached in the {@link
		SessionControl}:s session settings.
		@param experiment The experiment to get the overview for
	 	@param project Default from this project will be used if new overview need to be created.
		@return An <code>ExperimentOverview</code> object
	*/
	public static ExperimentOverview getOverview(Experiment experiment, Project project)
	{
		String overviewSetting = "experiment.overview." + experiment.getId();
		SessionControl sc = experiment.getSessionControl();
		ExperimentOverview overview = (ExperimentOverview)sc.getSessionSetting(overviewSetting);
		if (overview != null)
		{
			if ((project == null && overview.getProject() != null)
				|| (project != null && !project.equals(overview.getProject())))
			{
				// The project has changed; We need to generate a new overview
				overview = null;
			}
		}
		if (overview == null)
		{
			overview = new ExperimentOverview(experiment, project);
			sc.setSessionSetting(overviewSetting, overview);
		}
		return overview;
	}

	
	private static final String presetsSetting = "experiment.overview.presets";
	
	/**
		Get the validation options presets for the currently logged in user.
		If changes are made to the presets call {@link #saveValidationPresets(SessionControl)}
		to make sure the changes are saved to the database.
		
		@param sc The session control for the logged in user
		@return The validation option presets
	*/
	public static Presets getValidationPresets(SessionControl sc)
	{
		Presets presets = (Presets)sc.getSessionSetting(presetsSetting);
		if (presets == null)
		{
			String xml = sc.getUserDefaultSetting(presetsSetting);
			presets = new Presets();
			if (xml != null)
			{
				presets.loadFrom(xml);
			}
			sc.setSessionSetting(presetsSetting, presets);
		}
		return presets;
	}
	
	/**
		Save the validation option presets for the logged in user. The presets
		are stored as user default settings 
		({@link SessionControl#setUserDefaultSetting(String, String)})
		with the name <code>experiment.overview.presets</code>
		@param sc The session control for the logged in user 
	*/
	public static void saveValidationPresets(SessionControl sc)
	{
		Presets presets = getValidationPresets(sc);
		if (presets != null)
		{
			sc.setUserDefaultSetting(presetsSetting, presets.asXml());
		}
	}
	
	private Experiment experiment;
	private Project project;
	private Node rootNode;
	private ValidationOptions validationOptions;
	private List<Failure> failures;
	private Map<String, Node> allNodes;
	private Map<Item, Set<AnnotationType>> miameAnnotations;
	private Map<Object, Set<Annotatable>> annotatableParents;
	private Map<String, Node> usedHybArrays;
	
	/**
		Create a new overview for the experiment using defaults from 
		the specified project.
	*/
	private ExperimentOverview(Experiment experiment, Project project)
	{
		this.experiment = experiment;
		this.project = project;
		this.validationOptions = 
			new ValidationOptions(getValidationPresets(experiment.getSessionControl()).getDefault());
		DbControl dc = experiment.getDbControl();
		generateOverview(dc);
	}
	
	/**
		Reset the overview, regenerate the entire structure and revalidate
		everything. Use this method only if the structure or information
		in the experiment has changed. Use the {@link #updateFailureCountOnNodes()}
		method instead if only the validation options has changed. That method
		is a lot quicker and doesn't require any database access.

		@param dc The DbControl to use for database access
	*/
	public void reset(DbControl dc)
	{
		this.experiment = Experiment.getById(dc, experiment.getId());
		this.project = project == null ? null : Project.getById(dc, project.getId());
		generateOverview(dc);
	}
	
	/**
		Generates the overview and validation.
	*/
	private void generateOverview(DbControl dc)
	{
		this.failures = new LinkedList<Failure>();
		this.annotatableParents = new HashMap<Object, Set<Annotatable>>();
		this.usedHybArrays = new HashMap<String, Node>();
		
		// Load 'Required for MIAME' annotation types (parameter='false' must also be set)
		ItemQuery<AnnotationType> query = initQuery(AnnotationType.getQuery(null), null, "name");
		query.restrict(
			Restrictions.eq(
				Hql.property("requiredForMiame"), 
				Expressions.parameter("miameFlag", true, Type.BOOLEAN)
			)
		);
		query.restrict(
			Restrictions.eq(
				Hql.property("protocolParameter"), 
				Expressions.parameter("parameterFlag", false, Type.BOOLEAN)
			)
		);
		this.miameAnnotations = new HashMap<Item, Set<AnnotationType>>();
		for (AnnotationType at : query.list(dc))
		{
			for (Item itemType : at.getEnabledItems())
			{
				Set<AnnotationType> types = miameAnnotations.get(itemType);
				if (types == null)
				{
					types = new HashSet<AnnotationType>();
					miameAnnotations.put(itemType, types);
				}
				types.add(at);
			}
		}
		
		// Node and subnodes
		this.rootNode = new Node("root", experiment.getName(), experiment);
		addExperimentalFactors(dc, rootNode);
		addRawBioAssays(dc, rootNode);
		
		// Validation
		if (project != null)
		{
			RawDataType projectDefault = project.getDefaultRawDataType();
			if (projectDefault != null && experiment.getRawDataType() != projectDefault)
			{
				failures.add(
					new Failure(Validator.NONDEFAULT_RAWDATATYPE, rootNode, null,
						new Fix("Change default raw data type of project", project)
					)
				);
			}
		}
		
		usedHybArrays.clear(); // No longer needed
		this.allNodes = new HashMap<String, Node>();
		allNodes.put(rootNode.getId(), rootNode);
		cacheChildNodes(rootNode);

		// Validate the names of items
		Map<String, Map<Item, List<Node>>> names = new HashMap<String, Map<Item, List<Node>>>();
		Set<Node> nonUniqueGlobal = new HashSet<Node>();
		Set<Node> nonUniqueForType = new HashSet<Node>();
		for (Node node : allNodes.values())
		{
			if (node.getNodeType() == Node.Type.ITEM)
			{
				BasicItem item = node.getItem();
				if (item instanceof Nameable)
				{
					String name = ((Nameable)item).getName();
					Map<Item, List<Node>> typeMap = names.get(name);
					if (typeMap != null)
					{
						List<Node> otherNodes = typeMap.get(item.getType());
						if (otherNodes != null)
						{
							// Possible non-unique match; but check for self-match
							if (!item.equals(otherNodes.get(0).getItem()))
							{
								nonUniqueForType.add(node);
								for (Node otherNode : otherNodes)
								{
									nonUniqueForType.add(otherNode);
								}
							}
							else
							{
								otherNodes.add(node);
							}
						}
						else
						{
							nonUniqueGlobal.add(node);
							if (typeMap.size() == 1)
							{
								nonUniqueGlobal.add(typeMap.values().iterator().next().get(0));
							}
							List<Node> nodeList = new LinkedList<Node>();
							nodeList.add(node);
							typeMap.put(item.getType(), nodeList);
						}
					}
					else
					{
						List<Node> nodeList = new LinkedList<Node>();
						nodeList.add(node);
						typeMap = new HashMap<Item, List<Node>>();
						typeMap.put(item.getType(), nodeList);
						names.put(name, typeMap);
					}
				}
			}
		}
		names.clear();
		
		// Non-unique of same type
		for (Node node : nonUniqueForType)
		{
			failures.add(new Failure(Validator.NONUNIQUE_NAME_FOR_TYPE, node, null,
				new Fix("Change name of item", node.getItem())
			));
		}
		nonUniqueForType.clear();
		
		// Non-unique of different types
		for (Node node : nonUniqueGlobal)
		{
			failures.add(new Failure(Validator.NONUNIQUE_NAME_GLOBAL, node, null,
				new Fix("Change name of item", node.getItem())
			));
		}
		nonUniqueGlobal.clear();

		updateFailureCountOnNodes();
	}
	
	/**
		Get the experiment of this overview.
	*/
	public Experiment getExperiment()
	{
		return experiment;
	}
	
	/**
		Get the project of this overview (may be null).
	*/
	public Project getProject()
	{
		return project;
	}
	
	/**
		Get the annotation types that have the 'Required for MIAME' flag
		set to true and is enabled for the specified item type.
		@param itemType The item type
		@return An set containing the annotation types or null if no annotation
			types exists
	*/
	public Set<AnnotationType> getMiameAnnotationTypes(Item itemType)
	{
		return miameAnnotations.get(itemType);
	}
	
	/**
		Get all annotatatable parents, including parents of parents, 
		of the given annotatable item. This information is useful for
		checking that an item only inherits annotations from one of
		the parents.

		@param item The child item
		@return A set containing all annotatable parents.
	*/
	public Set<Annotatable> getAnnotatableParents(Annotatable item)
	{
		return getAnnotatableParents(item, item);
	}
	
	private Set<Annotatable> getAnnotatableParents(Annotatable current, Annotatable root)
	{
		Object cacheKey = current;
		Hybridization hyb = null;
		int arrayNum = 1;
		// Special case when starting with a raw bioassay since we only
		// want to get parents from the sub-array with the same 
		// index = rba.getArrayNum()
		if (root instanceof RawBioAssay)
		{
			RawBioAssay rba = (RawBioAssay)root;
			arrayNum = rba.getArrayNum();
			if (current instanceof Scan)
			{
				cacheKey = "SCAN:" + current.getId() + ":" + arrayNum;
			}
			else if (current instanceof Hybridization)
			{
				hyb = (Hybridization)current;
				cacheKey = "HYB:" + hyb.getId() + ":" + arrayNum;
			}
		}
	
		Set<Annotatable> parents = annotatableParents.get(cacheKey);
		if (parents == null)
		{
			parents = new HashSet<Annotatable>();
			annotatableParents.put(cacheKey, parents);
			Set<Annotatable> temp = hyb != null ?
				hyb.getAnnotatableParents(arrayNum) : current.getAnnotatableParents();
			if (temp != null)
			{
				for (Annotatable a : temp)
				{
					parents.add(a);
					parents.addAll(getAnnotatableParents(a, root));
				}
			}
		}
		return parents;
		
	}
	
	/**
		Get the validation options currently in use.
	*/
	public ValidationOptions getValidationOptions()
	{
		return validationOptions;
	}
	
	/**
		Get the root node (ie. the experiment node).
	*/
	public Node getRootNode()
	{
		return rootNode;
	}
	
	/**
		Updates the failure count (warnings and errors) on all nodes.
		This method should be called if the validation options has 
		changed. If the struture or information of the experiment has
		changed use the {@link #reset(DbControl)} method instead. That
		method rebuilds the entire overview.
	*/
	public void updateFailureCountOnNodes()
	{
		rootNode.clearFailures();
		for (Failure f : failures)
		{
			Severity s = validationOptions.getSeverity(f.getValidator());
			Node n = f.getNode();
			if (s == Severity.ERROR)
			{
				n.addErrors(1);
			}
			else if (s == Severity.WARNING)
			{
				n.addWarnings(1);
			}
		}
	}
	
	private void cacheChildNodes(Node node)
	{
		if (node.getChildren() != null)
		{
			for (Node child : node.getChildren())
			{
				allNodes.put(child.getId(), child);
				cacheChildNodes(child);
			}
		}
	}
	
	/**
		Get the node with the given ID.
		@param nodeId The ID of the node
		@return A node or null if no node with the ID exists
	*/
	public Node getNode(String nodeId)
	{
		return allNodes.get(nodeId);
	}
	
	/**
		Get all failures generated by validation rules. The list includes all
		failures regardless of validation options. The calling code should check 
		the validation options if a failure should be ignored, treated as a warning
		or treated as an error.
		@see ValidationOptions#getSeverity(Validator)
		@see Failure#getValidator()
	*/
	public List<Failure> getFailures()
	{
		return failures;
	}
	
	/**
		Add experimental factors child node to the experiment node.
		Creates a folder-type child node with the name <code>experimental.factors</code>
		and inside it nodes with names <code>experimental.factor.ID</code> where ID
		is the ID of the {@link AnnotationType} used as experimental factor.
		@see Experiment#getExperimentalFactors()
	*/
	private Node addExperimentalFactors(DbControl dc, Node rootNode)
	{
		Node efNode = new Node("experimental.factors", "Experimental factors", rootNode);
		Experiment experiment = (Experiment)rootNode.getItem();
		ItemQuery<AnnotationType> efQuery = initQuery(experiment.getExperimentalFactors(), null, "name");
		for (AnnotationType ef : efQuery.list(dc))
		{
			new Node("experimental.factor."+ef.getId(), ef.getName(), efNode, ef);
		}
		return efNode;
	}
	
	/**
		Add raw bioassays child node to the experiment node. Creates a folder-type
		child node with the name <code>rawbioassays</code> and calls 
		{@link #addRawBioAssay(DbControl, Node, RawBioAssay)} for each raw bioassay.
		@see Experiment#getRawBioAssays()
	*/
	private Node addRawBioAssays(DbControl dc, Node rootNode)
	{
		Node rawBioAssaysNode = new Node("rawbioassays", "Raw bioassays", rootNode);
		Experiment experiment = (Experiment)rootNode.getItem();
		ItemQuery<RawBioAssay> rbaQuery = initQuery(experiment.getRawBioAssays(), null, "name");
		Set<BasicItem> arrayDesigns = new HashSet<BasicItem>();
		for (RawBioAssay rba : rbaQuery.list(dc))
		{
			Node rbaNode = addRawBioAssay(dc, rawBioAssaysNode, rba);
			Node adNode = rbaNode.getChild("arraydesign");
			arrayDesigns.add(adNode == null ? null : adNode.getItem());
		}
		
		// Check if all raw bioassays use the same array design
		if (arrayDesigns.size() > 1)
		{
			failures.add(new Failure(Validator.MULTIPLE_ARRAYDESIGNS, rootNode));
		}
		
		return rawBioAssaysNode;
	}
	
	/**
	 	Add a raw bioassay child node to the rawbioassays folder. Creates an item-type
	 	subnode with the name <code>rawbioassay.ID</code> where ID is the ID of the
	 	{@link RawBioAssay}.
	*/
	private Node addRawBioAssay(DbControl dc, Node rawBioAssaysNode, RawBioAssay rba)
	{
		Node rbaNode = new Node("rawbioassay."+rba.getId(), rba.getName(), rawBioAssaysNode, rba);
		Node annotationsNode = addAnnotations(dc, rbaNode);
		addDataFiles(dc, rbaNode);
		Node adNode = addArrayDesign(dc, rbaNode);
		Node scanNode = addScan(dc, rbaNode);
		
		addPlatform(dc, rbaNode);
		addSoftware(dc, rbaNode);
		addProtocol(dc, rbaNode, Project.Default.FEATURE_EXTRACTION_PROTOCOL);
		
		// Check for project default raw data type
		if (project != null)
		{
			RawDataType projectDefault = project.getDefaultRawDataType();
			if (projectDefault != null && rba.getRawDataType() != projectDefault)
			{
				failures.add(
					new Failure(Validator.NONDEFAULT_RAWDATATYPE, rbaNode, null,
						new Fix("Change default raw data type of project", project)
					)
				);
			}
		}
		
		// Check for missing experimental factor values
		Node efNode = rootNode.getChild("experimental.factors");
		if (efNode != null && efNode.getChildren() != null)
		{
			for (Node factorNode : efNode.getChildren())
			{
				Node annotationNode = annotationsNode == null ? 
						null : annotationsNode.getChild("annotationtype." + factorNode.getItem().getId());
				if (annotationNode == null)
				{
					AnnotationType at = (AnnotationType)factorNode.getItem();
					Fix addFactor = at.isEnabledForItem(Item.RAWBIOASSAY) ?
						new Fix("Add value to experimental factor", rba, at, false) :
						new Fix("Inherit annotation from a parent item", rba, at, true);
					failures.add(new Failure(Validator.MISSING_FACTORVALUE, rbaNode, 
						"Missing value for experimental factor '" + at.getName() + "'", addFactor));
				}
			}
		}

		// Check that the array design is same as hybridizations array design
		Scan scan = scanNode == null ? null : (Scan)scanNode.getItem();
		if (scan != null)
		{
			BasicItem ad1 = adNode == null ? null : adNode.getItem();
			try
			{
				BasicItem ad2 = scan.getArrayDesign();
				if ((ad1 == null && ad2 != null) || (ad1 != null && !ad1.equals(ad2)))
				{
					Fix fix = null;
					try
					{
						Hybridization hyb = scan.getHybridization();
						if (hyb != null) fix = new Fix("Change array slide of hybridization", hyb);
					}
					catch (PermissionDeniedException ex)
					{}
					failures.add(new Failure(Validator.NONMATHCING_ARRAYDESIGN, rbaNode, null,
						fix));
				}
			}
			catch (PermissionDeniedException ex)
			{}
		}
		
		return rbaNode;
	}
	
	/**
		Add an array design child node to a raw bioassay or array batch.
		Creates an item-type node with the name <code>arraydesign</code>.
	*/
	private Node addArrayDesign(DbControl dc, Node parentNode)
	{
		BasicItem item = parentNode.getItem();
		ArrayDesign ad = null;
		Node adNode = null;
		try
		{
			if (item.getType() == Item.RAWBIOASSAY)
			{
				ad = ((RawBioAssay)item).getArrayDesign();
			}
			else if (item.getType() == Item.ARRAYBATCH)
			{
				ad = ((ArrayBatch)item).getArrayDesign();
			}
		}
		catch (PermissionDeniedException ex)
		{
			adNode = new Node("arraydesign", "Array design: denied", parentNode, null);
			failures.add(new Failure(Validator.DENIED_ARRAYDESIGN, adNode));
		}
		if (ad != null)
		{
			adNode = new Node("arraydesign", "Array design: " + ad.getName(), parentNode, ad);
			addDataFiles(dc, adNode);
			addPlatform(dc, adNode);

			// Check if the array design is the same as the project default
			if (project != null)
			{
				try
				{
					BasicItem defaultItem = project.getDefaultItem(dc, Project.Default.ARRAYDESIGN);
					if (defaultItem != null && !ad.equals(defaultItem))
					{
						failures.add(new Failure(Validator.NONDEFAULT_ARRAYDESIGN, parentNode, null,
							new Fix("Change default array design for project", project)));
					}
				}
				catch (BaseException ex)
				{}
			}
		}
		if (adNode == null)
		{
			Fix fix = item.getType() == Item.RAWBIOASSAY ?
					new Fix("Add array design to raw bioassay", item) : 
					null;
			failures.add(new Failure(Validator.MISSING_ARRAYDESIGN, parentNode, null, fix));
		}
		return adNode;
	}
	
	/**
		Add a scan child node to a raw bioassay.
		Creates an item-type node with the name <code>scan</code>.
	*/
	private Node addScan(DbControl dc, Node rawBioAssayNode)
	{
		RawBioAssay rba = (RawBioAssay)rawBioAssayNode.getItem();
		Scan scan = null;
		Node scanNode = null;
		try
		{
			scan = rba.getScan();
		}
		catch (PermissionDeniedException ex)
		{
			scanNode = new Node("scan", "Scan: denied", rawBioAssayNode, null);
			failures.add(new Failure(Validator.DENIED_SCAN, scanNode));
		}
		if (scan != null)
		{
			scanNode = new Node("scan", "Scan: " + scan.getName(), rawBioAssayNode, scan);

			addAnnotations(dc, scanNode);
			addImages(dc, scanNode);
			addHardware(dc, scanNode, "Scanner");
			addHybridization(dc, scanNode, rawBioAssayNode);
			addProtocol(dc, scanNode, Project.Default.SCANNING_PROTOCOL);
		}
		if (scanNode == null)
		{
			failures.add(new Failure(Validator.MISSING_SCAN, rawBioAssayNode, null,
				new Fix("Add scan to raw bioassay", rba)));
		}
		return scanNode;
	}
	
	/**
		Add a hybridization child node to a scan.
		Creates an item-type node with the name <code>hybridization</code>.
	*/
	private Node addHybridization(DbControl dc, Node scanNode, Node rawBioAssayNode)
	{
		Scan scan = (Scan)scanNode.getItem();
		RawBioAssay rba = (RawBioAssay)rawBioAssayNode.getItem();
		Node hybNode = null;
		Hybridization hyb = null;
		try
		{
			hyb = scan.getHybridization();
		}
		catch (PermissionDeniedException ex)
		{
			hybNode = new Node("hybridization", "Hybridization: denied", scanNode, null);
			failures.add(new Failure(Validator.DENIED_HYBRIDIZATION, hybNode));
		}
		if (hyb != null)
		{
			hybNode = new Node("hybridization", "Hybridization: " + hyb.getName(), scanNode, hyb);
			addAnnotations(dc, hybNode);
			addLabeledExtracts(dc, hybNode, rawBioAssayNode);
			addHardware(dc, hybNode, "Hyb. station");
			addProtocol(dc, hybNode, Project.Default.HYBRIDIZATION_PROTOCOL);
			addArraySlide(dc, hybNode);
			
			// Check that arrayNum <= hyb.getNumArrays()
			int arrayNum = rba.getArrayNum();
			if (arrayNum > hyb.getNumArrays())
			{
				Failure failure = new Failure(Validator.INCORRECT_ARRAY_NUM, rawBioAssayNode,
					"Incorrect sub-array index: " + arrayNum + " (expected 1--" + hyb.getNumArrays() + ")",
					new Fix("Change array index on raw bioassay", rba), 
					new Fix("Increase arrays on hybridization", hyb));
				failures.add(failure);
			}
			
			String usedHybIndex = hyb.getId() + ":" + arrayNum;
			if (usedHybArrays.containsKey(usedHybIndex))
			{
				Node other = usedHybArrays.get(usedHybIndex);
				Fix fixA = new Fix("Change array index on " + rba.getName(), rba);
				Fix fixB = new Fix("Change array index on " + other.getTitle(), other.getItem()); 

				Failure failure = new Failure(Validator.DUPLICATE_ARRAY_NUM, rawBioAssayNode,
						"Duplicate sub-array index: " + arrayNum + " (used by " + other.getTitle() + ")",
						fixA, fixB);
				failures.add(failure);
				
				failure = new Failure(Validator.DUPLICATE_ARRAY_NUM, other,
						"Duplicate sub-array index: " + arrayNum + " (used by " + rba.getName() + ")",
						fixB, fixA);
				failures.add(failure);
			}
			else
			{
				usedHybArrays.put(usedHybIndex, rawBioAssayNode);
			}
			
		}
		if (hybNode == null)
		{
			failures.add(new Failure(Validator.MISSING_HYBRIDIZATION, scanNode, null,
				new Fix("Add hybridization to scan", scan)));
		}
		return hybNode;
	}
	
	/**
		Add images child node to a scan node. Creates a folder-type
		child node with the name <code>images</code> and calls 
		{@link #addImage(DbControl, Node, Image)} for each image.
	*/
	private Node addImages(DbControl dc, Node scanNode)
	{
		Node imagesNode = new Node("images", "Images", scanNode);
		Scan scan = (Scan)scanNode.getItem();
		ItemQuery<Image> imageQuery = initQuery(scan.getImages(), null, "name");
		for (Image image : imageQuery.list(dc))
		{
			addImage(dc, imagesNode, image);
		}
		return imagesNode;
	}

	/**
	 	Add an image child node to the images folder. Creates an item-type
	 	subnode with the name <code>image.ID</code> where ID is the ID of the
	 	{@link Image}.
	*/
	private Node addImage(DbControl dc, Node imagesNode, Image image)
	{
		Node imageNode = new Node("image."+image.getId(), image.getName(), imagesNode, image);
		return imageNode;
	}

	/**
		Add labeled extracts child node to a hybridization node. Creates a 
		folder-type child node with the name <code>labeledextracts</code> and calls 
		{@link #addLabeledExtract(DbControl, Node, LabeledExtract)} for each labeled extract.
	*/
	private Node addLabeledExtracts(DbControl dc, Node hybNode, Node rawBioAssayNode)
	{
		Node leNode = new Node("labeledextracts", "Labeled extracts", hybNode);
		Hybridization hyb = (Hybridization)hybNode.getItem();
		RawBioAssay rba = (RawBioAssay)rawBioAssayNode.getItem();
		int arrayNum = rba.getArrayNum();
		ItemQuery<LabeledExtract> leQuery = 
			initQuery(hyb.getLabeledExtracts(arrayNum), null, "label.name");
		int added = 0;
		int channels = experiment.getRawDataType().getChannels();

		Map<Label, LabeledExtract> labels = new HashMap<Label, LabeledExtract>();
		Set<LabeledExtract> nonUnique = new HashSet<LabeledExtract>();
		for (LabeledExtract le : leQuery.list(dc))
		{
			Node llNode = addLabeledExtract(dc, leNode, le);
			++added;
			try
			{
				Label l = le.getLabel();
				if (labels.containsKey(l))
				{
					nonUnique.add(le);
					nonUnique.add(labels.get(l));
				}
				else
				{
					labels.put(l, le);
				}
			}
			catch (PermissionDeniedException ex)
			{}
		}
		
		// Check number of labeled extracts
		if (added != channels)
		{
			Fix fix = added > channels ?
				new Fix("Remove labeled extracts from hybridization", hyb) :
				new Fix("Add labeled extracts to hybridization", hyb);
			String message = added > channels ?
				"Too many labeled extracts" : "Too few labeled extracts";
			message += ": " + added + " (" + channels + " expected)";
			Failure failure = new Failure(Validator.INCORRECT_NUM_LABELEDEXTRACTS, hybNode, message, fix);
			failure.setDescriptor(arrayNum);
			failures.add(failure);
		}
		
		// Check that labeled extracts has unique labels
		if (nonUnique.size() > 0)
		{
			Failure failure = new Failure(Validator.NONUNIQUE_LABEL, hybNode);
			failure.setDescriptor(arrayNum);
			failure.addFix(new Fix("Change labeled extracts on hybridization", hyb));
			for (LabeledExtract le : nonUnique)
			{
				failure.addFix(new Fix("Change label of '" + le.getName() + "'", le));
			}
			failures.add(failure);
		}
		
		return leNode;
	}

	/**
	 	Add a labeled extract child node to the labeled extracts folder. Creates an 
	 	item-type subnode with the name <code>labeledextract.ID</code> where ID is the ID of the
	 	{@link LabeledExtract}.
	*/
	private Node addLabeledExtract(DbControl dc, Node parentNode, LabeledExtract le)
	{
		Label label = null;
		try
		{
			label = le.getLabel();
		}
		catch (PermissionDeniedException ex)
		{}
		Node leNode = new Node("labeledextract."+le.getId(), 
			(label != null ? label.getName() + ": " : "") + le.getName(), parentNode, le);
		
		if (label == null)
		{
			failures.add(new Failure(Validator.DENIED_LABEL, leNode));
		}
		
		addAnnotations(dc, leNode);
		if (le.isPooled())
		{
			addPooledLabeledExtracts(dc, leNode);
		}
		else
		{
			addExtract(dc, leNode, null);
		}
		addProtocol(dc, leNode, Project.Default.LABELING_PROTOCOL);
		return leNode;
	}
	
	/**
		Add labeled extract child nodes that are the source labeled
		extracts of a pooled labeled extract
	*/
	@SuppressWarnings("unchecked")
	private void addPooledLabeledExtracts(DbControl dc, Node parentNode)
	{
		LabeledExtract product = (LabeledExtract)parentNode.getItem();
		Label productLabel = null;
		try
		{
			productLabel = product.getLabel();
		}
		catch (PermissionDeniedException ex)
		{}
		
		ItemQuery<LabeledExtract> sourceQuery = 
			initQuery((ItemQuery<LabeledExtract>)product.getCreationEvent().getSources(), null, "name");
		
		int added = 0;
		for (LabeledExtract source : sourceQuery.list(dc))
		{
			++added;

			// Check circular reference
			boolean circular = false;
			Node tempNode = parentNode;
			BasicItem tempItem = tempNode.getItem();
			while (tempNode != null && tempItem != null && tempItem.getType() == Item.LABELEDEXTRACT)
			{
				if (source.equals(tempItem))
				{
					failures.add(new Failure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + ((LabeledExtract)tempItem).getName() + "'", 
						new Fix("Modify source labeled extracts", product)
					));
					tempNode = null;
					tempItem = null;
					circular = true;
				}
				else
				{
					tempNode = tempNode.getParent();
					tempItem = tempNode.getItem();
				}
			}
			
			if (!circular)
			{
				Node leNode = addLabeledExtract(dc, parentNode, source);
				
				// Check that source has same label as product
				if (productLabel != null)
				{
					try
					{
						Label label = source.getLabel();
						if (!productLabel.equals(label))
						{
							failures.add(new Failure(Validator.DIFFERENT_LABEL, leNode, null,
								new Fix("Change label of '" + source.getName() + "'", source),
								new Fix("Change label of '" + product.getName() + "'", product)
								));
						}
					}
					catch (PermissionDeniedException ex)
					{}
				}
			}
		}
		
		if (added == 0)
		{
			failures.add(new Failure(Validator.MISSING_LABELED_EXTRACT, parentNode, null,
				new Fix("Add sources to the labeled extract", product)
			));
		}
		
	}
	
	/**
		Add an extract child node to a non-pooled labeled extract.
		Creates an item-type node with the name <code>extract</code>.
	*/
	private Node addExtract(DbControl dc, Node parentNode, Extract extract)
	{
		Node extractNode = null;
		if (extract == null)
		{
			LabeledExtract le = (LabeledExtract)parentNode.getItem();
			try
			{
				extract = le.getExtract();
			}
			catch (PermissionDeniedException ex)
			{
				extractNode = new Node("extract", "Extract: denied", parentNode, null);
				failures.add(new Failure(Validator.DENIED_EXTRACT, extractNode));
			}
			if (extract != null)
			{
				extractNode = new Node("extract", "Extract: " + extract.getName(), parentNode, extract);
			}
		}
		else
		{
			// Parent is a pooled extract
			extractNode = new Node("extract." + extract.getId(), extract.getName(), parentNode, extract);
		}

		if (extract != null)
		{
			addAnnotations(dc, extractNode);
			if (extract.isPooled())
			{
				addPooledExtracts(dc, extractNode);
			}
			else
			{
				addSample(dc, extractNode, null);
			}
			addProtocol(dc, extractNode, Project.Default.EXTRACTING_PROTOCOL);
		}
		if (extractNode == null)
		{
			failures.add(new Failure(Validator.MISSING_EXTRACT, parentNode, null,
				new Fix("Add extract to labeled extract", parentNode.getItem())));
		}
		return extractNode;
	}
	
	/**
		Add extract child nodes that are the source extracts of a pooled extract.
	*/
	@SuppressWarnings("unchecked")
	private void addPooledExtracts(DbControl dc, Node parentNode)
	{
		Extract product = (Extract)parentNode.getItem();
		
		ItemQuery<Extract> sourceQuery = 
			initQuery((ItemQuery<Extract>)product.getCreationEvent().getSources(), null, "name");
		
		int added = 0;
		for (Extract source : sourceQuery.list(dc))
		{
			++added;
	
			// Check circular reference
			boolean circular = false;
			Node tempNode = parentNode;
			BasicItem tempItem = tempNode.getItem();
			while (tempNode != null && tempItem != null && tempItem.getType() == Item.EXTRACT)
			{
				if (source.equals(tempItem))
				{
					failures.add(new Failure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + ((Extract)tempItem).getName() + "'", 
						new Fix("Modify source extracts", product)
					));
					tempNode = null;
					tempItem = null;
					circular = true;
				}
				else
				{
					tempNode = tempNode.getParent();
					tempItem = tempNode.getItem();
				}
			}
			
			if (!circular)
			{
				Node extractNode = addExtract(dc, parentNode, source);
			}
		}
		
		if (added == 0)
		{
			failures.add(new Failure(Validator.MISSING_EXTRACT, parentNode, null,
				new Fix("Add sources to the extract", product)
			));
		}
	}
	
	/**
		Add a sample child node to a non-pooled extract.
		Creates an item-type node with the name <code>sample</code>.
	*/
	private Node addSample(DbControl dc, Node parentNode, Sample sample)
	{
		Node sampleNode = null;
		if (sample == null)
		{
			Extract extract = (Extract)parentNode.getItem();
			try
			{
				sample = extract.getSample();
			}
			catch (PermissionDeniedException ex)
			{
				sampleNode = new Node("sample", "Sample: denied", parentNode, null);
				failures.add(new Failure(Validator.DENIED_SAMPLE, sampleNode));
			}
			if (extract != null)
			{
				sampleNode = new Node("sample", "Sample: " + sample.getName(), parentNode, sample);
			}
		}
		else
		{
			// Parent is a pooled extract
			sampleNode = new Node("sample." + sample.getId(), sample.getName(), parentNode, sample);
		}

		if (sample != null)
		{
			addAnnotations(dc, sampleNode);
			if (sample.isPooled())
			{
				addPooledSamples(dc, sampleNode);
			}
			else
			{
				addBioSource(dc, sampleNode);
			}
			addProtocol(dc, sampleNode, Project.Default.SAMPLING_PROTOCOL);
		}
		if (sampleNode == null)
		{
			failures.add(new Failure(Validator.MISSING_SAMPLE, parentNode, null,
				new Fix("Add sample to extract", parentNode.getItem())));
		}
		return sampleNode;
	}
	
	/**
		Add sample child nodes that are the source samples of a pooled sample.
	*/
	@SuppressWarnings("unchecked")
	private void addPooledSamples(DbControl dc, Node parentNode)
	{
		Sample product = (Sample)parentNode.getItem();
		
		ItemQuery<Sample> sourceQuery = 
			initQuery((ItemQuery<Sample>)product.getCreationEvent().getSources(), null, "name");
		
		int added = 0;
		for (Sample source : sourceQuery.list(dc))
		{
			++added;
	
			// Check circular reference
			boolean circular = false;
			Node tempNode = parentNode;
			BasicItem tempItem = tempNode.getItem();
			while (tempNode != null && tempItem != null && tempItem.getType() == Item.SAMPLE)
			{
				if (source.equals(tempItem))
				{
					failures.add(new Failure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + ((Sample)tempItem).getName() + "'", 
						new Fix("Modify source samples", product)
					));
					tempNode = null;
					tempItem = null;
					circular = true;
				}
				else
				{
					tempNode = tempNode.getParent();
					tempItem = tempNode.getItem();
				}
			}
			
			if (!circular)
			{
				Node sampleNode = addSample(dc, parentNode, source);
			}
		}
		
		if (added == 0)
		{
			failures.add(new Failure(Validator.MISSING_SAMPLE, parentNode, null,
				new Fix("Add sources to the sample", product)
			));
		}
	}
	
	/**
		Add a biosource child node to a non-pooled sample.
		Creates an item-type node with the name <code>biosource</code>.
	*/
	private Node addBioSource(DbControl dc, Node sampleNode)
	{
		Sample sample = (Sample)sampleNode.getItem();
		Node bioSourceNode = null;
		BioSource bioSource = null;
		try
		{
			bioSource = sample.getBioSource();
		}
		catch (PermissionDeniedException ex)
		{
			bioSourceNode = new Node("biosource", "Biosource: denied", sampleNode, null);
			failures.add(new Failure(Validator.DENIED_BIOSOURCE, bioSourceNode));
		}
		if (bioSource != null)
		{
			bioSourceNode = new Node("biosource", "Biosource: " + bioSource.getName(), sampleNode, bioSource);
			addAnnotations(dc, bioSourceNode);
		}
		if (bioSourceNode == null)
		{
			failures.add(new Failure(Validator.MISSING_BIOSOURCE, sampleNode, null,
				new Fix("Add biosource to sample", sampleNode.getItem())));
		}
		return bioSourceNode;
	}
	
	/**
		Add an array slide child node to a hybridization.
		Creates an item-type node with the name <code>slide</code>.
	*/
	private Node addArraySlide(DbControl dc, Node hybNode)
	{
		Hybridization hyb = (Hybridization)hybNode.getItem();
		Node slideNode = null;
		ArraySlide slide = null;
		try
		{
			slide = hyb.getArraySlide();
		}
		catch (PermissionDeniedException ex)
		{
			slideNode = new Node("slide", "Array slide: denied", hybNode, null);
			failures.add(new Failure(Validator.DENIED_ARRAYSLIDE, slideNode));
		}
		if (slide != null)
		{
			slideNode = new Node("slide", "Arrray slide: " + slide.getName(), hybNode, slide);
			addAnnotations(dc, slideNode);
			addArrayBatch(dc, slideNode);
		}
		if (slideNode == null)
		{
			failures.add(new Failure(Validator.MISSING_ARRAYSLIDE, hybNode, null,
				new Fix("Add array slide to hybridization", hyb)));
		}
		return slideNode;
	}

	/**
		Add an array batch child node to an array slide.
		Creates an item-type node with the name <code>batch</code>.
	*/
	private Node addArrayBatch(DbControl dc, Node slideNode)
	{
		ArraySlide slide = (ArraySlide)slideNode.getItem();
		Node batchNode = null;
		ArrayBatch batch = null;
		try
		{
			batch = slide.getArrayBatch();
		}
		catch (PermissionDeniedException ex)
		{
			batchNode = new Node("batch", "Batch: denied", slideNode, null);
			failures.add(new Failure(Validator.DENIED_ARRAYBATCH, batchNode));
		}
		if (batch != null)
		{
			batchNode = new Node("batch", "Batch: " + batch.getName(), slideNode, batch);
			addAnnotations(dc, batchNode);
			addArrayDesign(dc, batchNode);
			addHardware(dc, batchNode, "Print robot");
			addProtocol(dc, batchNode, Project.Default.PRINTING_PROTOCOL);
		}
		if (batchNode == null)
		{
			failures.add(new Failure(Validator.MISSING_ARRAYBATCH, slideNode));
		}
		return batchNode;
	}
	
	/**
		Add a platform / variant child node to items with platform (raw bioassay 
		and array design). 
		Creates an item-type node with the name <code>platform</code> and a
		subnode <code>variant</code> if present.
	*/
	private Node addPlatform(DbControl dc, Node parentNode)
	{
		BasicItem item = parentNode.getItem();
		String itemTypeName = item.getType().toString().toLowerCase();
		Platform platform = null;
		PlatformVariant variant = null;
		Node platformNode = null;
		Node variantNode = null;
		boolean canChangePlatform = true;
		
		try
		{
			Item itemType = item.getType();
			if (itemType == Item.RAWBIOASSAY)
			{
				RawBioAssay rba = (RawBioAssay)item;
				canChangePlatform = rba.getNumDbSpots() == 0;
				platform = rba.getPlatform();
				variant = rba.getVariant();
			}
			else if (itemType == Item.ARRAYDESIGN)
			{
				ArrayDesign ad = (ArrayDesign)item;
				platform = ad.getPlatform();
				variant = ad.getVariant();
			}
		}
		catch (PermissionDeniedException ex)
		{
			platformNode = new Node("platform", "Platform: denied", parentNode, null);
			failures.add(new Failure(Validator.DENIED_PLATFORM, platformNode));
		}
		
		if (platform != null)
		{
			// Add child nodes for platform and variant
			platformNode = new Node("platform", "Platform: " + platform.getName(), parentNode, platform);
			if (variant != null)
			{
				variantNode = new Node("variant", "Variant: " + variant.getName(), platformNode, variant);
			}
			
			// Check if using project default platform / variant
			if (project != null)
			{
				try
				{
					BasicItem defaultPlatform = project.getDefaultItem(dc, Project.Default.PLATFORM);
					BasicItem defaultVariant = project.getDefaultItem(dc, Project.Default.PLATFORM_VARIANT);

					if ((defaultPlatform != null && !defaultPlatform.equals(platform)) ||
						(defaultVariant != null && !defaultVariant.equals(variant)))
					{
						failures.add(new Failure(Validator.NONDEFAULT_PLATFORM, parentNode, null,
							canChangePlatform ? new Fix("Change platform of " + itemTypeName, item) : null,
							new Fix("Change default platform of project", project)
						));
					}
				}
				catch (BaseException ex)
				{}
			}
		}
		return platformNode;
	}	
	
	/**
		Add a software child node to a raw bioassay.
		Creates an item-type node with the name <code>software</code>.
	*/
	private Node addSoftware(DbControl dc, Node parentNode)
	{
		BasicItem item = parentNode.getItem();
		Software software = null;
		Node softwareNode = null;
		Project.Default projectDefault = null;
		try
		{
			if (item.getType() == Item.RAWBIOASSAY)
			{
				software = ((RawBioAssay)item).getSoftware();
				projectDefault = Project.Default.SOFTWARE;
			}
		}
		catch (PermissionDeniedException ex)
		{
			softwareNode = new Node("software", "Software: denied", parentNode, null);
			failures.add(new Failure(Validator.DENIED_SOFTWARE, softwareNode));
		}
		if (software != null)
		{
			softwareNode = new Node("software", "Software: " + software.getName(), parentNode, software);

			// Check if using project default software
			if (project != null && projectDefault != null)
			{
				try
				{
					BasicItem defaultItem = project.getDefaultItem(dc, projectDefault);
					if (defaultItem != null && !software.equals(defaultItem))
					{
						failures.add(new Failure(Validator.NONDEFAULT_SOFTWARE, parentNode, null,
								new Fix("Change software of raw bioassay", item),
								new Fix("Change default software of project", project)
							));
					}
				}
				catch (BaseException ex)
				{}
			}
			
			// Check software type
			try
			{
				SoftwareType st = software.getSoftwareType();
				if (!st.getSystemId().equals(projectDefault.getType()))
				{
					failures.add(new Failure(Validator.INCORRECT_SOFTWARETYPE, parentNode, null,
						new Fix("Change software of raw bioassay", item),
						new Fix("Change type of software", software)
					));
				}
			}
			catch (PermissionDeniedException ex)
			{}
			
		}
		if (softwareNode == null)
		{
			failures.add(new Failure(Validator.MISSING_SOFTWARE, parentNode, null,
				new Fix("Add software to raw bioassay", item)));
		}
		return softwareNode;
	}
	
	/**
		Add a hardware child node to items with hardware (scanner, print robot, hyb. 
		station, etc.) Creates an item-type node with the name <code>hardware</code>.
	*/
	private Node addHardware(DbControl dc, Node parentNode, String hardwareType)
	{
		BasicItem item = parentNode.getItem();
		String itemTypeName = item.getType().toString().toLowerCase();
		Hardware hardware = null;
		Node hardwareNode = null;
		Project.Default projectDefault = null;
		try
		{
			Item itemType = item.getType();
			if (itemType == Item.SCAN)
			{
				hardware = ((Scan)item).getScanner();
				projectDefault = Project.Default.SCANNER_HARDWARE;
			}
			else if (itemType == Item.HYBRIDIZATION)
			{
				hardware = ((Hybridization)item).getCreationEvent().getHardware();
				projectDefault = Project.Default.HYBRIDIZATION_HARDWARE;
			}
			else if (itemType == Item.ARRAYBATCH)
			{
				hardware = ((ArrayBatch)item).getPrintRobot();
				projectDefault = Project.Default.PRINTROBOT_HARDWARE;
			}
		}
		catch (PermissionDeniedException ex)
		{
			hardwareNode = new Node("hardware", hardwareType + ": denied", parentNode, null);
			failures.add(new Failure(Validator.DENIED_HARDWARE, hardwareNode));
		}
		if (hardware != null)
		{
			hardwareNode = new Node("hardware", hardwareType + ": " + hardware.getName(), parentNode, hardware);
			
			// Check if using project default hardware
			if (project != null && projectDefault != null)
			{
				try
				{
					BasicItem defaultItem = project.getDefaultItem(dc, projectDefault);
					if (defaultItem != null && !hardware.equals(defaultItem))
					{
						failures.add(new Failure(Validator.NONDEFAULT_HARDWARE, parentNode, null,
							new Fix("Change " + hardwareType.toLowerCase() + " of " + itemTypeName, item),
							new Fix("Change default " + hardwareType.toLowerCase() + " of project", project)
						));
					}
				}
				catch (BaseException ex)
				{}
			}
			
			// Check hardware type
			try
			{
				HardwareType ht = hardware.getHardwareType();
				if (!ht.getSystemId().equals(projectDefault.getType()))
				{
					failures.add(new Failure(Validator.INCORRECT_HARDWARETYPE, parentNode, null,
						new Fix("Change " + hardwareType + " of " + itemTypeName, item),
						new Fix("Change type of " + hardwareType, hardware)
					));
				}
			}
			catch (PermissionDeniedException ex)
			{}

		}
		if (hardwareNode == null)
		{
			failures.add(new Failure(Validator.MISSING_HARDWARE, parentNode, null,
				new Fix("Add " + hardwareType.toLowerCase() + " to " + item.getType().toString().toLowerCase(), item)));
		}
		return hardwareNode;
	}
	
	/**
		Add a protocol child node to items with a protocol.
		Creates an item-type node with the name <code>protocol</code>.
	*/
	private Node addProtocol(DbControl dc, Node parentNode, Project.Default defaultProtocol)
	{
		BasicItem item = parentNode.getItem();
		Protocol protocol = null;
		Node protocolNode = null;
		try
		{
			if (item instanceof Annotatable)
			{
				protocol = ((Annotatable)item).getProtocol();
			}
		}
		catch (PermissionDeniedException ex)
		{
			protocolNode = new Node("protocol", "Protocol: denied", parentNode, null);
			failures.add(new Failure(Validator.DENIED_PROTOCOL, protocolNode));
		}
		if (protocol != null)
		{
			protocolNode = new Node("protocol", "Protocol: " + protocol.getName(), parentNode, protocol);
			addProtocolParameters(dc, protocolNode);
			
			// Check if using project default protocol
			if (project != null)
			{
				try
				{
					BasicItem defaultItem = project.getDefaultItem(dc, defaultProtocol);
					if (defaultItem != null && !protocol.equals(defaultItem))
					{
						failures.add(new Failure(Validator.NONDEFAULT_PROTOCOL, parentNode, null,
							new Fix("Change protocol of " + item.getType().toString().toLowerCase(), item),
							new Fix("Change default " + defaultProtocol.getShortName().toLowerCase() + " for project", project)
						));
					}
				}
				catch (BaseException ex)
				{}
			}
			
			// Check protocol type
			try
			{
				ProtocolType pt = protocol.getProtocolType();
				String requiredId = defaultProtocol == null ? null : defaultProtocol.getType();
				if (requiredId != null && !requiredId.equals(pt.getSystemId()))
				{
					failures.add(new Failure(Validator.INCORRECT_PROTOCOLTYPE, parentNode, null,
						new Fix("Change protocol of " + item.getType().toString().toLowerCase(), item),
						new Fix("Change type of protocol", protocol)
					));
				}
			}
			catch (PermissionDeniedException ex)
			{}
		}
		if (protocolNode == null)
		{
			failures.add(new Failure(Validator.MISSING_PROTOCOL, parentNode, null,
				new Fix("Add protocol to " + item.getType().toString().toLowerCase(), item)));
		}
		return protocolNode;
	}
	
	/**
		Add protocol parameters child node to a protocol. Creates a 
		folder-type child node with the name <code>parameters</code> and inside
		it item-type nodes with the name <code>parameter.ID</code> for each
		protocol parameter. ID is the ID of the annotation type.
		@see Protocol#getParameters()
	*/
	private Node addProtocolParameters(DbControl dc, Node protocolNode)
	{
		Protocol protocol = (Protocol)protocolNode.getItem();
		Node parentNode = protocolNode.getParent();
		BasicItem parentItem = parentNode.getItem();
		Annotatable annotatable = parentItem instanceof Annotatable ? (Annotatable)parentItem : null;
		AnnotationSet as = annotatable != null && annotatable.isAnnotated() ? 
				annotatable.getAnnotationSet() : null;
		ItemQuery<AnnotationType> parameterQuery = initQuery(protocol.getParameters(), null, "name");
		List<AnnotationType> parameters = parameterQuery.list(dc);
		Node parametersNode = null;
		if (parameters.size() > 0)
		{
			parametersNode = new Node("parameters", "Parameters", protocolNode);
			for (AnnotationType pp : parameters)
			{
				Node parameterNode = new Node("parameter."+pp.getId(), pp.getName(), parametersNode, pp);
				
				// Check if a value for this parameter has been specified
				if (as == null || !as.hasAnnotation(pp))
				{
					failures.add(new Failure(Validator.MISSING_PARAMETER, parentNode, 
						"Missing value for parameter: " + pp.getName(), 
						new Fix("Add value for: " + pp.getName(), parentItem, pp, false)));
				}
				else
				{
					Annotation a = as.getAnnotation(pp);
					try
					{
						List<?> values = a.getValues();
						if (values != null)
						{
							for (Object value : values)
							{
								pp.validateAnnotationValue(value);
							}
						}
					}
					catch (InvalidDataException ex)
					{
						failures.add(new Failure(Validator.ANNOTATION_INVALID_VALUE, parentNode, ex.getMessage(),
							new Fix("Change parameter value", parentItem, pp, false),
							new Fix("Change parameter restrictions", pp)
						));
					}
					
				}
			}
		}
		return parametersNode;
	}
	
	/**
		Add annotations child node to an annotatable item. Creates a 
		folder-type child node with the name <code>annotations</code> and inside
		it item-type nodes with the name <code>annotationtype.ID</code> for each
		annotation (primary and inherited). ID is the ID of the annotation type.
		If the logged in user doesn't have read permission for a particular annotation
		type a node with the name <code>annotation.ID</code> is created instead. In this
		case the ID is the ID of the annotation.
		@see Protocol#getParameters()
	*/
	private Node addAnnotations(DbControl dc, Node parentNode)
	{
		Annotatable parent = (Annotatable)parentNode.getItem();
		Node annotationsNode = null;

		Map<AnnotationType, Node> annotationTypes = new HashMap<AnnotationType, Node>();
		Set<AnnotationType> duplicates = new HashSet<AnnotationType>();
		
		if (parent.isAnnotated())
		{
			Protocol protocol = null;
			try
			{
				protocol = parent.getProtocol();
			}
			catch (PermissionDeniedException ex)
			{}
			annotationsNode = new Node("annotations", "Annotations", parentNode);
			AnnotationSet as = parent.getAnnotationSet();
			ItemQuery<Annotation> annotationQuery = initQuery(as.getAnnotations(), null, "annotationType.name");
		
			for (Annotation a : annotationQuery.list(dc))
			{
				try
				{
					AnnotationType at = a.getAnnotationType();
					// Protocol parameters are handled elsewhere
					if (protocol == null || !protocol.isParameter(at))
					{
						Node atNode = new Node("annotationtype."+at.getId(), at.getName(), annotationsNode, at);
						annotationTypes.put(at, atNode);
						
						// Check if annotation is a protocol parameter
						if (at.isProtocolParameter())
						{
							failures.add(new Failure(Validator.ANNOTATION_IS_PARAMETER, parentNode, 
								"Annotation '" + at.getName() + "' is a protocol parameter",
								new Fix("Remove annotation value", (BasicItem)parent, at, false),
								new Fix("Change 'parameter' flag of annotation type", at)
							));
						}
						
						// Check if annotation can be used on this item type
						if (!at.isEnabledForItem(parent.getType()))
						{
							failures.add(new Failure(Validator.ANNOTATION_INVALID_ITEM, parentNode, 
								"Annotation '" + at.getName() + "' is invalid for item type: " + parent.getType(), 
								new Fix("Remove annotation value", (BasicItem)parent, at, false),
								new Fix("Add '" + parent.getType().name() + "' to annotation type", at)
							));
						}
						try
						{
							List<?> values = a.getValues();
							if (values != null)
							{
								for (Object value : values)
								{
									at.validateAnnotationValue(value);
								}
							}
						}
						catch (InvalidDataException ex)
						{
							failures.add(new Failure(Validator.ANNOTATION_INVALID_VALUE, parentNode, ex.getMessage(),
								new Fix("Change annotation value", (BasicItem)parent, at, false),
								new Fix("Change annotation type restrictions", at)
							));
						}
					}
				}
				catch (PermissionDeniedException ex)
				{
					Node annotationNode = new Node("annotation."+a.getId(), a.getValueType().toString(), annotationsNode, a);
					failures.add(new Failure(Validator.DENIED_ANNOTATIONTYPE, annotationNode));
				}
			}
		}
		
		// Check if all MIAME annotations are specified
		Set<AnnotationType> miameAnnotations = getMiameAnnotationTypes(parent.getType());
		if (miameAnnotations != null)
		{
			for (AnnotationType at : miameAnnotations)
			{
				if (!annotationTypes.containsKey(at))
				{
					failures.add(new Failure(Validator.MISSING_MIAME_ANNOTATION, parentNode, null, 
							new Fix("Add value for annotation '" + at.getName() + "'", (BasicItem)parent, at, false),
							new Fix("Remove 'Required for MIAME' flag on '" + at.getName() + "'", at)
						));
				}
			}
		}
		
		if (parent.isAnnotated())
		{
			//annotationTypes.clear();
			AnnotationSet as = parent.getAnnotationSet();
			ItemQuery<Annotation> annotationQuery = initQuery(as.getAllInheritedAnnotations(), "at", "name");
			annotationQuery.join(Hql.innerJoin(null, "annotationType", "at", true));
			List<Annotation> inherited = annotationQuery.list(dc);
			if (inherited.size() > 0)
			{
				Set<Annotatable> parents = getAnnotatableParents(parent);

				for (Annotation a : inherited)
				{
					Node atNode = null;
					AnnotationType at = null;
					try
					{
						at = a.getAnnotationType();
						if (annotationTypes.containsKey(at))
						{
							duplicates.add(at);
							atNode = annotationTypes.get(at);
						}
						else
						{
							annotationTypes.put(at, atNode);
							atNode = new Node("annotationtype."+at.getId(), at.getName(), annotationsNode, at);
						}
					}
					catch (PermissionDeniedException ex)
					{
						atNode = new Node("annotation."+a.getId(), a.getValueType().toString(), annotationsNode, a);
						failures.add(new Failure(Validator.DENIED_ANNOTATIONTYPE, atNode));
					}
					try
					{
						Annotatable inheritFrom = a.getAnnotationSet().getItem();
						if (!parents.contains(inheritFrom))
						{
							failures.add(new Failure(Validator.ANNOTATION_INHERIT_FROM_NONPARENT, parentNode, 
								"Inheriting annotation '" + (at != null ? 
									at.getName() : a.getValueType().toString()) + "' from non-parent",
								new Fix("Inherit from another item", parentNode.getItem(), at, true)
							));
						}
					}
					catch (PermissionDeniedException ex)
					{}
				}
			}
			
			// Add warnings for duplicate annotation types
			if (duplicates.size() > 0)
			{
				for (AnnotationType dup : duplicates)
				{
					failures.add(new Failure(Validator.ANNOTATION_INHERIT_MULTIPLE, parentNode, 
						"Inherits multiple annotations of type '" + dup.getName() + "'",
						new Fix("Modify inherited annotations", parentNode.getItem(), dup, true)
					));
				}
			}
			
		}

		return annotationsNode;
	}

	/**
		Add data files child node to a FileStoreEnabled item. Creates a 
		folder-type child node with the name <code>fileset</code> and inside
		it item-type nodes with the name <code>member.ID</code> for each
		data file. ID is the ID of the file set member.
	*/
	private Node addDataFiles(DbControl dc, Node parentNode)
	{
		FileStoreEnabled parent = (FileStoreEnabled)parentNode.getItem();
		Node fileSetNode = null;
	
		Map<DataFileType, Node> fileTypes = new HashMap<DataFileType, Node>();
		
		if (parent.hasFileSet())
		{
			fileSetNode = new Node("fileset", "Data files", parentNode);
			FileSet fs = parent.getFileSet();
			ItemQuery<FileSetMember> memberQuery = initQuery(fs.getMembers(), null, "dataFileType.name");
		
			for (FileSetMember m : memberQuery.list(dc))
			{
				DataFileType dft = null;
				File file = null;
				boolean deniedFile = false;
				boolean deniedDataFileType = false;
				try
				{
					dft = m.getDataFileType();
				}
				catch (PermissionDeniedException ex)
				{
					deniedDataFileType = true;
				}
				try
				{
					file = m.getFile();
				}
				catch (PermissionDeniedException ex)
				{
					deniedFile = true;
				}
				String nodeName = dft == null ? "Data file: " : dft.getName() + ": ";
				nodeName += file == null ? "denied" : file.getName();
				Node fileNode = new Node("member."+m.getId(), nodeName, fileSetNode, m);

				// Do we have access to the file
				if (deniedDataFileType)
				{
					failures.add(new Failure(Validator.DENIED_DATAFILETYPE, fileNode));
				}
				if (deniedFile)
				{
					failures.add(new Failure(Validator.DENIED_FILE, fileNode));
				}
				
				// Check if the file is valid
				if (Boolean.FALSE.equals(m.isValid()))
				{
					String errorMsg = dft == null ? "Data file " : dft.getName();
					errorMsg += " is not valid: " + m.getErrorMessage();
					failures.add(new Failure(Validator.DATAFILE_INVALID, fileNode, errorMsg,
						new Fix("Select another file", (BasicItem)parent, m)
					));
				}
				
				if (dft != null)
				{
					fileTypes.put(dft, fileNode);
					
					// If the data file type has a validator, has the file been validated?
					if (dft.getValidatorClass() != null && m.isValid() == null)
					{
						failures.add(new Failure(Validator.DATAFILE_NOTVALIDATED, fileNode, 
							dft.getName() + " has not been validated", 
							new Fix("Validate the file / Select another file", (BasicItem)parent, m)));
					}
					
					// Check that the data file's item type is correct
					if (dft.getItemType() != parent.getType())
					{
						failures.add(new Failure(Validator.DATAFILE_INVALID_ITEM, fileNode, 
							"Data file '" + dft.getName() + "' can't be used on item type: " + parent.getType(), 
							new Fix("Remove file", (BasicItem)parent, m),
							new Fix("Change type of '" + dft.getName() + "'", dft)
						));
					}
				}
			}
		}
		
		// Check if all Platform required files are specified
		try
		{
			Platform platform = parent.getPlatform();
			PlatformVariant variant = parent.getVariant();
			if (platform != null)
			{
				// The item to apply fixes on, variant or platform
				String pvName = null;
				BasicItem pvItem = null;
				if (variant != null)
				{
					pvItem = variant;
					pvName = variant.getName();
				}
				else
				{
					pvItem = platform;
					pvName = platform.getName();
				}

				ItemQuery<PlatformFileType> pftQuery = platform.getFileTypes(variant, variant == null);
				pftQuery.restrict(
					Restrictions.eq(
						Hql.property("dataFileType.itemType"), 
						Expressions.integer(parent.getType().getValue())
					)
				);
				pftQuery.setCacheResult(true);
				List<PlatformFileType> platformFiles = pftQuery.list(dc);
				if (platformFiles != null)
				{
					for (PlatformFileType pft : platformFiles)
					{
						DataFileType dft = pft.getDataFileType();
						if (pft.isRequired() && !fileTypes.containsKey(dft))
						{
							failures.add(new Failure(Validator.MISSING_DATAFILE, parentNode, 
									"Missing '" + dft.getName() + "'", 
									new Fix("Add '" + dft.getName() + "' to " + parent.getType().toString().toLowerCase(), 
										(BasicItem)parent, dft),
									new Fix("Remove 'Required' flag from '" + dft.getName() + "'", pvItem, dft)
								));
						}
						fileTypes.remove(dft);
					}
				}

				// The remaining file types are not specified by the platform / variant
				for (Map.Entry<DataFileType, Node> entry : fileTypes.entrySet())
				{
					Node node = entry.getValue();
					DataFileType dft = entry.getKey();
					failures.add(new Failure(Validator.DATAFILE_NOTPLATFORM, node, 
						dft.getName() + " is not used by '" + pvName + "' platform", 
						new Fix("Remove '" + dft.getName() + "' from " + parent.getType().toString().toLowerCase(), 
							(BasicItem)parent, dft),
						new Fix("Add '" + dft.getName() +"' to '" + pvName + "' platform", pvItem, dft)
					));
				}
			}
		}
		catch (PermissionDeniedException ex)
		{}
		return fileSetNode;
	}
	
	
	/**
		Set include options and sorting order for a query.
	*/
	private <T extends BasicItem> ItemQuery<T> initQuery(ItemQuery<T> query, String sortAlias, String sortProperty)
	{
		query.include(Include.MINE, Include.IN_PROJECT, Include.SHARED, Include.OTHERS);
		query.order(Orders.asc(Hql.property(sortAlias, sortProperty)));
		return query;
	}
}
