/**
	$Id: Failure.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import net.sf.basedb.core.BasicItem;

/**
 	Holds information about a failed validation. Note that all 
 	rules are always validated. Use a {@link ValidationOptions}
 	object to keep track of which validators that should be
 	ignored, generate warnings or errors.
 	<p>
 	NOTE! The implementation of the {@link #equals(Object)} method of
 	this object is a bit unorthodox. Two failures are considered equal
 	if they have the same validator but different nodes referencing the
 	same item with the same descriptor. Two failures on the same node 
 	are never equal. This allows us to filter out duplicate failures that 
 	happens because there can be several ways to reach a certain item (for 
 	example a reference sample that is used on all hybridizations). 	
 	
	@author Nicklas
	@version 2.2
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
	@see Validator
	@see ValidationOptions
	@see ExperimentOverview#getFailures()
	@see ExperimentOverview#getValidationOptions()
*/
public class Failure
{
	
	private Validator validator;
	private Node node;
	private String message;
	private List<Fix> fixes;
	private Object descriptor;
	
	/**
		Create a new validation failure.
		@param validator The validation rule that fails
		@param node The node that contains the failure
	*/
	public Failure(Validator validator, Node node)
	{
		this(validator, node, null, (Fix[])null);
	}
	
	/**
		Create a new validation failure with a detailed message. 
		@param validator The validation rule that fails
		@param node The node that contains the failure
		@param message A detailed message about the failue, if null
			the {@link Validator#getFailureSummary()} is used
	 */
	public Failure(Validator validator, Node node, String message)
	{
		this(validator, node, message, (Fix[])null);
	}
	
	/**
		Create a new validation failure with a detailed message and a list
		of suggested fixes.
		
		@param validator The validation rule that fails
		@param node The node that contains the failure
		@param message A detailed message about the failue, if null
			the {@link Validator#getFailureSummary()} is used
		@param fixes A list of suggested fixes
	*/
	public Failure(Validator validator, Node node, String message, Fix... fixes)
	{
		this.validator = validator;
		this.node = node;
		this.message = message;
		this.fixes = new LinkedList<Fix>();
		if (fixes != null) this.fixes.addAll(Arrays.asList(fixes));
	}
	
	/**
		Set a descriptor on this failure. This affects the {@link #equals(Object)}
		method, since the descriptor must be equal for two failures to be
		equal. 
		<p>
		NOTE! The main reason for this property is that we need to discriminate
		between failures on different sub-arrays on hybridizations.
		@param descriptor Any object
		@since 2.6
	*/
	public void setDescriptor(Object descriptor)
	{
		this.descriptor = descriptor;
	}
	/**
		@return The descriptor
		@since 2.6
		@see #setDescriptor(Object)
	*/
	public Object getDescriptor()
	{
		return descriptor;
	}
	
	/**
		Get the validation rule that failed.
	*/
	public Validator getValidator()
	{
		return validator;
	}
	
	/**
		Get the node that failed validation.
	*/
	public Node getNode()
	{
		return node;
	}
	
	/**
		Get a message explaining the failure.
	*/
	public String getMessage()
	{
		return message != null ? message : validator.getFailureSummary();
	}
	
	/**
		Get a list of suggested fixes.
		@return A list of suggested fixes, or null if there are none
	*/
	public List<Fix> getFixes()
	{
		return fixes;
	}
	
	/**
		Add a suggested fix to this failure.
	 	@param fix The fix to add.
	*/
	public void addFix(Fix fix)
	{
		fixes.add(fix);
	}
	
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return getMessage();
	}
	
	/**
		One failure is the same as another if they have the same validator
		but different nodes referencing the same item with equal descriptors.
	*/
	@Override
	public boolean equals(Object o)
	{
		if (o == null || o.getClass() != this.getClass()) return false;
		Failure other = (Failure)o;
		BasicItem thisItem = this.node.getItem();
		boolean sameNode = this.node == other.node;
		boolean sameItem = sameNode || this.node.getItem() ==  other.node.getItem();
		boolean sameValidator = this.validator == other.validator;
		boolean sameSpecificator = sameNode || this.descriptor == other.descriptor ||
			(this.descriptor != null && this.descriptor.equals(other.descriptor));
		return !sameNode && sameItem && sameValidator && sameSpecificator;
	}

	@Override
	public int hashCode()
	{
		return validator.hashCode();
	}
	// -------------------------------------------
}
