/**
	$Id: ScanImporter.java 4551 2008-09-29 07:47:24Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.Scan;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

/**
	Plug-in for importing scan items in a batch. The plug-in can create new 
	items and updated existing items.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-29 09:47:24 +0200 (Mon, 29 Sep 2008) $
*/
public class ScanImporter
	extends AbstractItemImporter<Scan>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.SCAN, GuiContext.Type.LIST));

	private static final About about =
		new AboutImpl
	(
			"Scan importer",
			"Imports and updates scans in a batch.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2008, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
	);

	
	protected static final PluginParameter<String> hardwareColumnMapping = new PluginParameter<String>(
			"hardwareColumnMapping",
			"Scanner",
			"Mapping that picks the name or ID of the array slide scanner from the data columns. " +
			"The plug-in will first try to find a scanner with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Scanner\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> hybridizationColumnMapping = new PluginParameter<String>(
			"hybridizationColumnMapping",
			"Hybridization",
			"Mapping that picks the name or ID of the hybridization from the data columns. " +
			"The plug-in will first try to find a hybridization with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Hybridization\\",
			optionalColumnMapping
			);
	
	public ScanImporter()
	{}

	/*
		From the Plugin interface
		------------------------------------
	*/
	@Override
	public About getAbout()
	{
		return about;
	}
	// ------------------------------------

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper protocolMapper;
	private Mapper hardwareMapper;
	private Mapper hybridizationMapper;
	
	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME,  
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	/**
		Adds column mappings for name, externalId, description,
		protocol, creation date, original quantity, pooled, parent
		and used quantity from parent.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(hybridizationColumnMapping);
		parameters.add(protocolColumnMapping);
		parameters.add(hardwareColumnMapping);
	}
	
	/**
		Calls {@link Scan#getQuery()}.
	*/
	@Override
	protected ItemQuery<Scan> createItemQuery()
	{
		return Scan.getQuery();
	}
	
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? Scan.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? Scan.MAX_DESCRIPTION_LENGTH : null, null);
		protocolMapper = getMapper(ffp, (String)job.getValue("protocolColumnMapping"), null, null);
		hardwareMapper = getMapper(ffp, (String)job.getValue("hardwareColumnMapping"), null, null);
		hybridizationMapper = getMapper(ffp, (String)job.getValue("hybridizationColumnMapping"), null, null);
	}
	
	@Override
	protected Scan createItem(DbControl dc, FlatFileParser.Data data)
	{
		Hybridization hyb = null;
		if (hybridizationMapper != null) 
		{
			hyb = findHybridization(dc, FallbackIdMethod.NAME_OR_ID, hybridizationMapper.getValue(data));
		}

		Scan scan = Scan.getNew(dc, hyb);
		updateItem(dc, scan, data);
		return scan;
	}
	
	@Override
	protected void updateItem(DbControl dc, Scan scan, FlatFileParser.Data data)
	{
		if (nameMapper != null) scan.setName(nameMapper.getValue(data));
		if (descriptionMapper != null) scan.setDescription(descriptionMapper.getValue(data));
		if (hybridizationMapper != null && scan.isInDatabase()) 
		{
			String nameOrId = hybridizationMapper.getValue(data);
			if (nameOrId != null)
			{
				Hybridization hyb = findHybridization(dc, FallbackIdMethod.NAME_OR_ID, hybridizationMapper.getValue(data));
				scan.setHybridization(hyb);
			}
		}
		if (protocolMapper != null) 
		{
			String nameOrId = protocolMapper.getValue(data);
			ProtocolType type = ProtocolType.getById(dc, SystemItems.getId(ProtocolType.SCANNING));
			Protocol protocol = findProtocol(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, type);
			if (nameOrId == null || protocol != null) scan.setProtocol(protocol);
		}
		if (hardwareMapper != null)
		{
			String nameOrId = hardwareMapper.getValue(data);
			Hardware scanner = findScanner(dc, FallbackIdMethod.NAME_OR_ID, nameOrId);
			if (nameOrId == null || scanner != null) scan.setScanner(scanner);
		}
	}
	// ------------------------------------
}
