import java.util.Date;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SystemItems;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

public class TestDirectory
{
	private static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing directory");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(true, "Test directory1");
		int id2 = test_create(false, "Test directory2");
		test_load(id1);
		test_list();

		// Extra tests: create, list, load and delete subdirectory
		int id_sub = test_create(id1);
		int id_sub2 = test_create(id_sub);
		if (id_sub != 0) test_load("/Test directory1/Test subdirectory");
		test_list(id1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id_sub2, false);

		// Standard test list, delete
		test_list();
		test_delete(id_sub, false);
		test_delete(id1, false);
		test_delete(id2, false);
		write("++Testing directory "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean testAll, String directoryName)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.DIRECTORY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory root = Directory.getById(dc, SystemItems.getId(Directory.ROOT));
			Directory d = Directory.getNew(dc, root);
			if (testAll)
			{
				d.setName(directoryName);
				d.setDescription("Added at "+new Date());
				d.setAutoCompress(true);
			}
			dc.saveItem(d);
			dc.commit();
			id = d.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(d);
			write_item(0, d);
			write("--Create directory OK");
		}
		catch (Throwable ex)
		{
			write("--Create directory FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create(int parentId)
	{
		if (parentId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.DIRECTORY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory d = Directory.getById(dc, parentId);
			Directory sub = d.newSubDirectory();
			sub.setName("Test subdirectory");
			sub.setDescription("Added at "+new Date());
			dc.saveItem(sub);
			dc.commit();
			id = sub.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(sub);
			write_item(0, sub);
			write("--Create subdirectory OK");
		}
		catch (Throwable ex)
		{
			write("--Create subdirectory FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory d = Directory.getById(dc, id);
			write_item(0, d);
			write("--Load directory OK");
		}
		catch (Throwable ex)
		{
			write("--Load directory FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_load(String path)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory d = Directory.getByPath(dc, new Path(path, Path.Type.DIRECTORY));
			write_item(0, d);
			write("--Load directory by path OK");
		}
		catch (Throwable ex)
		{
			write("--Load directory by path FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}

	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Directory> l = Directory.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List directories OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List directories FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list(int parentId)
	{
		if (parentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory d = Directory.getById(dc, parentId);
			ItemQuery<Directory> q = d.getSubDirectories();
			
			int i = 0;
			for (Directory sub : q.list(dc))
			{
				write_item(i++, sub);
			}
			write("--List subdirectories OK ("+i+")");
		}
		catch (Throwable ex)
		{
			write("--List subdirectories FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id, boolean deleteAll)
	{
		if (id == 0) return;
		DbControl dc = null;
		Directory d = null;
		try
		{
			dc = TestUtil.getDbControl();
			d = Directory.getById(dc, id);
			if (deleteAll)
			{
				test_deleteAll(id, dc);
			}
			else
			{
				dc.deleteItem(d);				
			}
			
			dc.commit();
			write("--Delete directory "+d+" OK");
		}
		catch (Throwable ex)
		{
			write("--Delete directory "+d+" FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private static void test_deleteAll(int id, DbControl dc)
	{
		Directory d = Directory.getById(dc, id);
			
		ItemQuery<Directory> directories = d.getSubDirectories();
		for (Directory subd : directories.list(dc))
		{
			test_deleteAll(subd.getId(), dc);
		}
		
		ItemQuery<File> files = d.getFiles();
		for (File f : files.list(dc))
		{
			dc.deleteItem(f);
			write("--Delete file "+f+" OK");
		}
		
		dc.deleteItem(d);
		write("--Delete directory "+d+" OK");
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tAuto compress\tParent\tPath");
			write("-- \t-- \t--------- \t-----------\t-------------\t------\t----");
		}
	}
	static void write_item(int i, Directory d)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+d.getId()+"\t"+d.getName()+"\t"+
			d.getDescription()+"\t"+d.getAutoCompress()+"\t"+d.getParent()+"\t"+d.getPath());
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
