/*
	$Id$
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.AbsoluteProgressReporter;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.Application;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SimpleAbsoluteProgressReporter;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.util.ConsoleProgressReporter;
import net.sf.basedb.util.InputStreamSplitter;
import net.sf.basedb.util.zip.ZipUnpacker;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.zip.ZipInputStream;

public class TestFile
{
	private static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing file");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create("data/test.upload.txt", true, true);
		int id2 = test_create("data/test.upload.txt", false, false);
		test_load(id1);
		test_list();
		
		// Extra test: update
		test_update("data/test.upload.png", id2);

		// Extra test: unpack zip file
		int dirId = TestDirectory.test_create(true, "unpackzip");
		test_unpack_zip("data/test.unpack.zip", dirId);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestDirectory.test_delete(dirId, true);
		
		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		write("++Testing file "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_create(String filename, boolean compress, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.FILE)) return 0;
		int id = 0;
		DbControl dc = null;
		InputStream in = null;
		try
		{
			dc = TestUtil.getDbControl();
			File file = File.getNew(dc, Directory.getById(dc, SystemItems.getId(Directory.ROOT)));
			file.setName(filename.replace("/", "_")+Application.generateRandomId(2));
			if (setAll)
			{
				file.setDescription("Added at "+new Date());
				file.setMimeTypeAuto("text/plain", null);
				file.setCharacterSet(Charset.defaultCharset().name());
			}
			dc.saveItem(file);
			java.io.File fileToUpload = new java.io.File(filename);
			if (fileToUpload.exists())
			{
				in = new FileInputStream(fileToUpload);
				file.upload(in, true, compress);
				if (file.getSize() != fileToUpload.length())
				{
					throw new BaseException("Upload incomplete: original size=" + fileToUpload.length()+
							"; uploaded="+file.getSize());
				}
			}
			dc.commit();
			id = file.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(file);
			write_item(0, file);
			write("--Create file OK");
		}
		catch (Throwable ex)
		{
			write("--Create file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
			if (in != null)
			{
				try
				{
					in.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			File fi = File.getById(dc, id);
			java.io.File downloadFile = new java.io.File("download.txt");
			downloadFile.deleteOnExit();
			OutputStream out = new java.io.FileOutputStream(downloadFile);
			fi.download(out, 0);
			out.close();
			if(downloadFile.length() != fi.getSize())
			{
				throw new Exception("File download incomplete: size=" + fi.getSize() + 
						"; downloaded="+downloadFile.length());
			}
			write_item(0, fi);
			write("--Load file OK");
		}
		catch (Throwable ex)
		{
			write("--Load file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<File> l = File.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List files OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List files FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		File fi = null;
		try
		{
			dc = TestUtil.getDbControl();
			fi = File.getById(dc, id);
			dc.deleteItem(fi);
			dc.commit();
			write("--Delete file "+fi+" OK");
		}
		catch (Throwable ex)
		{
			write("--Delete file "+fi+"FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}

	}
	
	/**
	 	@since 2.5
	 */
	static void test_rename(int id, String newName)
	{
		if (id == 0) return;
		DbControl dc = null;
		File file = null;
		String oldName = null;
		try
		{
			dc = TestUtil.getDbControl();
			file = File.getById(dc, id);
			oldName = file.getName();
			file.setName(newName);
			dc.commit();
			write("--Rename fileId: " + file.getId() + " from " + oldName + " to " + newName + " OK");
		}
		catch (Throwable ex)
		{
			write("--Rename file: " + file + " FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tFiletype\tUsed\tLocation\tAction\tSize    \tCompressed size\tMime type\tCharacter set\tPath");
			write("-- \t-- \t--------- \t-----------\t--------\t----\t--------\t------\t--------\t---------------\t---------\t-------------\t----");
		}
	}
	static void write_item(int i, File fi)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+fi.getId()+"\t"+fi.getName()+"\t"+fi.getDescription()+"\t"+
			fi.getFileType()+"\t"+fi.isUsed()+"\t"+fi.getLocation()+"\t"+fi.getAction()+"\t"+
			fi.getSize()+"\t" + fi.getCompressedSize() + "\t"+fi.getMimeType()+"\t"+
			fi.getCharacterSet()+"\t"+fi.getPath());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_update(String filename, int fileid)
	{
		if (fileid == 0) return;
		DbControl dc = null;
		InputStream in = null;
		try
		{
			dc = TestUtil.getDbControl();
			File file = File.getById(dc, fileid);
			in = new FileInputStream(filename);
			file.setName(filename.replace("/", "_")+Application.generateRandomId(2));
			file.upload(in, false);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(file);
			write_item(0, file);
			write("--Update file OK");
		}
		catch (Throwable ex)
		{
			write("--Update file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
			if (in != null)
			{
				try
				{
					in.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
	}
	
	static void test_unpack_zip(String filename, int directoryId)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.FILE) || directoryId == 0) return;
		DbControl dc = null;
		ZipInputStream zipFile = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory dir = Directory.getById(dc, directoryId);
			
			File keepZip = File.getNew(dc, dir);
			keepZip.setName(filename.replace("/", "_")+Application.generateRandomId(2));
			dc.saveItem(keepZip);
			OutputStream keepZipStream = keepZip.getUploadStream(false);
			java.io.File fileToUpload = new java.io.File(filename);
			InputStream upload = new FileInputStream(fileToUpload);
			InputStreamSplitter splitter = new InputStreamSplitter(upload, true, true, keepZipStream);
			
			ZipUnpacker unpacker = new ZipUnpacker(dc);
			zipFile = new ZipInputStream(splitter);
			AbsoluteProgressReporter progress = TestUtil.getSilent() ? 
				null : new SimpleAbsoluteProgressReporter(new ConsoleProgressReporter(true), fileToUpload.length());
			unpacker.unpackZipFile(dir, zipFile, true, progress);
			if (progress != null) progress.append("\n");

			zipFile.close();
			keepZipStream.close();
			if (splitter.getNumRead() != fileToUpload.length())
			{
				throw new BaseException("Incomplete file upload; original size=" + fileToUpload.length()+
						"; uploaded size=" + splitter.getNumRead());
			}
			if (keepZip.getSize() != fileToUpload.length())
			{
				throw new BaseException("Incomplete file upload; original size=" + fileToUpload.length()+
						"; uploaded size=" + keepZip.getSize());
			}
			dc.commit();
			write("--Unpack zip file OK");
		}
		catch (Throwable ex)
		{
			write("--Unpack zip file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
			if (zipFile != null)
			{
				try
				{
					zipFile.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
	}
	
}
