/*
	$Id: TestProject.java 4646 2008-11-18 09:53:28Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.List;
import java.util.EnumSet;
import java.util.Set;

public class TestProject
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing projects");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(true);
		int id2 = test_create(false);
		test_load(id);
		test_list(-1);

		// Extra tests: add, list and remove users, use project
		int userId = TestUser.test_create(true);
		test_listusers(id, 0);
		test_adduser(id, userId, EnumSet.of(Permission.WRITE));
		test_listusers(id, 1);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_removeuser(id, userId);
		test_listusers(id, 0);
		TestUser.test_delete(userId);

		// Extra tests: add, list and remove groups
		int groupId = TestGroup.test_create(true);
		test_listgroups(id, 0);
		test_addgroup(id, groupId, EnumSet.of(Permission.WRITE));
		test_listgroups(id, 1);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_removegroup(id, groupId);
		test_listgroups(id, 0);
		TestGroup.test_delete(groupId);
	
		// Extra test: set default item.
		int samplingId = TestProtocol.test_create(
				SystemItems.getId(ProtocolType.SAMPLING), "Sampling protocol", false);
		int hybStationId = TestHardware.test_create(
				SystemItems.getId(HardwareType.HYBRIDIZATION_STATION), "Hyb station", false);
		test_defaultItem(id, Project.Default.SAMPLING_PROTOCOL, samplingId);
		test_defaultItem(id, Project.Default.HYBRIDIZATION_HARDWARE, hybStationId);

		// Extra test: activate project, create shared item, and list all items
		test_activate_project(id);
		int sampleId = TestSample.test_create(0, "Sample (shared)", false);
		int hybId = TestHybridization.test_create(0, false, new int[] {});
		test_list_items(id, 2);
		test_noactive_project();
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestSample.test_delete(sampleId);
		TestHybridization.test_delete(hybId);
		
		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		TestProtocol.test_delete(samplingId);
		TestHardware.test_delete(hybStationId);
		
		write("++Testing projects "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_defaultItem(int project_id, Project.Default defaultType, int item_id)
	{
		if (project_id == 0 || item_id == 0) return;
		DbControl dc = null;		
		try
		{			
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, project_id);
			BasicItem defaultItem = defaultType.getItemType().getById(dc, item_id);
			p.setDefaultItem(dc, defaultItem, defaultType);
			dc.commit();
			write("--Set default item OK (" + defaultType + "; id=" + item_id + ")");
		}
		catch(Throwable ex)
		{
			write("--Set default item FAILED (" + defaultType + "; id=" + item_id + ")");
			ex.printStackTrace();
			ok = false;
		}	
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PROJECT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getNew(dc);
			if (setAll)
			{
				p.setName("Test project");
				p.setDescription("Added at "+new Date());
			}
			dc.saveItem(p);
			dc.commit();
			id = p.getId();
			write_item(0, p);
			write("--Create project OK");
		}
		catch (Throwable ex)
		{
			write("--Create project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			write_item(0, p);
			write("--Load project OK");
		}
		catch (Throwable ex)
		{
			write("--Load project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Project> l = Project.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List projects OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List projects FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			dc.deleteItem(p);
			dc.commit();
			write("--Delete project OK");
		}
		catch (Throwable ex)
		{
			write("--Delete project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription");
			write("-- \t-- \t--------- \t-----------");
		}
	}
	static void write_item(int i, Nameable n)
		throws BaseException
	{
		write_item(i, n, "");
	}
	static void write_item(int i, Nameable n, String extra)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+n.getId()+"\t"+n.getName()+"\t"+n.getDescription()+"\t"+extra);
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void write_item(int i, Shareable item, String extra)
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.print(i+":\t"+item.getId()+"\t"+item.getType());
			if (item instanceof Nameable)
			{
				Nameable nameable = (Nameable)item;
				String description = nameable.getDescription();
				if (description != null && description.length() > 20) 
				{
					description = description.substring(0, 20) + "...";
				}
				System.out.println("\t" + nameable.getName() + "\t" + description+"\t"+extra);
			}
			else
			{
				System.out.println("\t" + item.toString()+"\t"+extra);
			}
		}
	}

	
	static void test_adduser(int id, int userId, EnumSet<Permission> permissions)
	{
		if (id == 0 || userId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			User u = User.getById(dc, userId);
			p.setPermissions(u, permissions);
			dc.commit();
			write("--Add user to project OK");
		}
		catch (Throwable ex)
		{
			write("--Add user to project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listusers(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			ItemResultList<User> l = p.getUsers().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				User u = l.get(i);
				write_item(i, u, ":"+p.getPermissions(u));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List users in project OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List users in project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_removeuser(int id, int userId)
	{
		if (id == 0 || userId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			User u = User.getById(dc, userId);
			p.setPermissions(u, EnumSet.noneOf(Permission.class));
			dc.commit();
			write("--Remove user from project OK");
		}
		catch (Throwable ex)
		{
			write("--Remove user from project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_addgroup(int id, int groupId, EnumSet<Permission> permissions)
	{
		if (id == 0 || groupId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			Group g = Group.getById(dc, groupId);
			p.setPermissions(g, permissions);
			dc.commit();
			write("--Add group to project OK");
		}
		catch (Throwable ex)
		{
			write("--Add group to project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listgroups(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			ItemResultList<Group> l = p.getGroups().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				Group g = l.get(i);
				write_item(i, g, ":"+p.getPermissions(g));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List groups in project OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List groups in project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_removegroup(int id, int groupId)
	{
		if (id == 0 || groupId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			Group g = Group.getById(dc, groupId);
			p.setPermissions(g, EnumSet.noneOf(Permission.class));
			dc.commit();
			write("--Remove group from project OK");
		}
		catch (Throwable ex)
		{
			write("--Remove group from project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_activate_project(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			TestUtil.getSessionControl().setActiveProject(p);
			write("--Activate project OK");
		}
		catch (Throwable ex)
		{
			write("--Activate project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_noactive_project()
	{
		try
		{
			TestUtil.getSessionControl().setActiveProject(null);
			write("--No active project OK");
		}
		catch (Throwable ex)
		{
			write("--No active project FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void test_list_items(int id, int expectedResults)
	{
		DbControl dc = null;
		try
		{
			TestUtil.getSessionControl().reloadPermissions();
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, id);
			List<Shareable> l = p.getItems((Set<Item>)null, 0, 0, null, null);
			for (int i = 0; i<l.size(); i++)
			{
				Shareable s = l.get(i);
				write_item(i, s, ":" + s.getProjectKey().getPermissions(p));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List items in project OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List items in project FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}

