/**
	$Id: RawBioAssayClient.java 4513 2008-09-11 20:06:36Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import net.sf.basedb.info.AnnotationInfo;
import net.sf.basedb.info.ArrayDesignInfo;
import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.QueryOptions;
import net.sf.basedb.info.RawBioAssayInfo;

import java.io.IOException;
import java.io.InputStream;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Webservice client for RawBioAssay.
 	This class covers all services provided in 
 	{@link net.sf.basedb.ws.server.RawBioAssayService}
  	@author Martin
  	@version 2.5
 	
 */
public class RawBioAssayClient 
	extends AbstractRPCClient
{

	private SessionClient session;
	
	/**
	 	Creates a new object for a session.
	  	@param session
	 */
	public RawBioAssayClient(SessionClient session)
	{
		super(session.getUrl(), "RawBioAssay", session.getServiceFactory());
		this.session = session;
	}
	
	/**
		Get info about a raw bioassay when you know the ID.
		@param itemId The ID of the raw bioassay
		@return An RawBioAssayInfo object
		@throws AxisFault If communication with web service fails.
	*/
	public RawBioAssayInfo getById(int itemId)
		throws AxisFault
	{
		return invokeBlocking("getById", RawBioAssayInfo.class, 
			session.getId(), itemId);
	}
	
	/**
	 	Gets info about raw bioassays that are available in the logged in session.
	  	@param qOpt Options to put on the query
	  	@return An array with info object for each raw bioassay that was found. 
	  	@throws AxisFault If communication with web service fails.
	 */
	public RawBioAssayInfo[] getRawBioAssays(QueryOptions qOpt)
		throws AxisFault
	{
		RawBioAssayInfo[] result = invokeBlocking("getRawBioAssays", RawBioAssayInfo[].class, session.getId(), qOpt);
		return result;
	}	
	
	/**
	 	Returns information about the array design used in a rawbioassay
	  	@param rawBioAssayId RawBioAssay to get the array design for.
	  	@return ArrayDesignInfo object.
	  	@throws AxisFault If communication with the webservice fails. 
	 */
	public ArrayDesignInfo getArrayDesign(int rawBioAssayId)
		throws AxisFault
	{
		return invokeBlocking
			(
				"getArrayDesign",
				ArrayDesignInfo.class,
				session.getId(),
				rawBioAssayId
			);
	}
	
	/**
	 	Gets RawData as a file for a rawbioassay if it is stored in a file
	 	in BASE.
	  	@param rawBioAssayId Id of rawbioassay to get raw data from
	  	@param dataFileType The external ID of the data file type to download
	  	@return An InputStream to read the file data from, or null
	  		if no file of the specified type exists
	  	@throws AxisFault If communication with webservices fails.
	  	@throws IOException If writing to the target file fails.
	 */
	public InputStream downloadRawDataByType(int rawBioAssayId, String dataFileType) 
		throws AxisFault, IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadRawDataByType", 
				session.getId(), 
				rawBioAssayId,
				dataFileType
			);
	}
	
	/**
	 	Returns if a rawbioassay has raw data that can be downloaded or not.
	  	@param rawBioAssayId Id of the RawBioAssay it concerns.
	  	@return TRUE if there are data to be downloaded, FALSE otherwise.
	  	@throws AxisFault If the communication with the webservice fails.
	 */
	public Boolean hasDownloadableData(int rawBioAssayId) 
		throws AxisFault
	{
		return invokeBlocking
			(
				"hasDownloadableData",
				Boolean.class,
				session.getId(),
				rawBioAssayId
			);
	}
	
	/**
	 	Returns information about data file types that are used storing
	 	raw data for a raw bioassay.
	  	@param rawBioAssayId Id of the raw bioassay to get the information about.
	  	@param qOpt Restrictions and includes to put on the query when getting the information.
	  	@return An array of DataFileTypeInfo, one for each data file type used in the raw bioassay.
	  	@throws AxisFault If communication with webservice fails.
	 */
	public DataFileTypeInfo[] getDataFileTypes(int rawBioAssayId, QueryOptions qOpt)
		throws AxisFault
	{
		return invokeBlocking
			(
				"getDataFileTypes",
				DataFileTypeInfo[].class,
				session.getId(),
				rawBioAssayId,
				qOpt
			);
	}
	
	/**
		Get annotation values for one or more annotation types.
		@param rawBioAssayId Id of the raw bioassay to load the annotations from
		@param annotationTypes Array with the ID:s of the annotation types
		@param findInherited TRUE to look among inherited annotations, FALSE to only
			look among primary annotations
		@return An array with annotation information, each element corresponds to the
			annotation type at the same position in the annotationTypes array
		@throws AxisFault
	*/
	public AnnotationInfo[] getAnnotationValues(int rawBioAssayId, Integer[] annotationTypes, boolean findInherited)
		throws AxisFault
	{
		return invokeBlocking("getAnnotations", AnnotationInfo[].class, 
			session.getId(), rawBioAssayId, annotationTypes, findInherited);
	}
}
